{
  - Master Cluster's Library -

  Copyright  1996-2006 Leonid Belousov (aka Master Cluster)
  www.MasterCluster.com
  
  - MCPGPDemo: example on using MCPGP component and routines -
}
unit uMCPGPDemo;

{$IFDEF VER150}
  {$WARN UNSAFE_TYPE OFF}
  {$WARN UNSAFE_CODE OFF}
  {$WARN UNSAFE_CAST OFF}
{$ENDIF}

interface

uses
  Windows, Messages, SysUtils, StrUtils, Variants, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, ExtCtrls,
  MCPGP, XPMan;

type
  TForm1 = class(TForm)
    ofd: TOpenDialog;
    grpOptions: TGroupBox;
    grpMode: TRadioGroup;
    grpAction: TRadioGroup;
    grpKey1: TGroupBox;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    edUserID: TEdit;
    edPwd: TEdit;
    bnCreate: TButton;
    edFpt: TEdit;
    bnViewKey: TButton;
    grpEncDec: TGroupBox;
    labSrcText: TLabel;
    labDstText: TLabel;
    labFile1: TLabel;
    labFile2: TLabel;
    mSrcText: TMemo;
    mDstText: TMemo;
    bnExec: TButton;
    edFile1: TEdit;
    bnFile1: TButton;
    edFile2: TEdit;
    bnFile2: TButton;
    edKeySize: TEdit;
    Label4: TLabel;
    Label5: TLabel;
    edSubKeySize: TEdit;
    bnChgPwd: TButton;
    xpman: TXPManifest;
    procedure bnViewKeyClick(Sender: TObject);
    procedure bnCreateClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure bnExecClick(Sender: TObject);
    procedure bnFile1Click(Sender: TObject);
    procedure bnFile2Click(Sender: TObject);
    procedure grpModeClick(Sender: TObject);
    procedure bnChgPwdClick(Sender: TObject);
  private
    { Private declarations }
    sKey: String;
    procedure evIdle(Sender: TObject; var Done: Boolean);
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}


function ViewTextDlg(const sCaption, sText: String; bReadOnly: Boolean = True): Boolean;
var
  fm: TForm;
  me: TMemo;
begin
  fm := TForm.Create(Application);
  with fm do
  try
    Caption := sCaption;

    me := TMemo.Create(fm);
    me.Parent     := fm;
    me.Align      := alClient;
    me.Font.Name  := 'Courier';
    me.Lines.Text := sText;
    me.ReadOnly   := bReadOnly;
    me.ScrollBars := ssBoth;
    me.WordWrap   := False;

    result := (ShowModal = mrOK);
  finally
    Free;
  end;
end; // ViewTextDlg



procedure TForm1.FormCreate(Sender: TObject);
begin
  Application.OnIdle := evIdle;
  grpModeClick(Self);

  if not pgp_Available then
    MessageBox(Handle
      , PChar('File "PGP_SDK.DLL" not found, demo will not functioning.'#13
            + 'Installation of PGP Freeware 6.5.8 required.')
      , 'Warning', mb_IconExclamation + mb_OK);
end;

procedure TForm1.evIdle(Sender: TObject; var Done: Boolean);
begin
  pgp_CollectRandomData;
end;

procedure TForm1.bnCreateClick(Sender: TObject);
var
  info: TPGPKeyInfoRec;
begin
  Screen.Cursor := crHourglass;
  try
    if pgp_GenerateKey(Handle, edUserID.Text, edPwd.Text, sKey,
      StrToInt(edKeySize.Text), StrToInt(edSubKeySize.Text)) then
    begin
      pgp_KeyInfo(sKey, info);
      edFpt.Text := info.sFingerprint;
    end;
  finally
    Screen.Cursor := crDefault;
  end;
end;

procedure TForm1.bnViewKeyClick(Sender: TObject);
begin
  ViewTextDlg('ASCII-armored Key Pair', sKey);
end;


procedure TForm1.bnFile1Click(Sender: TObject);
begin
  ofd.FileName := edFile1.Text;
  if ofd.Execute then
    edFile1.Text := ofd.FileName;
end;

procedure TForm1.bnFile2Click(Sender: TObject);
begin
  ofd.FileName := edFile2.Text;
  if ofd.Execute then
    edFile2.Text := ofd.FileName;
end;


procedure TForm1.grpModeClick(Sender: TObject);
var
  i: Integer;
begin
  for i := 0 to grpMode.Items.Count-1 do
    if grpAction.ItemIndex < 3 then
      grpMode.Items[i] := StringReplace(grpMode.Items[i], ' and DetSig ', ' to ', [])
    else
      grpMode.Items[i] := StringReplace(grpMode.Items[i], ' to ', ' and DetSig ', []);


  labSrcText.Visible := (
    ((grpAction.ItemIndex in [0,2,3]) and (grpMode.ItemIndex in [0, 1]))
    or
    ((grpAction.ItemIndex in [1]) and (grpMode.ItemIndex in [0, 3]))
  );
  mSrcText.Visible   := labSrcText.Visible;

  labDstText.Visible :=  (
    ((grpAction.ItemIndex in [0,2,3]) and (grpMode.ItemIndex in [0, 3]))
    or
    ((grpAction.ItemIndex in [1]) and (grpMode.ItemIndex in [0, 1]))
  );
  mDstText.Visible   := labDstText.Visible;

  labFile1.Visible := grpMode.ItemIndex in [1, 2, 3];
  edFile1.Visible  := labFile1.Visible;
  bnFile1.Visible  := labFile1.Visible;

  labFile2.Visible := grpMode.ItemIndex = 2;
  edFile2.Visible  := labFile2.Visible;
  bnFile2.Visible  := labFile2.Visible;


  if labSrcText.Visible then
    if grpAction.ItemIndex < 2 then // Enc/Dec
      labSrcText.Caption := IfThen(odd(grpAction.ItemIndex), 'Destination', 'Source') + ' Data:'
    else
      labSrcText.Caption := 'Data to be ' + IfThen(odd(grpAction.ItemIndex), 'Verified', 'Signed') + ':';

  if labDstText.Visible then
    if grpAction.ItemIndex < 2 then // Enc/Dec
      labDstText.Caption := 'Encoded Data (UUE):'
    else                            // Sig/Ver
      labDstText.Caption := 'Detached Signature (UUE):';


  if labFile1.Visible then
    case grpAction.ItemIndex of
      0, 1: // Enc / Dec
        labFile1.Caption := IfThen(grpMode.ItemIndex in [2, 3], 'Source', 'Destination') + ' File:';
      2, 3: // detsig / ver
        if grpMode.ItemIndex < 2 then
          labFile1.Caption := 'Detached Signature:'
        else
          labFile1.Caption := 'File to be ' + IfThen(odd(grpAction.ItemIndex), 'Verified', 'Signed') + ':';
    end;

  if labFile2.Visible then
    case grpAction.ItemIndex of
      0: // Enc
        labFile2.Caption := 'Destination File:';
      1: // Dec
        labFile2.Caption := 'Encoded Source File:';
      2, 3: // detsig / ver
        labFile2.Caption := 'Detached Signature:';
    end;
end; // grpModeClick


procedure TForm1.bnExecClick(Sender: TObject);

  procedure ShowError;
  begin
    ShowMessage('Error: ' + grpAction.Items[grpAction.ItemIndex] + ' with ' + grpMode.Items[grpMode.ItemIndex]);
  end;

var
  //info: TPGPKeyInfoRec;
  Pwd, Dst: String;
  Signers: TSigners;
  kir: TPGPKeyInfoRec;
  sci: TPGPCFB;
begin
  Pwd := edPwd.Text;


  case grpMode.ItemIndex of
    0: // Text - Text
      case grpAction.ItemIndex of
        0: // encode data
          begin
          if pgp_Encode(sKey, mSrcText.Lines.Text, False, Dst, False, [], []) then
            mDstText.Lines.Text := Dst
          else
            ShowError;

          {begin
            sci := TPGPCFB.Create(Pwd, ca3des);
            try
              sci.Encode(mSrcText.Lines.Text, Dst);
              mDstText.Lines.Text := Dst;
            finally
              sci.Free;
            end;
          end;}

            {with TFileStream.Create('C:\Documents and Settings\Family\My Documents\aaa.asc', fmOpenRead) do
            try
              SetLength(Dst, Size);
              Read(Dst[1], Size);
            finally
              Free;
            end;}

            //with TFileStream.Create('C:\Documents and Settings\Family\My Documents\aaa.asc', fmOpenRead) do
            {with TMCPGP.Create(nil) do
            try
              //ImportKeyFromFile('C:\Documents and Settings\Family\My Documents\aaa.asc')

              //ImportKeyFromString(Dst);

              //SetLength(Dst, Size);
              //Read(Dst[1], Size);
              //Dst := ExportKeyToString('aaa <aaa>', True);
              //Dst := ExportKeyToString('MC <mc@tesis.ru>', True);
              //pgp_KeyInfo(Dst, kir);

              DeleteKey('aaa <aaa>');
            finally
              Free;
            end;}

          end;
        1: // decode data
          if pgp_Decode(sKey, Pwd, mDstText.Lines.Text, False, Dst, False, Signers) then
            mSrcText.Lines.Text := Dst
          else
            ShowError;
        2: // create Detached sig
          if pgp_CreateDetachedSignature(sKey, Pwd, mSrcText.Lines.Text, False, Dst, False) then
            mDstText.Lines.Text := Dst
          else
            ShowError;
        3: // verify Detached sig
          if pgp_VerifyDetachedSignature(sKey,
               mSrcText.Lines.Text, False, mDstText.Lines.Text, False, Signers)
          then
            ShowMessage('Signature verified OK')
          else
            ShowError;
      end;

    1: // Text - File
      begin
        Dst := edFile1.Text;

        case grpAction.ItemIndex of
          0: // encode data
            if pgp_Encode(sKey, mSrcText.Lines.Text, False, Dst, True, [], []) then
              edFile1.Text := Dst
            else
              ShowError;
          1: // decode data
            if pgp_Decode(sKey, Pwd, mDstText.Lines.Text, False, Dst, True, Signers) then
              edFile1.Text := Dst
            else
              ShowError;
          2: // create Detached sig
            if pgp_CreateDetachedSignature(sKey, Pwd, mSrcText.Lines.Text, False, Dst, True) then
              edFile1.Text := Dst
            else
              ShowError;
          3: // verify Detached sig
            if pgp_VerifyDetachedSignature(sKey, mSrcText.Lines.Text, False, Dst, True, Signers) then
              begin
                edFile1.Text := Dst;
                ShowMessage('Signature verified OK');
              end
            else
              ShowError;
        end;
      end;


    2: // File - File
      case grpAction.ItemIndex of
        0: // encode data
          begin
            Dst := edFile2.Text;
            if pgp_Encode(sKey, edFile1.Text, True, Dst, True, [], []) then
              edFile2.Text := Dst
            else
              ShowError;
          end;
        1: // decode data
          begin
            Dst := edFile1.Text;
            if pgp_Decode(sKey, Pwd, edFile2.Text, True, Dst, True, Signers) then
              edFile1.Text := Dst
            else
              ShowError;
          end;
        2: // create Detached sig
          begin
            Dst := edFile2.Text;
            if pgp_CreateDetachedSignature(sKey, Pwd, edFile1.Text, True, Dst, True) then
              edFile2.Text := Dst
            else
              ShowError;
          end;
        3: // verify Detached sig
          if pgp_VerifyDetachedSignature(sKey, edFile1.Text, True, edFile2.Text, True, Signers) then
            ShowMessage('Signature verified OK')
          else
            ShowError;
      end;

    3: // File - Text
      case grpAction.ItemIndex of
        0: // encode data
          if pgp_Encode(sKey, edFile1.Text, True, Dst, False, [], []) then
            mDstText.Lines.Text := Dst
          else
            ShowError;
        1: // decode data
          if pgp_Decode(sKey, Pwd, edFile1.Text, True, Dst, False, Signers) then
            mSrcText.Lines.Text := Dst
          else
            ShowError;
        2: // create Detached sig
          if pgp_CreateDetachedSignature(sKey, Pwd, edFile1.Text, True, Dst, False) then
            mDstText.Lines.Text := Dst
          else
            ShowError;
        3: // verify Detached sig
          if pgp_VerifyDetachedSignature(sKey, edFile1.Text, True, mDstText.Lines.Text, False, Signers) then
            ShowMessage('Signature verified OK')
          else
            ShowError;
      end;

  end;
end; // bnExecClick

procedure TForm1.bnChgPwdClick(Sender: TObject);
var
  {sOldPwd,} sNewPwd: String;
begin
  //sOldPwd := '';
  sNewPwd := '';

  //if not InputQuery('Change Password', 'Type OLD password', sOldPwd) then
  //  Exit;

  if not InputQuery('Change UserID', 'Type NEW UserID', sNewPwd) then
    Exit;

  if pgp_ReplaceAllUserIDsBySingleUserID(sKey, edPwd.Text, sNewPwd) then
    edUserID.Text := sNewPwd
  else
    ShowMessage('Error change UserID.');


  {if not InputQuery('Change Password', 'Type NEW password', sNewPwd) then
    Exit;

  if pgp_ChangePassphrase(sKey, edPwd.Text, sNewPwd) then
    edPwd.Text := sNewPwd
  else
    ShowMessage('Error change password.');}
end;

end.
