{*******************************************************************}
{                                                                   }
{      Description:                                                 }
{      PGP_SDK.pas - PGP SDK API (1.7.x and 2.x) interface unit     }
{                                                                   }
{      Contributors:                                                }
{      The accessing open arrays technique is based on the original }
{      work of Grahame Grieve (Kestral Computing) PGPAPI.PAS file   }
{      http://www.kestral.com.au/devtools/pgp                       }
{                                                                   }
{      Portions created by Leonid Belousov are                      }
{      Copyright  2010 Leonid Belousov (aka Master Cluster)        }
{      Master Cluster's Visual Component Library                    }
{      PGP component and routines - Added PGP SDK 2.x compatibility }
{      www.MasterCluster.com                                        }
{                                                                   }
{ The contents of this file are used with permission, subject to    }
{ the Mozilla Public License Version 1.1 (the "License"); you may   }
{ not use this file except in compliance with the License. You may  }
{ obtain a copy of the License at                                   }
{ <http://www.mozilla.org/MPL/MPL-1.1.html>                         }
{                                                                   }
{ Software distributed under the License is distributed on an       }
{ "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or    }
{ implied. See the License for the specific language governing      }
{ rights and limitations under the License.                         }
{                                                                   }
{*******************************************************************}

unit PGP_SDK;

{.$IFDEF VER150}
  {$WARN UNSAFE_TYPE OFF}
  {$WARN UNSAFE_CODE OFF}
  {$WARN UNSAFE_CAST OFF}
{.$ENDIF}

{------------------------------------------------------------------------------
 Interface to PGP_SDK.DLL version 1.7.2

 Some notes on the translation

* This was translated from the various headers that come with the source to
  pgp 5.5.3i (www.pgpi.com)

* PGPSDK is loaded dynamically. the variable PGPDLLLoaded is true if it was
  successfully loaded. You have to check this yourself, there's no attempt
  to protect you if the dll doesn't load

* Not all the routines have been translated. a few Mac specific routines
  were ignored, and there's a few yet to be done. Also, there's a few
  declared routines not actually exported from PGPSDK.dll

* Several of the functions take dynamic parameter lists. These have been
  represented in Delphi with OpenArrays. the c++ headers have
  FirstOption:pPGPOptionsList; ... which exactly duplicates the Openarray,
  you must specify at least one Option (you can use PGPONullOption if you don't
  want any option).

* The routines are divided roughly into sections
    1. Utility Routines
    2. Option List Routines
    3. Keyring Navigation & Maintenance
    4. Misc Encoding/Decoding Routines
    5. Encoding / Decoding Wrapper routines and options
    6. Key Server Support routines
    7. Support Routines

 PGPDelphi home page: http://www.kestral.com.au/devtools/pgp
 Grahame Grieve
 ------------------------------------------------------------------------------}

{!!}
{0.00.016  01 Sep 98 18:40    User : Grahame Grieve    
Comment: 
          Fix AddJobOptions & PGPODetachedSig (don't call selves no more)
          Add cdecl to call backs where missing
          Add event description
          Remove hardcoded #error in PGPErrorMsg (oops!)
          Add kPGPError_NoError
          }
{0.00.015  01 Sep 98 10:21    User : Grahame Grieve    }
{0.00.014  31 Aug 98 19:35    User : Jeff Sinclair    }
{0.00.013  31 Aug 98 18:35    User : Grahame Grieve    }
{0.00.012  30 Jul 98 18:38    User : Grahame Grieve    }
{0.00.011  24 Jul 98 11:54    User : Grahame Grieve    }
{0.00.010  23 Jul 98 23:23    User : Grahame Grieve    }
{0.00.009  23 Jul 98 15:37    User : Jeff Sinclair    }

interface

uses
  Windows;

type
// base types
  PGPUInt16 = Word;
  PGPBoolean = Byte; {= can be TRUE(1) or FALSE(0) }
  PGPUInt8 = Byte;
  PGPInt8 = Char;
  PGPInt16 = SmallInt;
  PGPUInt32 = Cardinal;
  PGPInt32 = Integer;
  PGPByte = PGPUINT8;
  PGPError = PGPINT32;
  PGPUserValue = Pointer;// a simple value sufficient to hold any numeric or pointer type
  PGPSize = PGPUINT32;   // PGPSize refers to memory sizes. Use PGPFileOffset for file offsets (was size_t)
  PGPFileOffset = PGPINT32;
  PGPFlags = PGPUINT32;
  PGPTime = PGPUINT32;
  PGPTimeInterval = PGPUINT32;{= In milliseconds }
  TIME_T = integer;
  PGPString = array [0..255] of char;
  PGPEnumType = DWORD;//integer;
  pPChar = ^PChar;

  // 64 bit types PGPUInt64 and PGPInt64 not translated
const
  PGPTrue = 1;
  PGPFalse = 0;
  kPGPMemoryFlags_None  = 0;
  kPGPMemoryFlags_Clear = 1;
//kPGPsdkAPIVersion = $03002010; //$02000010;
  kPGPsdkAPIVersion = $01000000;

// enumerated types: have been translated as constants but the type has
// been brought across so that it's easier to find the right constants

type
  PGPExportFormat = PGPEnumType;

const
  kPGPExportFormat_Basic    = 1;
  kPGPExportFormat_Complete = 255;


type
  PGPPublicKeyMessageFormat = PGPEnumType;
const
  //Encryption/Signature Message Formats
  kPGPPublicKeyMessageFormat_PGP = 1;
  kPGPPublicKeyMessageFormat_PKCS1 = 2;

type
  PGPCipherAlgorithm = PGPEnumType;
const
  //Symmetric Ciphers
  kPGPCipherAlgorithm_None	 = 0;
  kPGPCipherAlgorithm_IDEA	 = 1;
  kPGPCipherAlgorithm_3DES	 = 2;
  kPGPCipherAlgorithm_CAST5	 = 3;
  // ver 7.x+
  kPGPCipherAlgorithm_Blowfish   = 4;
  kPGPCipherAlgorithm_AES128     = 7;
  kPGPCipherAlgorithm_AES192     = 8;
  kPGPCipherAlgorithm_AES256     = 9;
  kPGPCipherAlgorithm_Twofish256 = 10;

  //kPGPNumPossibleCipherAlgorithms = 8;

type
  PGPPublicKeyAlgorithm = PGPEnumType;
const
  //Public/Private key algorithms
  kPGPPublicKeyAlgorithm_Invalid        = $FFFFFFFF;
  kPGPPublicKeyAlgorithm_RSA            = 1;
  kPGPPublicKeyAlgorithm_RSAEncryptOnly = 2;
  kPGPPublicKeyAlgorithm_RSASignOnly    = 3;
  kPGPPublicKeyAlgorithm_ElGamal        = $10; // A.K.A.Diffie-Hellman
  kPGPPublicKeyAlgorithm_DSA            = $11;
  // ver 7.x+
  kPGPPublicKeyAlgorithm_ECEncrypt      = $12;
  kPGPPublicKeyAlgorithm_ECSign         = $13;

  //kPGPPublicKeyAlgorithm_First          = kPGPPublicKeyAlgorithm_RSA;
  //kPGPPublicKeyAlgorithm_Last           = kPGPPublicKeyAlgorithm_DSA;

type
  PGPKeyOrdering = PGPEnumType;
const
  //Key ordering 6.5.x
  kPGPInvalidOrdering		    =  0;
  kPGPAnyOrdering                   =  1;
  kPGPUserIDOrdering                =  2;
  kPGPReverseUserIDOrdering         =  3;
  kPGPKeyIDOrdering                 =  4;
  kPGPReverseKeyIDOrdering          =  5;
  kPGPValidityOrdering              =  6;
  kPGPReverseValidityOrdering       =  7;
  kPGPTrustOrdering                 =  8;
  kPGPReverseTrustOrdering          =  9;
  kPGPEncryptKeySizeOrdering        = 10;
  kPGPReverseEncryptKeySizeOrdering = 11;
  kPGPSigKeySizeOrdering            = 12;
  kPGPReverseSigKeySizeOrdering     = 13;
  kPGPCreationOrdering              = 14;
  kPGPReverseCreationOrdering       = 15;
  kPGPExpirationOrdering            = 16;
  kPGPReverseExpirationOrdering     = 17;

  // ver 7.x+
  kPGPKeyOrdering_Invalid           = 0;
  kPGPKeyOrdering_Any               = 1;
  kPGPKeyOrdering_UserID            = 2;
  kPGPKeyOrdering_KeyID             = 3;
  kPGPKeyOrdering_Validity          = 4;
  kPGPKeyOrdering_Trust             = 5;
  kPGPKeyOrdering_EncryptionKeySize = 6;
  kPGPKeyOrdering_SigningKeySize    = 7;
  kPGPKeyOrdering_CreationDate      = 8;
  kPGPKeyOrdering_ExpirationDate    = 9;

// ver 7.x+
type
  PGPKeyDBObjProperty = PGPEnumType;
const
  kPGPKeyDBObjProperty_Invalid 			= 000;

  // Generic numeric properties
  kPGPKeyDBObjProperty_ObjectType		= 020;

  // Key boolean properties
  kPGPKeyProperty_IsSecret 			= 100;
  kPGPKeyProperty_IsAxiomatic			= 101;
  kPGPKeyProperty_IsRevoked			= 102;
  kPGPKeyProperty_IsDisabled			= 103;
  kPGPKeyProperty_IsNotCorrupt			= 104;
  kPGPKeyProperty_IsExpired			= 105;
  kPGPKeyProperty_NeedsPassphrase		= 106;
  kPGPKeyProperty_HasUnverifiedRevocation	= 107;
  kPGPKeyProperty_CanEncrypt			= 108;
  kPGPKeyProperty_CanDecrypt			= 109;
  kPGPKeyProperty_CanSign			= 110;
  kPGPKeyProperty_CanVerify			= 111;
  kPGPKeyProperty_IsEncryptionKey		= 112;
  kPGPKeyProperty_IsSigningKey			= 113;
  kPGPKeyProperty_IsSecretShared		= 114;
  kPGPKeyProperty_IsRevocable			= 115;
  kPGPKeyProperty_HasThirdPartyRevocation	= 116;
  kPGPKeyProperty_HasCRL			= 117;
  kPGPKeyProperty_IsOnToken			= 118;

  // Key numeric properties
  kPGPKeyProperty_AlgorithmID 			= 200;
  kPGPKeyProperty_Bits				= 201;
  kPGPKeyProperty_Trust				= 202;
  kPGPKeyProperty_Validity			= 203;
  kPGPKeyProperty_LockingAlgorithmID		= 204;
  kPGPKeyProperty_LockingBits			= 205;
  kPGPKeyProperty_Flags				= 206;
  kPGPKeyProperty_HashAlgorithmID		= 207;
  kPGPKeyProperty_Version			= 208;
  kPGPKeyProperty_KeyServerPreferences		= 209;
  kPGPKeyProperty_TokenNum			= 210;

  // Key time properties
  kPGPKeyProperty_Creation 			= 300;
  kPGPKeyProperty_Expiration			= 301;
  kPGPKeyProperty_CRLThisUpdate			= 302;
  kPGPKeyProperty_CRLNextUpdate			= 303;

  // Key data (variable sized) properties
  kPGPKeyProperty_Fingerprint			= 401;
  kPGPKeyProperty_KeyID				= 402;
  kPGPKeyProperty_PreferredAlgorithms		= 403;
  kPGPKeyProperty_ThirdPartyRevocationKeyID 	= 404;
  kPGPKeyProperty_KeyData			= 405;
  kPGPKeyProperty_X509MD5Hash			= 406;
  kPGPKeyProperty_PreferredKeyServer		= 407;

  // SubKey boolean properties
  kPGPSubKeyProperty_IsRevoked			= 501;
  kPGPSubKeyProperty_IsNotCorrupt		= 502;
  kPGPSubKeyProperty_IsExpired			= 503;
  kPGPSubKeyProperty_NeedsPassphrase		= 504;
  kPGPSubKeyProperty_HasUnverifiedRevocation	= 505;
  kPGPSubKeyProperty_IsRevocable		= 506;
  kPGPSubKeyProperty_HasThirdPartyRevocation	= 507;
  kPGPSubKeyProperty_IsOnToken			= 508;

  // SubKey numeric properties
  kPGPSubKeyProperty_AlgorithmID 		= 600;
  kPGPSubKeyProperty_Bits			= 601;
  kPGPSubKeyProperty_LockingAlgorithmID		= 602;
  kPGPSubKeyProperty_LockingBits		= 603;
  kPGPSubKeyProperty_Version			= 604;
  kPGPSubKeyProperty_Flags			= 605;

  // SubKey time properties
  kPGPSubKeyProperty_Creation 			= 700;
  kPGPSubKeyProperty_Expiration			= 701;

  // SubKey data (variable sized) properties
  kPGPSubKeyProperty_KeyData			= 800;
  kPGPSubKeyProperty_KeyID			= 801;

  // User ID boolean properties
  kPGPUserIDProperty_IsAttribute		= 900;

  // User ID numeric properties
  kPGPUserIDProperty_Validity 			= 1000;
  kPGPUserIDProperty_Confidence			= 1001;
  kPGPUserIDProperty_AttributeType		= 1002;

  // User ID data (variable sized) properties
  kPGPUserIDProperty_Name 			= 1200;
  kPGPUserIDProperty_AttributeData		= 1201;
  kPGPUserIDProperty_CommonName			= 1202;
  kPGPUserIDProperty_EmailAddress		= 1203;

  // Signature boolean properties
  kPGPSigProperty_IsRevoked 			= 1300;
  kPGPSigProperty_IsNotCorrupt			= 1301;
  kPGPSigProperty_IsTried			= 1302;
  kPGPSigProperty_IsVerified			= 1303;
  kPGPSigProperty_IsMySig			= 1304;
  kPGPSigProperty_IsExportable			= 1305;
  kPGPSigProperty_HasUnverifiedRevocation	= 1306;
  kPGPSigProperty_IsExpired			= 1307;
  kPGPSigProperty_IsX509			= 1308;

  // Signature numeric properties
  kPGPSigProperty_AlgorithmID			= 1400;
  kPGPSigProperty_TrustLevel			= 1401;
  kPGPSigProperty_TrustValue			= 1402;

  // Signature time properties
  kPGPSigProperty_Creation 			= 1500;
  kPGPSigProperty_Expiration			= 1501;

  // Signature data (variable sized) properties
  kPGPSigProperty_KeyID 			= 1600;
  kPGPSigProperty_X509Certificate		= 1601;
  kPGPSigProperty_X509IASN			= 1602;
  kPGPSigProperty_X509LongName			= 1603;
  kPGPSigProperty_X509IssuerLongName		= 1604;
  kPGPSigProperty_X509DNSName			= 1605;
  kPGPSigProperty_X509IPAddress			= 1606;
  kPGPSigProperty_X509DERDName			= 1607;

// ver 7.x+
type
  // For kPGPKeyProperty_Version
  PGPKeyVersion = PGPEnumType;
const
  kPGPKeyVersion_V3				= 3;
  kPGPKeyVersion_V4				= 4;


type
  PGPKeyPropName = PGPEnumType;
  // Key properties
const
  // String properties */
 // kPGPKeyPropUNUSED_1                = 10;  // feel free to use this

  kPGPKeyPropFingerprint	       = 1;
  kPGPKeyPropPreferredAlgorithms       = 2;
  kPGPKeyPropThirdPartyRevocationKeyID = 3;
  kPGPKeyPropKeyData		       = 4;
  kPGPKeyPropX509MD5Hash	       = 5;

  // Number properties
  kPGPKeyPropAlgID 		     = 20;
  kPGPKeyPropBits		     = 21;
  kPGPKeyPropTrust		     = 22;	// old trust model only
  kPGPKeyPropValidity		     = 23;	// both trust models
  kPGPKeyPropLockingAlgID	     = 24;
  kPGPKeyPropLockingBits	     = 25;
  kPGPKeyPropFlags		     = 26;
  // Time properties
  kPGPKeyPropCreation 		     = 40;
  kPGPKeyPropExpiration		     = 41;
  kPGPKeyPropCRLThisUpdate	     = 42;
  kPGPKeyPropCRLNextUpdate	     = 43;
  // PGPBoolean properties
  kPGPKeyPropIsSecret 		     = 60;
  kPGPKeyPropIsAxiomatic	     = 61;
  kPGPKeyPropIsRevoked		     = 62;
  kPGPKeyPropIsDisabled		     = 63;
  kPGPKeyPropIsNotCorrupt	     = 64;
  kPGPKeyPropIsExpired		     = 65;
  kPGPKeyPropNeedsPassphrase	     = 66;
  kPGPKeyPropHasUnverifiedRevocation = 67;
  kPGPKeyPropCanEncrypt		     = 68;
  kPGPKeyPropCanDecrypt		     = 69;
  kPGPKeyPropCanSign		     = 70;
  kPGPKeyPropCanVerify		     = 71;
  kPGPKeyPropIsEncryptionKey	     = 72;
  kPGPKeyPropIsSigningKey	     = 73;
  kPGPKeyPropIsSecretShared	     = 74;
  kPGPKeyPropIsRevocable	     = 75;
  kPGPKeyPropHasThirdPartyRevocation = 76;
  kPGPKeyPropHasCRL		     = 77;

type
  PGPUserIDPropName = PGPEnumType;
const
  //User ID properties
  // String properties
  kPGPUserIDPropName 	      = 80;
  kPGPUserIDPropAttributeData = 81;
  kPGPUserIDPropCommonName    = 82;
  kPGPUserIDPropEmailAddress  = 83;

  // Number properties
  kPGPUserIDPropValidity      = 100; // both trust models
  kPGPUserIDPropConfidence    = 101; // new trust model only
  kPGPUserIDPropAttributeType = 102;

  // Time properties

  // PGPBoolean properties
  kPGPUserIDPropIsAttribute   = 110;

type
  PGPSigPropName = PGPEnumType;
const
  // Signature properties
  kPGPSigPropKeyID 		     = 120;
  kPGPSigPropX509Certificate	     = 121;
  kPGPSigPropX509IASN		     = 122;
  kPGPSigPropX509LongName	     = 123;
  kPGPSigPropX509IssuerLongName	     = 124;
  kPGPSigPropX509DNSName	     = 125;
  kPGPSigPropX509IPAddress	     = 126;
  kPGPSigPropX509DERDName	     = 127;

  // Number properties
  kPGPSigPropAlgID 		     = 140;
  kPGPSigPropTrustLevel		     = 141;
  kPGPSigPropTrustValue		     = 142;

  // Time properties
  kPGPSigPropCreation 		     = 160;
  kPGPSigPropExpiration		     = 161;

  // PGPBoolean properties
  kPGPSigPropIsRevoked 		     = 180;
  kPGPSigPropIsNotCorrupt	     = 181;
  kPGPSigPropIsTried		     = 182;
  kPGPSigPropIsVerified		     = 183;
  kPGPSigPropIsMySig		     = 184;
  kPGPSigPropIsExportable	     = 185;
  kPGPSigPropHasUnverifiedRevocation = 186;
  kPGPSigPropIsExpired		     = 187;
  kPGPSigPropIsX509		     = 188;

  // Note on kPGPSigPropIsMySig.  This is a convenience property for
  // determining whether the certification was made by one of the
  // caller's own private keys.  This can only return true if the
  // signing key is in the same base keyset as the certification. If the
  // signing key is( suspected to be) in a different base keyset, call
  // PGPGetSigCertifierKey( certset, signerset, &key) followed by
  // PGPGetKeyBoolean( key, kPGPKeyPropIsSecret, &secret).


type
  PGPKeyRingOpenFlags = PGPUInt32;   //!! note different type - cardinal not integer??
const
  kPGPKeyRingOpenFlags_Reserved  = (1 shl 0);
  kPGPKeyRingOpenFlags_Mutable   = (1 shl 1);
  kPGPKeyRingOpenFlags_Create    = (1 shl 2);
  // The following flags are only used by PGPOpenKeyRing
  kPGPKeyRingOpenFlags_Private   = (1 shl 8);
  kPGPKeyRingOpenFlags_Trusted   = (1 shl 9);


// ver 7.x+
type
  // Key DB object types for the property kPGPKeyDBObjProperty_ObjectType
  PGPKeyDBObjType = PGPEnumType;
const
  kPGPKeyDBObjType_Invalid				= 0;
  kPGPKeyDBObjType_Key					= (1 shl 0);
  kPGPKeyDBObjType_SubKey				= (1 shl 1);
  kPGPKeyDBObjType_UserID				= (1 shl 2);
  kPGPKeyDBObjType_Signature				= (1 shl 3);
  kPGPKeyDBObjType_Any					= -1;


type
  PGPMatchCriterion = PGPEnumType;
  // Used by filtering functions to specify type of match.
const
  kPGPMatchDefault          = 1;
  kPGPMatchEqual            = 1;    // searched val == supplied val
  kPGPMatchGreaterOrEqual   = 2;    // searched val >= supplied val
  kPGPMatchLessOrEqual      = 3;    // searched val <= supplied val
  kPGPMatchSubString        = 4;    // searched val is contained in supplied val

type
  PGPKeyIDStringType = PGPEnumType;
const
  kPGPKeyIDString_Abbreviated = 2;
  kPGPKeyIDString_Full        = 3;
  kPGPMaxKeyIDStringSize      = (127 + 1);

type
  PGPHashAlgorithm = PGPEnumType;
const
  kPGPHashAlgorithm_Invalid   = 0;
  kPGPHashAlgorithm_MD5       = 1;
  kPGPHashAlgorithm_SHA       = 2;
  kPGPHashAlgorithm_RIPEMD160 = 3;
  kPGPHashAlgorithm_SHADouble = 4;
  // ver 7.x+
  kPGPHashAlgorithm_SHA256    = 8; // from draft-ietf-openpgp-rfc2440bis-03.txt
  kPGPHashAlgorithm_SHA384    = 9;
  kPGPHashAlgorithm_SHA512    = 10;

type
  PGPValidity = PGPEnumType;
const
  // Trust values, used to set validity values
  kPGPKeyTrust_Mask      = $7;
  kPGPKeyTrust_Undefined = $0;
  kPGPKeyTrust_Unknown   = $1;
  kPGPKeyTrust_Never     = $2;
  kPGPKeyTrust_Marginal  = $5;
  kPGPKeyTrust_Complete  = $6;
  kPGPKeyTrust_Ultimate  = $7;

  kPGPNameTrust_Mask      = $3;
  kPGPNameTrust_Unknown   = $0;
  kPGPNameTrust_Untrusted = $1;
  kPGPNameTrust_Marginal  = $2;
  kPGPNameTrust_Complete  = $3;

  kPGPValidity_Unknown   = kPGPNameTrust_Unknown;
  kPGPValidity_Invalid   = kPGPNameTrust_Untrusted;
  kPGPValidity_Marginal  = kPGPNameTrust_Marginal;
  kPGPValidity_Complete  = kPGPNameTrust_Complete;
  kPGPValidity_UNTRUSTED = kPGPValidity_Invalid;

  kPGPMimeSeparatorSize = 81;

type
  PGPLineEndType = PGPEnumType;
const
  kPGPLineEnd_Default = 0;
  kPGPLineEnd_LF      = 1;
  kPGPLineEnd_CR      = 2;
  kPGPLineEnd_CRLF    = kPGPLineEnd_LF or kPGPLineEnd_CR;

  // PGPOLocalEncoding options argument
  // Only one of Force or Auto should be used, and the other values are modifiers.
  kPGPLocalEncoding_None = $0; {/* nothing on*/}
  kPGPLocalEncoding_Force = $01;
  kPGPLocalEncoding_Auto = $02;
  kPGPLocalEncoding_NoMacBinCRCOkay = $04;

{This is the value of the Expiration time which means "never expires"}
  kPGPExpirationTime_Never    : PGPTime = 0;

type
  PGPEventType = PGPEnumType;
const
  kPGPEvent_NullEvent              =  0; // Nothing happened
  kPGPEvent_InitialEvent           =  1; // Final event
  kPGPEvent_FinalEvent             =  2; // Final event
  kPGPEvent_ErrorEvent             =  3; // An error occurred
  kPGPEvent_WarningEvent           =  4; // Warning event
  kPGPEvent_EntropyEvent           =  5; // More entropy is needed
  kPGPEvent_PassphraseEvent        =  6; // A passphrase is needed
  kPGPEvent_InsertKeyEvent         =  7; // Smart card must be inserted
  kPGPEvent_AnalyzeEvent           =  8; // Initial analysis event, before any output
  kPGPEvent_RecipientsEvent        =  9; // Recipient list report, before any output
  kPGPEvent_KeyFoundEvent          = 10; // Key packet found
  kPGPEvent_OutputEvent            = 11; // Output specification needed
  kPGPEvent_SignatureEvent         = 12; // Signature status report
  kPGPEvent_BeginLexEvent          = 13; // Initial event per lexical unit
  kPGPEvent_EndLexEvent            = 14; // Final event per lexical unit
  kPGPEvent_RecursionEvent         = 15; // Notification of recursive job creation
  kPGPEvent_DetachedSignatureEvent = 16; // Need input for verification of detached signature
  kPGPEvent_KeyGenEvent            = 17; // Key generation progress
  kPGPEvent_KeyServerEvent         = 18; // Key Server progress
  kPGPEvent_KeyServerSignEvent     = 19; // Key Server passphrase

type
  PGPAnalyzeType = PGPEnumType;
const
  kPGPAnalyze_Encrypted            =  0; // Encrypted message
  kPGPAnalyze_Signed               =  1; // Signed message
  kPGPAnalyze_DetachedSignature    =  2; // Detached signature
  kPGPAnalyze_Key                  =  3; // Key data
  kPGPAnalyze_Unknown              =  4; // Non-pgp message

type
  PGPFeatureSelector = PGPEnumType;
const
  kPGPFeatures_GeneralSelector        = 1;
  kPGPFeatures_ImplementationSelector = 2;

  kPGPFeatureMask_CanEncrypt   = $1;
  kPGPFeatureMask_CanDecrypt   = $2;
  kPGPFeatureMask_CanSign      = $4;
  kPGPFeatureMask_CanVerify    = $8;
  kPGPFeatureMask_IsDebugBuild = $1;
  kPGPFeatureMask_HasTimeout   = $2;

type
  PGPsdkPrefSelector = PGPEnumType;
const
  kPGPsdkPref_First             = 0;
  kPGPsdkPref_PublicKeyring     = 0;
  kPGPsdkPref_PrivateKeyring    = 1;
  kPGPsdkPref_RandomSeedFile    = 2;
  kPGPsdkPref_DefaultKeyID      = 3;
  kPGPsdkPref_LastPlusOne       = 4;

  kPGPKeyServerStateConnect     = $01;
  kPGPKeyServerStateWait        = $02;
  kPGPKeyServerStateReceive     = $03;
  kPGPKeyServerStateSend        = $04;
  kPGPKeyServerStateDisconnect  = $05;

type
  PGPKeyServerKeySpace = PGPEnumType;
const
  kPGPKSKeySpaceDefault = 1;
  kPGPKSKeySpaceNormal  = 2;
  kPGPKSKeySpacePending = 3;

type
  PGPKeyServerAccessType = PGPEnumType;
const
  kPGPKSAccess_Default       = 1;
  kPGPKSAccess_Normal        = 2;
  kPGPKSAccess_Administrator = 3;

  kPGPKeyServerMonitorMagic = $4D4F4E49;

// errors
// PGPSDK.dll gives other errors than those listed here, but you can't
// get useful error information on them. A project for the future is to
// gather all those out of the code and gather them here
  kPGPError_NoError                  = 0;

  kPGPError_FirstError               = -11500;
  kPGPError_LastError                = -10500;
  kPGPError_BadPassphrase            = -11500;
  kPGPError_OptionNotFound           = -11499;

  // Errors from pgpEncode
  kPGPError_RedundantOptions         = -11498;
  kPGPError_KeyRevoked               = -11497;
  kPGPError_KeyExpired               = -11496;
  kPGPError_KeyDisabled              = -11495;
  kPGPError_KeyInvalid               = -11494;
  kPGPError_KeyUnusableForEncryption = -11493;
  kPGPError_KeyUnusableForSignature  = -11492;
  kPGPError_OutputBufferTooSmall     = -11491;
  kPGPError_InconsistentEncryptionAlgorithms = -11490;
  kPGPError_MissingPassphrase        = -11489;
  kPGPError_CombinedConventionalAndPublicEncryption = -11488;
  kPGPError_DetachedSignatureWithoutSigningKey = -11487;
  kPGPError_DetachedSignatureWithEncryption = -11486;
  kPGPError_NoInputOptions           = -11485;
  kPGPError_MultipleInputOptions     = -11484;
  kPGPError_InputFile                = -11483;
  kPGPError_NoOutputOptions          = -11482;
  kPGPError_MultipleOutputOptions    = -11481;
  kPGPError_MissingEventHandler      = -11480;
  kPGPError_MissingKeySet            = -11479;
  kPGPError_DetachedSignatureFound   = -11478;
  kPGPError_NoDecryptionKeyFound     = -11477;
  kPGPError_CorruptSessionKey        = -11476;
  kPGPError_SkipSection              = -11475;
  kPGPError_Interrupted              = -11474;
  kPGPError_TooManyARRKs             = -11473;
  kPGPError_IncompatibleAPI          = -11460;

  // misc errors
  kPGPError_NotMacBinary             = -11450;
  kPGPError_NoMacBinaryTranslationAvailable = -11449;
  kPGPError_BadSignature             = -11448;
  kPGPError_CAPIUnsupportedKey       = -11447; // PGP60+
  kPGPError_SelfTestFailed           = -11446; // PGP65+
  kPGPError_SelfTestsNotExecuted     = -11445; // PGP65+
  kPGPError_BadIntegrity             = -11444; // PGP65+
  kPGPError_DeCompressionFailed      = -11443; // PGP65+


  // filter errors
  kPGPError_InconsistentFilterClasses = -11440;
  kPGPError_UnsupportedLDAPFilter     = -11439;
  kPGPError_UnsupportedHKPFilter      = -11438;
  kPGPError_UnknownFilterType         = -11437;
  kPGPError_InvalidFilterParameter    = -11436;

  // old errors
  kPGPError_OutOfRings                = -11420;
  kPGPError_BadHashNumber             = -11419;
  kPGPError_BadCipherNumber           = -11418;
  kPGPError_BadKeyLength              = -11417;
  kPGPError_SizeAdviseFailure         = -11416;
  kPGPError_ConfigParseFailure        = -11415;
  kPGPError_ConfigParseFailureBadFunction = -11414;
  kPGPError_ConfigParseFailureBadOptions = -11413;
  kPGPError_KeyIsLocked               = -11412;
  kPGPError_CantDecrypt               = -11411;
  kPGPError_UnknownString2Key         = -11410;
  kPGPError_BadSessionKeySize         = -11409;
  kPGPError_UnknownVersion            = -11408;
  kPGPError_BadSessionKeyAlgorithm    = -11407;
  kPGPError_UnknownSignatureType      = -11406;
  kPGPError_BadSignatureSize          = -11405;
  kPGPError_SignatureBitsWrong        = -11404;
  kPGPError_ExtraDateOnSignature      = -11403;
  kPGPError_SecretKeyNotFound         = -11402;
  kPGPError_AdditionalRecipientRequestKeyNotFound = -11401;
  kPGPError_InvalidCommit             = -11400;
  kPGPError_CantHash                  = -11399;
  kPGPError_UnbalancedScope           = -11398;
  kPGPError_WrongScope                = -11397;
  kPGPError_FIFOReadError             = -11396;
  kPGPError_RandomSeedTooSmall        = -11395;
  kPGPError_EnvPriorityTooLow         = -11394;
  kPGPError_UnknownCharMap            = -11393;
  kPGPError_AsciiParseIncomplete      = -11392;
  kPGPError_BadPacket                 = -11391;
  kPGPError_TroubleKeySubKey          = -11390;
  kPGPError_TroubleSigSubKey          = -11389;
  kPGPError_TroubleBadTrust           = -11388;
  kPGPError_TroubleUnknownPacketByte  = -11387;
  kPGPError_TroubleUnexpectedSubKey   = -11386;
  kPGPError_TroubleUnexpectedName     = -11385;
  kPGPError_TroubleUnexpectedSignature = -11384;
  kPGPError_TroubleUnexpectedUnknown  = -11383;
  kPGPError_TroubleUnexpectedTrust    = -11382;
  kPGPError_TroubleKeyTooBig          = -11381;
  kPGPError_TroubleSecretKeyTooBig    = -11380;
  kPGPError_TroubleNameTooBig         = -11379;
  kPGPError_TroubleSignatureTooBig    = -11378;
  kPGPError_TroubleUnknownTooBig      = -11377;
  kPGPError_TroubleDuplicateKeyID     = -11376;
  kPGPError_TroubleDuplicateKey       = -11375;
  kPGPError_TroubleDuplicateSecretKey = -11374;
  kPGPError_TroubleDuplicateName      = -11373;
  kPGPError_TroubleDuplicateSignature = -11372;
  kPGPError_TroubleDuplicateUnknown   = -11371;
  kPGPError_TroubleBareKey            = -11370;
  kPGPError_TroubleVersionBugPrev     = -11369;
  kPGPError_TroubleVersionBugCur      = -11368;
  kPGPError_TroubleOldSecretKey       = -11367;
  kPGPError_TroubleNewSecretKey       = -11366;
  kPGPError_TroubleImportingNonexportableSignature= -11365;

  // The set of errors in this range are the ones which will NOT abort
  // a keyring check operation.  These errors just make us skip the key
  // and go on to the next.
  kPGPError_KEY_MIN                   = -11350;
  kPGPError_KEY_LONG                  = kPGPError_KEY_MIN;
  kPGPError_KeyPacketTruncated        = -11349;
  kPGPError_UnknownKeyVersion         = -11348;
  kPGPError_UnknownPublicKeyAlgorithm = -11347;
  kPGPError_MalformedKeyModulus       = -11346;
  kPGPError_MalformedKeyExponent      = -11345;
  kPGPError_RSAPublicModulusIsEvent   = -11344;
  kPGPError_RSAPublicExponentIsEvent  = -11343;
  kPGPError_MalformedKeyComponent     = -11342;
  kPGPError_KeyTooLarge               = -11341;
  kPGPError_PublicKeyTooSmall         = -11340;
  kPGPError_PublicKeyTooLarge         = -11339;
  kPGPError_PublicKeyUnimplemented    = -11338;
  kPGPError_KEY_MAX                   = kPGPError_PublicKeyUnimplemented;

  // kPGPError_SIG_MAX
  kPGPError_SIG_LONG                  = -11330;
  kPGPError_TruncatedSignature        = -11329;
  kPGPError_MalformedSignatureInteger = -11328;
  kPGPError_UnknownSignatureAlgorithm = -11327;
  kPGPError_ExtraSignatureMaterial    = -11326;
  kPGPError_UnknownSignatureVersion   = -11325;
  kPGPError_RevocationKeyNotFound     = -11324;
  // kPGPError_SIG_MIN

  //  kPGPError_KEYDB_MAX
  kPGPError_OutOfEntropy              = -11320;
  kPGPError_ItemIsReadOnly            = -11319;
  kPGPError_InvalidProperty           = -11318;
  kPGPError_FileCorrupt               = -11317;
  kPGPError_DuplicateCert             = -11316;
  kPGPError_DuplicateUserID           = -11315;
  kPGPError_CertifyingKeyDead         = -11314;
  kPGPError_ItemWasDeleted            = -11313;
  kPGPError_KeyDBMismatch             = -11312;
  // kPGPError_KEYDB_MIN = kPGPError_KeyDBMismatch

  // kPGPError_SERVER_MAX
  kPGPError_ServerOpenFailed          = -11300;
  kPGPError_ServerSearchFailed        = -11299;
  kPGPError_ServerBindFailed          = -11298;
  kPGPError_ServerAddFailed           = -11297;
  kPGPError_ServerPartialAddFailure   = -11296;
  kPGPError_ServerInvalidProtocol     = -11295;
  kPGPError_ServerOperationNotAllowed = -11294;
  kPGPError_ServerAuthorizationRequired = -11293;
  kPGPError_ServerAuthorizationFailed = -11292;
  kPGPError_ServerUnknownResponse     = -11291;
  kPGPError_ServerUnknownHost         = -11290;
  kPGPError_ServerSocketError         = -11289;
  kPGPError_ServerConnectFailed       = -11288;
  kPGPError_ServerCorruptKeyBlock     = -11287;
  kPGPError_ServerRequestFailed       = -11286;
  kPGPError_ServerKeyFailedPolicy     = -11285;
  kPGPError_ServerBadKeysInSearchResults = -11284;
  kPGPError_ServerKeyAlreadyExists    = -11283;
  kPGPError_ServerTooManyResults      = -11282;


type
// opaque types
  // General data types used by PGPsdk
  pPGPContext     = pointer;
  pPGPFileSpec    = pointer;
  pPGPOptionsList = pointer;

  //Data types used by the key manipulation functions
  pPGPKeyDB    = pointer;
  pPGPKeyDBRef = pointer;
  pPGPKeyDBObj = pointer;
  pPGPKey      = pointer;
  pPGPSubKey   = pointer;
  pPGPUserID   = pointer;
  pPGPSig      = pointer;
  pPGPKeySet   = pointer;
  pPGPKeyList  = pointer;
  pPGPKeyIter  = pointer;
  pPGPFilter   = pointer;

  // do not attempt to interpret these bytes; they *will* change
  TPGPKeyID = record
    //opaqueBytes: array [0..33] of PGPByte; // 6.5.x
    opaqueBytes: array [0..35] of PGPByte; // 7.x+
  end;

  pPGPKeyID = ^TPGPKeyID;

  //Data types used by symmetric ciphers, cipher modes, hashing
  pPGPHashContext       = pointer;
  pPGPPublicKeyContext  = pointer;
  pPGPPrivateKeyContext = pointer;
  pPGPCBCContext        = pointer;
  pPGPCFBContext        = pointer;
  pPGPSymmetricCipherContext = pointer;
  pPGPJob               = pointer;
  pPGPKeyServer         = pointer;
  pPGPMemoryMgr         = Pointer;

//   misc types
  TPGPVersion = record
    majorVersion: PGPUINT16;
    minorVersion: PGPUINT16;
  end {TPGPVersion};

  // PGPOLocalEncoding options argument
  // Only one of Force or Auto should be used, and the other values are modifiers.
  TPGPLocalEncodingFlags = PGPFLAGS;
  TPGPMemoryFlags        = PGPFLAGS;

  // event records
  TPGPEventNullData = record
    bytesWritten: PGPFILEOFFSET;
    bytesTotal  : PGPFILEOFFSET;
  end;

  TPGPEventErrorData = record
    error   : PGPERROR;
    errorArg: Pointer;
  end;

  TPGPEventWarningData = record
    warning   : PGPERROR;
    warningArg: Pointer;
  end;

  TPGPEventEntropyData = record
    entropyBitsNeeded: PGPUINT32;
  end;

  TPGPEventPassphraseData = record
    fConventional: PGPBOOLEAN;
    keyset       : pPGPKeySet;
  end;

  TPGPEventRecipientsData = record
    recipientSet: pPGPKeySet;
    conventionalPassphraseCount: PGPUINT32;
    keyCount    : PGPUINT32;
    keyIDArray  : pPGPKEYID;
  end;

  TPGPEventKeyFoundData = record
    keySet: pPGPKeySet;
  end;

  TPGPEventSignatureData = record
    signingKeyID: TPGPKeyID;
    signingKey  : pPGPKey;
    checked     : PGPBOOLEAN;
    verified    : PGPBOOLEAN;
    keyRevoked  : PGPBOOLEAN;
    keyDisabled : PGPBOOLEAN;
    keyExpired  : PGPBOOLEAN;
    keyMeetsValidityThreshold: PGPBOOLEAN;
    keyValidity : PGPVALIDITY;
    creationTime: PGPTIME;
  end;

  TPGPEventAnalyzeData = record
    sectionType: PGPANALYZETYPE;
  end;

  TPGPEventOutputData = record
    messageType    : PGPUINT32;
    suggestedName  : PChar;
    forYourEyesOnly: PGPBOOLEAN;
  end;

  TPGPEventBeginLexData = record
    sectionNumber: PGPUINT32;
    sectionOffset: PGPSIZE;
  end;

  TPGPEventEndLexData = record
    sectionNumber: PGPUINT32;
  end;

  TPGPEventKeyGenData = record
    state: PGPUINT32;
  end;

  TPGPEventKeyServerData = record
    state: PGPUINT32;
    soFar: PGPUINT32;
    total: PGPUINT32;
  end;

  TPGPEventKeyServerSignData = record
    state: PGPUINT32;
  end;
  // The following events have no event-specific data defined for them:
  // kPGPEvent_InsertKeyEvent
  // kPGPEvent_RecursionEvent
  // kPGPEvent_DetachedSignatureEvent
  // kPGPEvent_InitialEvent
  // kPGPEvent_FinalEvent

  TPGPEventData = record
    case Word of
      1: (nullData: TPGPEVENTNULLDATA;);
      2: (errorData: TPGPEVENTERRORDATA;);
      3: (warningData: TPGPEVENTWARNINGDATA;);
      4: (entropyData: TPGPEVENTENTROPYDATA;);
      5: (passphraseData: TPGPEVENTPASSPHRASEDATA;);
      6: (recipientsData: TPGPEVENTRECIPIENTSDATA;);
      7: (keyFoundData: TPGPEVENTKEYFOUNDDATA;);
      8: (signatureData: TPGPEVENTSIGNATUREDATA;);
      9: (analyzeData: TPGPEVENTANALYZEDATA;);
     10: (outputData: TPGPEVENTOUTPUTDATA;);
     11: (beginLexData: TPGPEVENTBEGINLEXDATA;);
     12: (endLexData: TPGPEVENTENDLEXDATA;);
     13: (keyGenData: TPGPEVENTKEYGENDATA;);
     14: (keyServerData: TPGPEVENTKEYSERVERDATA;);
     15: (keyServerSignData: TPGPEventKeyServerSignData;);
  end;

  pPGPEvent = ^TPGPEvent;
  TPGPEvent = record
    version  : TPGPVersion;
    NextEvent: pPGPEvent;
    job      : pPGPJob;
    etype    : PGPEventType;
    data     : TPGPEventData;
  end;

  //typedef PGPError (*PGPEventHandlerProcPtr)
  //                        ( pPGPContext context,
  //                          PGPEvent *event,
  //                          PGPUserValue userValue);
  TPGPEventHandlerProcPtr = function (context:pPGPContext;
                                     event:pPGPEvent;
                                     UserValue:PGPUserValue):PGPError; cdecl;

  //typedef void	*(*PGPMemoryAllocationProc)
  //                        ( pPGPContext context,
  //                          PGPSize allocationSize, PGPMemoryFlags flags,
  //                          PGPUserValue userValue );
  TPGPMemoryAllocationProc = function (Context:pPGPContext;
                                      AllocSize:PGPSize;
                                      flags:TPGPMemoryFlags;
                                      UserValue:PGPUserValue):pointer; cdecl;

  //typedef PGPError (*PGPMemoryReallocationProc)
  //                                ( pPGPContext context,
  //                                  void **allocation, PGPSize newAllocationSize,
  //                                  PGPMemoryFlags flags, PGPUserValue userValue );
  TPGPMemoryReallocationProc = function (context:pPGPContext;
                                        var allocation:pointer;
                                        newAllocationSize:PGPSize;
                                        flags:TPGPMemoryFlags;
                                        userValue:PGPUserValue):PGPError; cdecl;

  //typedef PGPError (*PGPMemoryDeallocationProc)
  //                                  ( pPGPContext context,
  //                                    void *allocation, PGPUserValue userValue );
  TPGPMemoryDeallocationProc = function (context:pPGPContext;
                                        allocation:pointer;
                                        userValue:PGPUserValue):PGPError;    cdecl;

  TPGPNewContextStruct = record
    sizeofStruct: PGPUINT32; // sizeofStruct must be inited to sizeof( PGPNewContextStruct )
    allocProc   : TPGPMEMORYALLOCATIONPROC;
    reallocProc : TPGPMEMORYREALLOCATIONPROC;
    deallocProc : TPGPMEMORYDEALLOCATIONPROC;
    allocUserValue: PGPUSERVALUE;
  end {PGPNewContextStruct};

  TPGPAlgorithmInfo = record
    ShortName: array [0..31] of char;
    LongName : array [0..96] of char;
    CopyRight: array [0..128] of char;
    flags    : PGPFlags;
    reserved : PGPUInt32;
  end;

  TPGPPublicKeyAlgorithmInfo = record
    info      : TPGPAlgorithmInfo;
    aldID     : PGPPublicKeyAlgorithm;
    CanEncrypt: PGPBoolean;
    CanSign   : PGPBoolean;
    reserved1 : PGPBoolean;
    reserved2 : PGPBoolean;
    reserved  : array [0..7] of PGPUInt32;
  end;

  TPGPSymmetricCipherInfo = record
    info: TPGPAlgorithmInfo;
    algID: PGPPublicKeyAlgorithm;
    reserved: array [0..7] of PGPUInt32;
  end;

  pPGPKeyServerMonitor = ^TPGPKeyServerMonitor;
  TPGPKeyServerMonitor = record
    magic: PGPUInt32;
    monitorTag: PChar;
    monitorValues: PPChar;
    next: pPGPKeyServerMonitor;
  end;

function GetPGPDLLPath: String;
function IsLoadedDLL: Boolean;
function IsOldSDK: Boolean;

{------------------------------------------------------------------------------
  1. Utility Routines
 ------------------------------------------------------------------------------}
// Call PGPsdkInit() before using the SDK.
// Call PGPsdkCleanup() when you are } done (but after disposing of any PGPContexts).
// You may call multiple times (with no effect), but be sure to call PGPsdkCleanup()
// for each call to PGPsdkInit().

function PGPsdkInit: PGPERROR;
function PGPsdkCleanup: PGPERROR;

// always pass kPGPsdkVersion
function PGPNewContext(clientAPIVersion: PGPUINT32; var newContext: pPGPContext): PGPERROR;

//var PGPNewContextCustom: function(clientAPIVersion: PGPUINT32; var custom: TPGPNEWCONTEXTSTRUCT; var newContext: pPGPContext): PGPERROR cdecl;

function PGPFreeContext(context: pPGPContext): PGPERROR;

(*
var PGPGetContextUserValue: function(context: pPGPContext;
                                     var userValue: PGPUSERVALUE): PGPERROR cdecl;

var PGPSetContextUserValue: function(context: pPGPContext;
                                     userValue: PGPUSERVALUE): PGPERROR cdecl;

var PGPContextGetRandomBytes: function(context: pPGPContext; buf: Pointer;
                                       len: PGPSIZE): PGPERROR cdecl;

// Memory allocation
//
// The PGPsdk uses PGPNewData() for allocated return parameters.
// Clients should use PGPFreeData() to deallocate memory allocated with
// PGPNewData().
//
// PGPNewData should generally not be used by clients (there are a few
// special cases).
//
// PGPNewSecureData is intended to store sensitive data such as passphrases.
//var PGPNewData: function(context: pPGPContext; allocationSize: PGPSIZE):pointer cdecl;

// 'didLock' is TRUE if the memory is guaranteed to not be paged
// Some OSes don't support this feature
var PGPNewSecureData: function(context: pPGPContext; allocationSize: PGPSIZE;
                               var didLock: PGPBOOLEAN):pointer cdecl;
*)

// can free allocation from PGPNewData or PGPNewSecureData
function PGPFreeData(allocation: Pointer): PGPERROR;

//--- File Handling --------------------------------------
//var PGPCopyFileSpec: function(fileRef: pPGPFileSpec; var ref: pPGPFileSpec): PGPERROR cdecl;

function PGPFreeFileSpec(fileRef: pPGPFileSpec): PGPERROR;

function PGPNewFileSpecFromFullPath(context: pPGPContext;
                                    path: PChar;
                                    var ref: pPGPFileSpec): PGPERROR;

// fullPathPtr***must*** be freed with PGPFreeData
//var PGPGetFullPathFromFileSpec: function(fileRef: pPGPFileSpec;  var fullPathPtr: PChar): PGPERROR cdecl;

//--- Time Functions --------------------------------------

//var PGPGetTime: function: PGPTIME cdecl;

// these use time_t type as returned by time() in Std C libraries
//var PGPGetStdTimeFromPGPTime: function(theTime: PGPTIME): TIME_T cdecl;
//var PGPGetPGPTimeFromStdTime: function(theTime: TIME_T): PGPTIME cdecl;

// year, month, day may be NULL if desired
procedure PGPGetYMDFromPGPTime(theTime: PGPTIME; var year: PGPUINT16; var month: PGPUINT16; var day: PGPUINT16);

//--- Randomisation Functions -------------------------------------------------
function PGPGlobalRandomPoolAddKeystroke(event: PGPINT32): PGPUINT32;
function PGPGlobalRandomPoolAddMouse(x: PGPUINT32; y: PGPUINT32): PGPUINT32; // deprecated, use xxxxMouseMoved
function PGPGlobalRandomPoolMouseMoved: PGPUInt32;

// Extra functions for entropy estimation
function PGPGlobalRandomPoolGetEntropy: PGPUINT32;
function PGPGlobalRandomPoolGetSize: PGPUINT32;
function PGPGlobalRandomPoolGetMinimumEntropy: PGPUINT32;
function PGPGlobalRandomPoolHasMinimumEntropy: PGPBOOLEAN;

function PGPGetErrorString(theError: PGPERROR; bufferSize: PGPSIZE; theString: PChar): PGPERROR;

{
	API version:
	Top byte is major, next 3 nibbles minor, next 2 bug fix,
	last nibble reserved:  0xMMmmmrrR

	example: 1.0.0 = 0x01000000

	0x01000000		SDK 1.0.0
	0x02000000		SDK 1.1.0
	0x02000010		SDK 1.1.1
	0x03000000		SDK 1.5
	0x03000010		SDK 1.5.1
	0x03000020		SDK 1.5.2
	0x03001000		SDK 1.6
	0x03002000		SDK 1.7
	0x03002010		SDK 1.7.1
}
//#define kPGPsdkAPIVersion		( (PGPUInt32)0x03002010 )
function PGPMajorVersion(v: PGPUINT32): Integer;
function PGPMinorVersion(v: PGPUINT32): Integer;
function PGPRevVersion(v: PGPUINT32): Integer;

// Determine which features are present in the PGPsdk. This is the only
// way to correctly determine which features are present. The version
// number may be the same for different builds that lack some features.

// Each selector designates a PGPFlags word, which can be obtained via
// PGPGetFeatureFlags().  The flags can be tested using the
// supplied masks.  We can add more selectors as needed.  The masks
// are not intended to be restricted to a single bit.
// Flags should not be used for attributes that have unknown length.
//
// A kPGPError_ItemNotFound will be returned if the caller specifies
// a selector which is not recognized by the PGPsdk.  This could
// occur if an app links to an older version of the SDK.
function PGPGetSDKVersion(var version: PGPUINT32): PGPERROR;

(*
// Return a C string of the form:
// "PGPsdk version 1.0 (C) 1997 Pretty Good Privacy, Inc"
var PGPGetSDKString: function(var TheString:PGPString): PGPERROR cdecl;

// return a flags word for the feature selector
var PGPGetFeatureFlags: function(selector: PGPFeatureSelector; var flags: PGPFlags): PGPERROR cdecl;

// Routines to determine which algorithms are present
// To determine if a specific algorithm is available, you will need to
// index through the available algorithms and check the algorithm ID.
var PGPCountPublicKeyAlgorithms: function(var numPKAlgs: PGPUInt32): PGPError cdecl;
*)
function PGPGetIndexedPublicKeyAlgorithmInfo(index: PGPUInt32; var info: TPGPPublicKeyAlgorithmInfo): PGPError;

(*
var PGPCountSymmetricCiphers: function(var numPKAlgs: PGPUInt32): PGPError cdecl;
var PGPGetIndexedSymmetricCipherInfo: function(index: PGPUInt32;
                                 var info: TPGPSymmetricCipherInfo): PGPError cdecl;

// call this if you want the SDK to use prefs stored in the prefs file

var PGPsdkLoadPrefs: function(context: pPGPContext; prefSpec: pPGPFileSpec): PGPError cdecl;

var PGPsdkLoadDefaultPrefs: function(context: pPGPContext): PGPError cdecl;

// save any changed preferences to disk
var PGPsdkSavePrefs: function(context: pPGPContext): PGPError cdecl;

var PGPsdkPrefSetFileSpec: function(context: pPGPContext; selector: PGPsdkPrefSelector;
                                    ref: pPGPFileSpec): PGPError cdecl;

// caller must deallocate outRef with PGPFreeFileSpec
var PGPsdkPrefGetFileSpec: function(context: pPGPContext; selector: PGPsdkPrefSelector;
                                    var outRef: pPGPFileSpec): PGPError cdecl;

var PGPsdkPrefSetData: function(context: pPGPContext; selector: PGPsdkPrefSelector;
                                data: Pointer; size: PGPSize): PGPError cdecl;

// caller must deallocate*dataOut with PGPFreeData
var PGPsdkPrefGetData: function(context: pPGPContext; selector: PGPsdkPrefSelector;
                                var dataOut: Pointer; var sizeOut: PGPSize): PGPError cdecl;
*)

{------------------------------------------------------------------------------
  2. Option List Routines
 ------------------------------------------------------------------------------}

// ** The following functions are used to create pPGPOptionsList's for
// ** specifying the various options to PGPEncode() and PGPEncode(). The
// ** functions can be used as inline parameters in a temporary manner or
// ** used with PGPBuildOptionList() to create persistent lists.
// ** Special pPGPOptionsList to mark last option passed to those functions
// ** which take variable lists of pPGPOptionsList's:

function PGPNewOptionList(context: pPGPContext; var outList: pPGPOptionsList): PGPERROR;
//var PGPCopyOptionList: function(optionList: pPGPOptionsList; var outList: pPGPOptionsList): PGPERROR cdecl;
function PGPFreeOptionList(optionList: pPGPOptionsList): PGPERROR;

// Delphi: You don't generally need to use this
function PGPOLastOption(context: pPGPContext): pPGPOptionsList; cdecl;

// ** Special pPGPOptionsList which is always ignored
function PGPONullOption(context: pPGPContext): pPGPOptionsList; cdecl;

{------------------------------------------------------------------------------
  3. Keyring Navigation & Maintenance
 ------------------------------------------------------------------------------}

//var PGPIncFilterRefCount: function(filter: pPGPFilter): PGPERROR cdecl;

function PGPFreeFilter(filter: pPGPFilter): PGPERROR;

(*
var PGPNewKeyIDFilter: function(context: pPGPContext;
                                var keyID: TPGPKeyID;
                                var outFilter: pPGPFilter): PGPERROR cdecl;


var PGPNewSubKeyIDFilter: function(context: pPGPContext;
                                   var  ubKeyID: TPGPKeyID;
                                   var outFilter: pPGPFilter): PGPERROR cdecl;


var PGPNewKeyEncryptAlgorithmFilter: function(context: pPGPContext;
                                              encryptAlgorithm: PGPPUBLICKEYALGORITHM;
                                              var outFilter: pPGPFilter): PGPERROR cdecl;


var PGPNewKeyFingerPrintFilter: function(context: pPGPContext;
                                         fingerPrint: Pointer;
                                         fingerPrintLength: PGPSIZE;
                                         var outFilter: pPGPFilter): PGPERROR cdecl;


var PGPNewKeyCreationTimeFilter: function(context: pPGPContext;
                                          creationTime: PGPTIME;
                                          match: PGPMATCHCRITERION;
                                          var outFilter: pPGPFilter): PGPERROR cdecl;


var PGPNewKeyExpirationTimeFilter: function(context: pPGPContext;
                                            expirationTime: PGPTIME;
                                            match: PGPMATCHCRITERION;
                                            var outFilter: pPGPFilter): PGPERROR cdecl;


var PGPNewKeyRevokedFilter: function(context: pPGPContext;
                                     revoked: PGPBOOLEAN;
                                     var outFilter: pPGPFilter): PGPERROR cdecl;


var PGPNewKeyDisabledFilter: function(context: pPGPContext;
                                      disabled: PGPBOOLEAN;
                                      var outFilter: pPGPFilter): PGPERROR cdecl;


var PGPNewKeySigAlgorithmFilter: function(context: pPGPContext;
                                          sigAlgorithm: PGPPUBLICKEYALGORITHM;
                                          var outFilter: pPGPFilter): PGPERROR cdecl;


var PGPNewKeyEncryptKeySizeFilter: function(context: pPGPContext;
                                            keySize: PGPUINT32;
                                            match: PGPMATCHCRITERION;
                                            var outFilter: pPGPFilter): PGPERROR cdecl;


var PGPNewKeySigKeySizeFilter: function(context: pPGPContext;
                                        keySize: PGPUINT32;
                                        match: PGPMATCHCRITERION;
                                        var outFilter: pPGPFilter): PGPERROR cdecl;

*)

function PGPNewUserIDStringFilter(context: pPGPContext;
                                  userIDString: PChar;
                                  match: PGPMATCHCRITERION;
                                  var outFilter: pPGPFilter): PGPERROR;

(*
var PGPNewUserIDEmailFilter: function(context: pPGPContext;
                                      emailString: PChar;
                                      match: PGPMATCHCRITERION;
                                      var outFilter: pPGPFilter): PGPERROR cdecl;


var PGPNewUserIDNameFilter: function(context: pPGPContext;
                                     nameString: PChar;
                                     match: PGPMATCHCRITERION;
                                     var outFilter: pPGPFilter): PGPERROR cdecl;


var PGPNewSigKeyIDFilter: function(context: pPGPContext;
                                   var keyID: TPGPKeyID;
                                   var outFilter: pPGPFilter): PGPERROR cdecl;


// freeing outfilter will call PGPFreeFilter on filter
var PGPNegateFilter: function(filter: pPGPFilter;
                              var outFilter: pPGPFilter): PGPERROR cdecl;

// freeing outfilter will call PGPFreeFilter on filter1, filter2
var PGPIntersectFilters: function(filter1: pPGPFilter;
                                  filter2: pPGPFilter;
                                  var outFilter: pPGPFilter): PGPERROR cdecl;

// freeing outfilter will call PGPFreeFilter on filter1, filter2
var PGPUnionFilters: function(filter1: pPGPFilter;
                              filter2: pPGPFilter;
                              var outFilter: pPGPFilter): PGPERROR cdecl;
*)

function PGPFilterKeySet(origSet: pPGPKeySet;
                         filter: pPGPFilter;
                         var resultSet: pPGPKeySet): PGPERROR;

(*
// Keyserver filter functions
var PGPLDAPQueryFromFilter: function(filter: pPGPFilter;
                                     var queryOut: PChar): PGPERROR cdecl;

var PGPHKSQueryFromFilter: function(filter: pPGPFilter;
                                    var queryOut: PChar): PGPERROR cdecl;
*)

//-----------------------
// Key DB management 7.X and later
//-----------------------

function PGPNewKeyDB(Context: pPGPContext; var KeyDB: pPGPKeyDB): PGPError;
{PGPOpenKeyDBFile: function(Context: pPGPContext;
              Options: PGPOpenKeyDBFileOptions;
              PubKeysFileSpec: pPGPFileSpec;
              PrivKeysFileSpec: pPGPFileSpec;
              var KeyDB: pPGPKeyDB): PGPError; cdecl;}
function PGPFreeKeyDB(KeyDB: pPGPKeyDB): PGPError;
//PGPFlushKeyDB: function(KeyDB: pPGPKeyDB): PGPError; cdecl;
//PGPIncKeyDBRefCount: function(KeyDB: pPGPKeyDB): PGPError; cdecl;
//PGPKeyDBIsMutable: function(KeyDB: pPGPKeyDB): PGPBoolean; cdecl;
//PGPCountKeysInKeyDB: function(KeyDB: pPGPKeyDB; var NumKeys: PGPUInt32): PGPError; cdecl;
//PGPKeyDBIsUpdated: function(KeyDB: pPGPKeyDB; var IsUpdated: PGPBoolean): PGPError; cdecl;
// PGP 8.X
//PGPCacheKeyDB: function(KeyDB: pPGPKeyDB; TimeOutSeconds: PGPUInt32): PGPError; cdecl;
//PGPPurgeKeyDBCache: function(Context: pPGPContext): PGPError; cdecl;

// PGP 7.X
function PGPPeekKeySetKeyDB(KeySet: pPGPKeySet): pPGPKeyDB;
function PGPPeekKeyDBRootKeySet(KeyDB: pPGPKeyDB): pPGPKeySet;
// Getting parent objects from key related items
function PGPPeekKeyDBObjKeyDB(KeyDBObj: pPGPKeyDBObj): pPGPKeyDB; cdecl;
function PGPPeekKeyDBObjKey(KeyDBObj: pPGPKeyDBObj): pPGPKeyDBObj; cdecl;
function PGPPeekKeyDBObjUserID(KeyDBObj: pPGPKeyDBObj): pPGPKeyDBObj; cdecl;


function PGPExport(Context: pPGPContext; Options: pPGPOptionsList): PGPError;
function PGPImport(Context: pPGPContext; var ImportedKeys: pPGPKeyDB; Options: pPGPOptionsList): PGPError;
function PGPCopyKeys(KeySet: pPGPKeySet; DestKeyDB: pPGPKeyDB; var DestKeySet: pPGPKeySet): PGPError;

function PGPNewKeyIterFromKeyDB(KeyDB: pPGPKeyDB; var OutRef: pPGPKeyIter): PGPError;
function PGPKeyIterNextKeyDBObj(Iter: pPGPKeyIter; ObjectType: PGPKeyDBObjType; var OutRef: pPGPKeyDBObj): PGPError;

{
// Get property functions
PGPGetKeyDBObjBooleanProperty: function(Obj: pPGPKeyDBObj;
		WhichProperty: PGPKeyDBObjProperty; var Prop: PGPBoolean): PGPError; cdecl;

PGPGetKeyDBObjNumericProperty: function(Obj: pPGPKeyDBObj;
		WhichProperty: PGPKeyDBObjProperty; var Prop: PGPInt32): PGPError; cdecl;

PGPGetKeyDBObjTimeProperty: function(Obj: pPGPKeyDBObj;
		WhichProperty: PGPKeyDBObjProperty; var Prop: PGPTime): PGPError; cdecl;

// Get the data for a binary property. Returns kPGPError_BufferTooSmall if
// the buffer is too small. Both buffer and dataSize can be NULL.
PGPGetKeyDBObjDataProperty: function(Obj: pPGPKeyDBObj;
		WhichProperty: PGPKeyDBObjProperty; Buffer: PChar;
		BufferSize: PGPSize; var DataSize: PGPSize): PGPError; cdecl;
}

//----------------------
// KeySet manipulations
//----------------------

// Creates a new memory-based KeyDB and returns its root set
function PGPNewKeySet(context: pPGPContext; var keySet: pPGPKeySet): PGPERROR;

// Used for building arbitrary subsets of an existing KeyDB
//var PGPNewEmptyKeySet: function(origSet: pPGPKeySet; var newSet: pPGPKeySet): PGPERROR cdecl;

function PGPNewSingletonKeySet(key: pPGPKey; var keySet: pPGPKeySet): PGPERROR;

(*
var PGPUnionKeySets: function(set1: pPGPKeySet;
                              set2: pPGPKeySet;
                              var newSet: pPGPKeySet): PGPERROR cdecl;
*)

function PGPOpenDefaultKeyRings(context: pPGPContext;
  openFlags: PGPKEYRINGOPENFLAGS; var keySet: pPGPKeySet): PGPERROR;

(*
var PGPOpenKeyRingPair: function(context: pPGPContext;
                                 openFlags: PGPKEYRINGOPENFLAGS;
                                 pubFileRef: pPGPFileSpec;
                                 secFileRef: pPGPFileSpec;
                                 var keySet: pPGPKeySet): PGPERROR cdecl;

var PGPOpenKeyRing: function(context: pPGPContext;
                             openFlags: PGPKEYRINGOPENFLAGS;
                             fileRef: pPGPFileSpec;
                             var keySet: pPGPKeySet): PGPERROR cdecl;
*)

function PGPCommitKeyRingChanges(keys: pPGPKeySet): PGPERROR;
function PGPPropagateTrust(keys: pPGPKeySet): PGPERROR;

//var PGPRevertKeyRingChanges: function(keys: pPGPKeySet): PGPERROR cdecl;

function PGPCheckKeyRingSigs(keysToCheck: pPGPKeySet;
                             keysSigning: pPGPKeySet;
                             checkAll: PGPBOOLEAN;
                             progress: TPGPEVENTHANDLERPROCPTR;
                             userValue: PGPUSERVALUE): PGPERROR;

//var PGPReloadKeyRings: function(keys: pPGPKeySet): PGPERROR cdecl;

function PGPGetKeyByKeyID(keys: pPGPKeySet; const keyID: TPGPKeyID;
                          //pubKeyAlgorithm: PGPPUBLICKEYALGORITHM;
                          var outRef: pPGPKey): PGPERROR;

//var PGPKeySetIsMember: function(key: pPGPKey; aset: pPGPKeySet): PGPBOOLEAN cdecl;

function PGPCountKeys(keys: pPGPKeySet; var numKeys: PGPUINT32): PGPERROR;

//var PGPIncKeySetRefCount: function(keys: pPGPKeySet): PGPERROR cdecl;

function PGPFreeKeySet(keys: pPGPKeySet): PGPERROR;

//var PGPKeySetIsMutable: function(keys: pPGPKeySet): PGPBOOLEAN cdecl;
//var PGPKeySetNeedsCommit: function(keys: pPGPKeySet): PGPBOOLEAN cdecl;

function PGPAddKey(keyToAdd: pPGPKeyDBObj; aset: pPGPKeySet): PGPERROR; // ver 7.x+
function PGPAddKeys(keysToAdd: pPGPKeySet; aset: pPGPKeySet): PGPERROR;
function PGPRemoveKeys(keysToRemove: pPGPKeySet; aset: pPGPKeySet): PGPERROR;


// Key manipulation functions

//var PGPDisableKey: function(key: pPGPKey): PGPERROR cdecl;
//var PGPEnableKey: function(key: pPGPKey): PGPERROR cdecl;

function PGPChangePassphrase(key: pPGPKey; Args: array of pPGPOptionsList): PGPERROR;
function PGPChangeSubKeyPassphrase(subkey: pPGPSubKey; Args: array of pPGPOptionsList): PGPERROR;
//var PGPRemoveSubKey: function(subkey: pPGPSubKey): PGPERROR cdecl;
function PGPRemoveUserID(userID: pPGPUserID): PGPERROR;
function PGPSetPrimaryUserID(userid: pPGPUserID; Args: array of pPGPOptionsList): PGPERROR;

(*
var PGPGetSigCertifierKey: function(cert: pPGPSig;
                                    allkeys: pPGPKeySet;
                                    var certkey: pPGPKey): PGPERROR cdecl;

var PGPRemoveSig: function(cert: pPGPSig): PGPERROR cdecl;
*)

function PGPCountAdditionalRecipientRequests(basekey: pPGPKey;
  var numARKeys: PGPUINT32): PGPERROR;

function PGPGetIndexedAdditionalRecipientRequest(basekey: pPGPKey;
  allkeys: pPGPKeySet;
  nth: PGPUINT32;
  var arkey: pPGPKey;
  var arclass: PGPBYTE): PGPERROR;

function PGPGetIndexedAdditionalRecipientRequestKeyID(basekey: pPGPKey;
  nth: PGPUINT32;
  var arkeyid: TPGPKeyID;
  var arclass: PGPBYTE): PGPERROR;

//function PGPPassphraseIsValid(key: pPGPKey; passphrase: PChar): PGPBOOLEAN;
function PGPPassphraseIsValid(key: pPGPKey; Args: array of pPGPOptionsList): PGPError;

// Trust values for PGPSetKeyTrust and kPGPKeyPropTrust property: }
// kPGPKeyTrust_Undefined (do not pass to PGPSetKeyTrust) }
// kPGPKeyTrust_Unknown (unknown) }
// kPGPKeyTrust_Never (never) }
// kPGPKeyTrust_Marginal (sometimes) }
// kPGPKeyTrust_Complete (always) }
// kPGPKeyTrust_Ultimate (do not pass to PGPSetKeyTrust) }

// old trust model
function PGPSetKeyTrust(key: pPGPKey; trust: PGPUINT32): PGPERROR;
//function PGPSetKeyAxiomatic(key: pPGPKey; checkPassphrase: PGPBOOLEAN; passphrase: String): PGPERROR;
function PGPSetKeyAxiomatic(key: pPGPKey; Args: array of pPGPOptionsList): PGPERROR;
function PGPUnsetKeyAxiomatic(key: pPGPKey): PGPERROR;

// Get property functions

function PGPGetKeyBoolean(key: pPGPKey; propname: PGPKEYPROPNAME; var prop: PGPBOOLEAN): PGPERROR;
function PGPGetKeyNumber(key: pPGPKey; propname: PGPKEYPROPNAME; var prop: PGPINT32): PGPERROR;

// 'buffer' is NOT null-terminated
// returns kPGPError_BufferTooSmall if buffer is too small, but fill buffer to capacity
function PGPGetKeyPropertyBuffer(key: pPGPKey; propname: PGPKEYPROPNAME;
                                 bufferSize: PGPSIZE; data: Pointer;
                                 var fullSize: PGPSIZE): PGPERROR;

function PGPGetKeyTime(key: pPGPKey; propname: PGPKEYPROPNAME; var prop: PGPTIME): PGPERROR;
function PGPGetSubKeyBoolean(subkey: pPGPSubKey; propname: PGPKEYPROPNAME; var prop: PGPBOOLEAN): PGPERROR;
function PGPGetSubKeyNumber(subkey: pPGPSubKey; propname: PGPKEYPROPNAME; var prop: PGPINT32): PGPERROR;

// returns kPGPError_BufferTooSmall if buffer is too small, but fill buffer to capacity
function PGPGetSubKeyPropertyBuffer(subkey: pPGPSubKey;
                                    propname: PGPKEYPROPNAME;
                                    bufferSize: PGPSIZE;
                                    prop: Pointer;
                                    var fullSize: PGPSIZE): PGPERROR;

function PGPGetSubKeyTime(subkey: pPGPSubKey; propname: PGPKEYPROPNAME; var prop: PGPTIME): PGPERROR;

(*
var PGPGetUserIDNumber: function(userID: pPGPUserID;
                                 propname: PGPUSERIDPROPNAME;
                                 var prop: PGPINT32): PGPERROR cdecl;
*)

// 'string' is always NULL-terminated
// returns kPGPError_BufferTooSmall if buffer is too small, but fill buffer to capacity
function PGPGetUserIDStringBuffer(userID: pPGPUserID; propname: PGPUSERIDPROPNAME;
                                  bufferSize: PGPSIZE; astring: PChar;
                                  var fullSize: PGPSIZE): PGPERROR;

(*
var PGPGetSigBoolean: function(cert: pPGPSig;
                               propname: PGPSIGPROPNAME;
                               var prop: PGPBOOLEAN): PGPERROR cdecl;

var PGPGetSigNumber: function(cert: pPGPSig;
                              propname: PGPSIGPROPNAME;
                              var prop: PGPINT32): PGPERROR cdecl;

var PGPGetKeyIDOfCertifier: function(sig: pPGPSig;
                                     var outID: TPGPKeyID): PGPERROR cdecl;

var PGPGetSigTime: function(cert: pPGPSig;
                            propname: PGPSIGPROPNAME;
                            var prop: PGPTIME): PGPERROR cdecl;

var PGPGetHashAlgUsed: function(key: pPGPKey;
                                var hashAlg: PGPHASHALGORITHM): PGPERROR cdecl;

*)

// Convenience property functions

function PGPGetPrimaryUserID(key: pPGPKey; var outRef: pPGPUserID): PGPERROR;

// 'string' is always a C string and fullSize includes the '\0' terminator
// returns kPGPError_BufferTooSmall if buffer is too small, but fill buffer to capacity

function PGPGetPrimaryUserIDNameBuffer(key: pPGPKey; bufferSize: PGPSIZE;
  astring: PChar; var fullSize: PGPSIZE): PGPERROR;

//var PGPGetPrimaryUserIDValidity: function(key: pPGPKey; var validity: PGPVALIDITY): PGPERROR cdecl;

function PGPCompareKeys(a: pPGPKey; b: pPGPKey; order: PGPKEYORDERING): PGPINT32;

//var PGPCompareUserIDStrings: function(a: PChar; b: PChar): PGPINT32 cdecl;

function PGPOrderKeySet(src: pPGPKeySet;
                        order: PGPKEYORDERING;
                        var outRef: pPGPKeyList): PGPERROR;

//var PGPIncKeyListRefCount: function(keys: pPGPKeyList): PGPERROR cdecl;
function PGPFreeKeyList(keys: pPGPKeyList): PGPERROR;

// Key iteration functions

function PGPNewKeyIter(keys: pPGPKeyList; var outRef: pPGPKeyIter): PGPERROR;

//var PGPCopyKeyIter: function(orig: pPGPKeyIter; var outRef: pPGPKeyIter): PGPERROR cdecl;

function PGPFreeKeyIter(iter: pPGPKeyIter): PGPERROR;


//var PGPKeyIterIndex: function(iter: pPGPKeyIter): PGPINT32 cdecl;

//var PGPKeyIterRewind: function(iter: pPGPKeyIter): PGPERROR cdecl;

//var PGPKeyIterSeek: function(iter: pPGPKeyIter; var key: pPGPKey): PGPINT32 cdecl;

//var PGPKeyIterMove: function(iter: pPGPKeyIter; relOffset: PGPINT32; var outRef: pPGPKey): PGPERROR cdecl;

function PGPKeyIterNext(iter: pPGPKeyIter; var outRef: pPGPKey): PGPERROR;

//var PGPKeyIterPrev: function(iter: pPGPKeyIter; var outRef: pPGPKey): PGPERROR cdecl;

//var PGPKeyIterKey: function(iter: pPGPKeyIter; var outRef: pPGPKey): PGPERROR cdecl;

function PGPKeyIterSubKey(iter: pPGPKeyIter; var outRef: pPGPSubKey): PGPERROR;

function PGPKeyIterUserID(iter: pPGPKeyIter; var outRef: pPGPUserID): PGPERROR;

//var PGPKeyIterSig: function(iter: pPGPKeyIter; var outRef: pPGPSig): PGPERROR cdecl;

function PGPKeyIterNextSubKey(iter: pPGPKeyIter; var outRef: pPGPSubKey): PGPERROR;

//var PGPKeyIterPrevSubKey: function(iter: pPGPKeyIter; var outRef: pPGPSubKey): PGPERROR cdecl;

//var PGPKeyIterRewindSubKey: function(iter: pPGPKeyIter): PGPERROR cdecl;

function PGPKeyIterNextUserID(iter: pPGPKeyIter; var outRef: pPGPUserID): PGPERROR;

//var PGPKeyIterPrevUserID: function(iter: pPGPKeyIter; var outRef: pPGPUserID): PGPERROR cdecl;

//var PGPKeyIterRewindUserID: function(iter: pPGPKeyIter): PGPERROR cdecl;

//var PGPKeyIterNextUIDSig: function(iter: pPGPKeyIter; var outRef: pPGPSig): PGPERROR cdecl;

//var PGPKeyIterPrevUIDSig: function(iter: pPGPKeyIter; var outRef: pPGPSig): PGPERROR cdecl;

//var PGPKeyIterRewindUIDSig: function(iter: pPGPKeyIter): PGPERROR cdecl;


// Get/set default private key

//var PGPGetDefaultPrivateKey: function(keyset: pPGPKeySet; var outRef: pPGPKey): PGPERROR cdecl;

//var PGPSetDefaultPrivateKey: function(key: pPGPKey): PGPERROR cdecl;


// Get/set user value

//var PGPSetKeyUserVal: function(key: pPGPKey; userValue: PGPUSERVALUE): PGPERROR cdecl;

//var PGPSetUserIDUserVal: function(userid: pPGPUserID; userValue: PGPUSERVALUE): PGPERROR cdecl;

//var PGPSetSubKeyUserVal: function(subkey: pPGPSubKey; userValue: PGPUSERVALUE): PGPERROR cdecl;

//var PGPSetSigUserVal: function(cert: pPGPSig; userValue: PGPUSERVALUE): PGPERROR cdecl;

//var PGPGetKeyUserVal: function(key: pPGPKey; var userValue: PGPUSERVALUE): PGPERROR cdecl;

//var PGPGetUserIDUserVal: function(userid: pPGPUserID; var userValue: PGPUSERVALUE): PGPERROR cdecl;

//var PGPGetSubKeyUserVal: function(subkey: pPGPSubKey; var userValue: PGPUSERVALUE): PGPERROR cdecl;

//var PGPGetSigUserVal: function(cert: pPGPSig; var userValue: PGPUSERVALUE): PGPERROR cdecl;


//Key IDs

function PGPGetKeyIDFromString(astring: String; var id: TPGPKeyID): PGPERROR;
function PGPGetKeyIDFromKey(key: pPGPKey; var id: TPGPKeyID): PGPERROR;
function PGPGetKeyIDFromSubKey(key: pPGPSubKey; var id: TPGPKeyID): PGPERROR;

type
  PGPChar = array [0..KPGPMaxKeyIDStringSize] of Char;

function PGPGetKeyIDString(var ref: TPGPKeyID;
                           atype: PGPKEYIDSTRINGTYPE; outstring: PGPChar): PGPERROR;

(*
// outputs opaque string of bytes for storage of maximum size as keyID
// do NOT attempt to parse the output; it is opaque to you
const
  kPGPMaxExportedKeyIDSize = ( sizeof( TPGPKeyID ) );
type
  PGPByteArray = array [0..kPGPMaxExportedKeyIDSize] of char;

var PGPExportKeyID: function(var keyID: TPGPKeyID;
                             EXPORTEDDATA:PGPByteArray;
                             var exportedLength: PGPSIZE): PGPERROR cdecl;

// must be in format output by PGPExportKeyID
var PGPImportKeyID: function(data: Pointer; var id: TPGPKeyID): PGPERROR cdecl;
*)

// returns 0 if equal, -1 if key1 < key2, 1 if key1 > key2
function PGPCompareKeyIDs(var key, key2: TPGPKeyID): PGPINT32;

(*
// Getting contexts back from key related items. If the key is invalid, you get kPGPInvalidRef back.

var PGPGetKeyListContext: function(ref: pPGPKeyList): pPGPContext cdecl;
var PGPGetKeySetContext:  function(ref: pPGPKeySet): pPGPContext cdecl;
var PGPGetKeyIterContext: function(ref: pPGPKeyIter): pPGPContext cdecl;
var PGPGetKeyContext:     function(ref: pPGPKey): pPGPContext cdecl;
var PGPGetSubKeyContext:  function(ref: pPGPSubKey): pPGPContext cdecl;
var PGPGetUserIDContext:  function(ref: pPGPUserID): pPGPContext cdecl;

{------------------------------------------------------------------------------
  4. Misc Encoding/Decoding Routines
 ------------------------------------------------------------------------------}

// Return a context for public-key operations based on the specified key.
// The specified message format is used for all operations with this
// context.
var PGPNewPublicKeyContext: function(publicKeyRef: pPGPKey;
                                     messageFormat: PGPPublicKeyMessageFormat;
                                     var outRef: pPGPPublicKeyContext): PGPERROR; cdecl;

// Dispose of a public-key context.
var PGPFreePublicKeyContext: function(ref: pPGPPublicKeyContext): PGPERROR; cdecl;

// Determine maximum sizes for inputs and outputs.
var PGPGetPublicKeyOperationSizes: function(ref: pPGPPublicKeyContext;
                                            var maxDecryptedBufferSize: PGPSIZE;
                                            var maxEncryptedBufferSize: PGPSIZE;
                                            var maxSignatureSize: PGPSIZE): PGPERROR; cdecl;

// Encrypt one block of data, using PKCS-1 padding. Output buffer must
// be of size maxEncryptedBufferSize from PGPGetPublicKeyEncryptionSize. (=PGPGetPublicKeyOperationSizes GG.)
// outSize is a return parameter. For some formatting modes the actual
// output size may be less than the maximum possible.
var PGPPublicKeyEncrypt: function(ref: pPGPPublicKeyContext;
                                  data: Pointer;
                                  inSize: PGPSIZE;
                                  dataout: Pointer;
                                  var outSize: PGPSIZE): PGPERROR; cdecl;

// Verify a signature on a message hash. Returns kPGPError_NoErr on
// correct verification, else an error code. The message hash is
// finalized by this call (and should not have been finalized prior
// to the call).
var PGPPublicKeyVerifySignature: function(ref: pPGPPublicKeyContext;
                                          hashContext: pPGPHashContext;
                                          signature: Pointer;
                                          signatureSize: PGPSIZE): PGPERROR; cdecl;

// Return a context for private-key operations based on the specified
// key (which must have a private part). The specified message
// format is used for all operations with this context. Unlocks key
// data using passphrase.
var PGPNewPrivateKeyContext: function(privateKeyRef: pPGPKey;
                                      messageFormat: PGPPublicKeyMessageFormat;
                                      passphrase: PChar;
                                      var outRef: pPGPPrivateKeyContext): PGPERROR; cdecl;

// Dispose of a private-key context. All sensitive data is wiped before
// being deleted.
var PGPFreePrivateKeyContext: function(ref: pPGPPrivateKeyContext): PGPERROR; cdecl;

// Determine maximum sizes for inputs and outputs.
var PGPGetPrivateKeyOperationSizes: function(ref: pPGPPrivateKeyContext;
                                             var maxDecryptedBufferSize: PGPSIZE;
                                             var maxEncryptedBufferSize: PGPSIZE;
                                             var maxSignatureSize: PGPSIZE): PGPERROR; cdecl;

// Decrypt one block of data. Output buffer must be of size at least
// maxDecryptedBufferSize from PGPGetPrivateKeyDecryptionSize.
// outSize is a return parameter. For some formatting modes the actual
// output size may be less than the maximum possible.
var PGPPrivateKeyDecrypt: function(ref: pPGPPrivateKeyContext;
                                   datain: Pointer;
                                   inSize: PGPSIZE;
                                   dataout: Pointer;
                                   var outSize: PGPSIZE): PGPERROR; cdecl;

// Sign a message hash. Output signature buffer must be of size at
// least maxSignatureSize from PGPGetPrivateKeyDecryptionSize.
// signatureSize is a return parameter. For some formatting modes
// the actual signature size may be less than the maximum possible.
// The message hash is finalized by this call (and should not have been
// finalized prior to the call).
var PGPPrivateKeySign: function(ref: pPGPPrivateKeyContext;
                                hashContext: pPGPHashContext;
                                signature: Pointer;
                                var signatureSize: PGPSIZE): PGPERROR; cdecl;
*)

function PGPNewMemoryMgr(reserved: PGPFlags; var mgr: pPGPMemoryMgr): PGPError;
function PGPFreeMemoryMgr(mgr: pPGPMemoryMgr): PGPError;

//---------- Symmetric Cipher -----------------------------------------------

// Create a new cipher of the specified algorithm. Cannot be used until
// PGPSetSymmetricCipherKey() has been called. }
// If the algorithm is not available then kPGPError_AlgorithmNotAvailable is }
// returned.

// Existing algorithms have only one key size. Values:
// kPGPCipherAlgorithm_CAST5 128 / 8 = 16
// kPGPCipherAlgorithm_3DES 192 / 8 = 24
// kPGPCipherAlgorithm_IDEA 128 / 8 = 16
// In the future symmetric ciphers could be added that have different key
// sizes for the same algorithm.
function PGPNewSymmetricCipherContext(pMemoryMgr: pPGPMemoryMgr;
                                      algorithm: PGPCIPHERALGORITHM; keySizeInBytes: PGPSIZE;
                                      var outRef: pPGPSymmetricCipherContext): PGPERROR;

// Disposal clears all data in memory before releasing it. }
function PGPFreeSymmetricCipherContext(ref: pPGPSymmetricCipherContext): PGPERROR;

(*
// Make an exact copy of the cipher, including the key. }
var PGPCopySymmetricCipherContext: function(ref: pPGPSymmetricCipherContext;
                                            var outRef: pPGPSymmetricCipherContext): PGPERROR cdecl;

// The key must be set before using; a cipher can be repeatedly reset and
// reused with different keys to avoid having to create and destroy new
// contexts each time (and it's also cryptographically better not to reuse
// a key).

// kKey size is implicit based on algorithm. 'key' is*copied*. Caller
// may want to destroy the original after passing it in.
*)
function PGPInitSymmetricCipher(ref: pPGPSymmetricCipherContext; key: Pointer): PGPERROR;

(*
// Wipe any sensitive data in the cipher. Cipher remains alive, but
//  key must be set before any data is encrypted.
var PGPWipeSymmetricCipher: function(ref: pPGPSymmetricCipherContext): PGPERROR cdecl;

// "Wash" the symmetric cipher
var PGPWashSymmetricCipher: function(ref: pPGPSymmetricCipherContext;
                                     buf: Pointer;
                                     len: PGPSIZE): PGPERROR cdecl;
*)

// Encrypt or decrypt one "block" of data. The block size is determined
// by the cipher (see PGPGetSymmetricCipherBlockSize()).
function PGPSymmetricCipherEncrypt(ref: pPGPSymmetricCipherContext;
                                   datain, dataout: Pointer): PGPERROR;

function PGPSymmetricCipherDecrypt(ref: pPGPSymmetricCipherContext;
                                   datain, dataout: Pointer): PGPERROR;

// Determine key and block size for specified algorithm. Stateless routine
// does not need a context.
function PGPGetSymmetricCipherSizes(ref: pPGPSymmetricCipherContext;
                                         var keySize, blockSize: PGPSIZE): PGPERROR;

// A CFB context requires use of a symmetric cipher which has been created
// and whose key has been set. An error will be returned if this is not
// the case.
// After the call, the CFBRef "owns" the symmetric ref and will
// dispose of it properly (even if an error occurs).
// The caller should no longer reference it.
function PGPNewCFBContext(ref: pPGPSymmetricCipherContext; interleaveFactor: PGPUINT16;
                          var outRef: pPGPCFBContext): PGPERROR;

// Disposal clears all data in memory before releasing it.
function PGPFreeCFBContext(ref: pPGPCFBContext): PGPERROR;

{// Make an exact copy, including current state. Original is not changed.
var PGPCopyCFBContext: function(ref: pPGPCFBContext; var outRef: pPGPCFBContext): PGPERROR;}

// IV size is implicit (same size as the symmetric cipher block size).
// IV is*copied*.
// Caller may want to destroy the original after passing it in.
// Calling this implicitly calls PGPResetCFB().
function PGPInitCFB(ref: pPGPCFBContext; key: Pointer; initializationVector: Pointer): PGPERROR;

// Call repeatedly to process arbitrary amounts of data. }
function PGPCFBEncrypt(ref: pPGPCFBContext; datain: Pointer; bytesIn: PGPSIZE; dataout: Pointer): PGPERROR;
function PGPCFBDecrypt(ref: pPGPCFBContext; datain: Pointer; bytesIn: PGPSIZE; dataout: Pointer): PGPERROR;

(*
// Get the symmetric cipher being used for this CFB context.
// You can use this to determine useful things about the underlying cipher
// such as its block size.
var PGPCFBGetSymmetricCipher: function(ref: pPGPCFBContext;
                                       var outRef: pPGPSymmetricCipherContext): PGPERROR cdecl;

// Reset the feedback mechanism to use whatever we have so far, plus previous
// bytes for a total of the cipher block size bytes. This effectively
// changes the cipher block boundary.
var PGPCFBSync: function(ref: pPGPCFBContext): PGPERROR cdecl;

// Fetch random bytes from the cipher. Returns the actual number of
// random bytes obtained.
var PGPCFBGetRandom: function(ref: pPGPCFBContext;
                              requestCount: PGPSIZE;
                              dataout: Pointer;
                              var outCount: PGPSIZE): PGPERROR cdecl;

// Make more random bytes available using the supplied salt, which must
// be the same as the symmetric cipher block size.
var PGPCFBRandomCycle: function(ref: pPGPCFBContext;
                                salt: Pointer): PGPERROR cdecl;

// Make more random bytes available using the supplied salt, which must
// be the same as the symmetric cipher block size.
var PGPCFBRandomWash: function(ref: pPGPCFBContext;
                               datain: Pointer;
                               bytesIn: PGPSIZE): PGPERROR cdecl;
*)

// A CBC context requires use of a symmetric cipher which has been created
// and whose key has been set. An error will be returned if this is not
// the case.
// After the call, the CBCContextRef "owns" the
// symmetric ref and will dispose of it properly (even if an error
// occurs). The caller should no longer reference it.
function PGPNewCBCContext(ref: pPGPSymmetricCipherContext; var outRef: pPGPCBCContext): PGPERROR;

// Disposal clears all data in memory before releasing it. }
function PGPFreeCBCContext(ref: pPGPCBCContext): PGPERROR;

// Make an exact copy, including current state. Original is not changed. }
function PGPCopyCBCContext(ref: pPGPCBCContext; var outRef: pPGPCBCContext): PGPERROR;

// IV size is implicit (same size as the symmetric cipher block size).
// IV is*copied*.
// Caller may want to destroy the original after passing it in.
function PGPInitCBC(ref: pPGPCBCContext; key, initializationVector: Pointer): PGPERROR;

// Call repeatedly to process arbitrary amounts of data. Each call must
// have bytesIn be a multiple of the cipher block size.
function PGPCBCEncrypt(ref: pPGPCBCContext; datain: Pointer; bytesIn: PGPSIZE; dataout: Pointer): PGPERROR;

function PGPCBCDecrypt(ref: pPGPCBCContext; datain: Pointer; bytesIn: PGPSIZE; dataout: Pointer): PGPERROR;

// Get the symmetric cipher being used for this CBC context. }
// You can use this to determine useful things about the underlying cipher }
// such as its block size. }
function PGPCBCGetSymmetricCipher(ref: pPGPCBCContext; var outRef: pPGPSymmetricCipherContext): PGPERROR;

//--------------------- HASH FUNCTIONs ------------------------------------------

// Create a new hash of the specified algorithm.
// If the algorithm is not available then kPGPError_AlgorithmNotAvailable is returned.
function PGPNewHashContext(pMemoryMgr: pPGPMemoryMgr; algID: PGPHashAlgorithm; var outRef: pPGPHashContext): PGPERROR;

// Any existing intermediate hash is lost. }
function PGPFreeHashContext(ref: pPGPHashContext): PGPERROR;

// An exact duplicate of the hash is made. }
//var PGPCopyHashContext: function(ref: pPGPHashContext; var outRef: pPGPHashContext): PGPERROR cdecl;

// Reset a hash as if it had been created anew. Any existing intermediate
// hash is lost. }
function PGPResetHash(ref: pPGPHashContext): PGPERROR;

// Continue the hash, accumulating an intermediate result }
function PGPContinueHash(ref: pPGPHashContext; datain: Pointer; numBytes: PGPSIZE): PGPERROR;

// Finalize the hash, depositing the result into 'hashOut'. }
// After calling this routine, the hash is reset via PGPResetHash(). }
// If you want an intermediate result, use PGPCopyHash() and finalize the }
// copy. }
function PGPFinalizeHash(ref: pPGPHashContext; hashOut: Pointer): PGPERROR;

// Determine size of resulting hash in bytes e.g. a 160 bit hash yields 20. }
// Used for generic code which may not know how big a hash is being produced. }
// Question: can we reasonably assume 8 bits per byte? If not, how does }
// PGPFinalizeHash return its result? }

function PGPGetHashSize(ref: pPGPHashContext; var hashSize: PGPSIZE): PGPERROR;

{------------------------------------------------------------------------------
  5. Encoding / Decoding Wrapper routines and options
 ------------------------------------------------------------------------------}

// ** Functions to encode and decode. The variable parameters are one or more
// ** pPGPOptionsList's which describe the inputs, outputs, and options.
function PGPEncode(context: pPGPContext; Args: array of pPGPOptionsList): PGPError;

function PGPDecode(context: pPGPContext; Args: array of pPGPOptionsList): PGPError;

// Data input (required):
function PGPOInputFile(context: pPGPContext; fileRef: pPGPFileSpec): pPGPOptionsList; cdecl;
function PGPOInputBuffer(context: pPGPContext; buffer: Pointer;
  bufferSize: PGPSIZE): pPGPOptionsList; cdecl;

// Data output (optional, generates event if missing):
function PGPOOutputFile(context: pPGPContext; fileRef: pPGPFileSpec): pPGPOptionsList; cdecl;
function PGPOOutputBuffer(context: pPGPContext; buffer: Pointer; bufferSize: PGPSIZE;
                          var outputDataLength: PGPSIZE): pPGPOptionsList; cdecl;

// '*buffer' must be disposed of via PGPFreeData()
// maximum memory usage will be no more than maximumBufferSize
function PGPOAllocatedOutputBuffer(context: pPGPContext; var buffer: Pointer;
      maximumBufferSize: PGPSIZE; var actualBufferSize: PGPSIZE): pPGPOptionsList; cdecl;

// Encrypting and signing
function PGPOSignWithKey(context: pPGPContext; keyRef: pPGPKey;
                         Args: array of pPGPOptionsList): pPGPOptionsList; cdecl;

function PGPOConventionalEncrypt(context: pPGPContext;
                                 Args: array of pPGPOptionsList): pPGPOptionsList; cdecl;

function PGPODetachedSig(context: pPGPContext;
                         Args: array of pPGPOptionsList): pPGPOptionsList; cdecl;

(*
{
not in pgpsdk.dll??
var PGPOAppendOutput: function(context: pPGPContext;
                               appendOutput: PGPBOOLEAN): pPGPOptionsList cdecl;
var PGPOAskUserForOutput: function(context: pPGPContext;
                                   askUserForOutput: PGPBOOLEAN): pPGPOptionsList cdecl;
}
var PGPODiscardOutput: function(context: pPGPContext;
                                discardOutput: PGPBOOLEAN): pPGPOptionsList cdecl;

var PGPOEncryptToKey: function(context: pPGPContext;
                               keyRef: pPGPKey): pPGPOptionsList cdecl;
*)

function PGPOEncryptToKeySet(context: pPGPContext;
                             keySetRef: pPGPKeySet): pPGPOptionsList; cdecl;

//var PGPOEncryptToUserID: function(context: pPGPContext; userIDRef: pPGPUserID): pPGPOptionsList cdecl;

function PGPOPassphraseBuffer(context: pPGPContext; passphrase: Pointer; passphraseLength: PGPSIZE): pPGPOptionsList cdecl;
function PGPOPassphrase(context: pPGPContext; passphrase: PChar): pPGPOptionsList; cdecl;

function PGPOAskUserForEntropy(context: pPGPContext; askUserForEntropy: PGPBOOLEAN): pPGPOptionsList cdecl;

//var PGPORawPGPInput: function(context: pPGPContext; rawPGPInput: PGPBOOLEAN): pPGPOptionsList cdecl;

function PGPOCompression(context: pPGPContext; compression: PGPBOOLEAN): pPGPOptionsList; cdecl;

(*
var PGPOLocalEncoding: function(context: pPGPContext;
                                localEncode: TPGPLOCALENCODINGFLAGS): pPGPOptionsList cdecl;

var PGPOOutputLineEndType: function(context: pPGPContext;
                                    lineEnd: PGPLINEENDTYPE): pPGPOptionsList cdecl;

type TMimeSepArray = array [0..kPGPMimeSeparatorSize-1] of char;

var PGPOPGPMIMEEncoding: function(context: pPGPContext;
                                  mimeEncoding: PGPBOOLEAN;
                                  var mimeBodyOffset: PGPSIZE;
                                  mimeSeperator: TMimeSepArray): pPGPOptionsList cdecl;

var PGPOOmitMIMEVersion: function(context: pPGPContext;
                                  omitVersion: PGPBOOLEAN): pPGPOptionsList cdecl;

var PGPOCipherAlgorithm: function(context: pPGPContext;
                                  algorithm: PGPCIPHERALGORITHM): pPGPOptionsList cdecl;

var PGPOHashAlgorithm: function(context: pPGPContext;
                                algorithm: PGPHASHALGORITHM): pPGPOptionsList cdecl;

var PGPOFailBelowValidity: function(context: pPGPContext;
                                    minValidity: PGPVALIDITY): pPGPOptionsList cdecl;

var PGPOWarnBelowValidity: function(context: pPGPContext;
                                    minValidity: PGPVALIDITY): pPGPOptionsList cdecl;
*)

function PGPOEventHandler(context: pPGPContext; handler: TPGPEVENTHANDLERPROCPTR;
                          userValue: PGPUSERVALUE): pPGPOptionsList; cdecl;

function PGPOSendNullEvents(context: pPGPContext;
                            approxInterval: PGPTIMEINTERVAL): pPGPOptionsList cdecl;

function PGPOArmorOutput(context: pPGPContext; armorOutput: PGPBOOLEAN): pPGPOptionsList; cdecl;

{
var PGPODataIsASCII: function(context: pPGPContext;
                              dataIsASCII: PGPBOOLEAN): pPGPOptionsList cdecl;
}
function PGPOClearSign(context: pPGPContext; clearSign: PGPBOOLEAN): pPGPOptionsList;

{
var PGPOForYourEyesOnly: function(context: pPGPContext;
                                  forYourEyesOnly: PGPBOOLEAN): pPGPOptionsList cdecl;
}


// ver 7.x+
function PGPOKeyDBRef(Context: pPGPContext; KeyDBRef: pPGPKeyDBRef): pPGPOptionsList;



function PGPOKeySetRef(context: pPGPContext; keysetRef: pPGPKeySet): pPGPOptionsList;

function PGPOImportKeysTo(context: pPGPContext; keysetRef: pPGPKeySet): pPGPOptionsList;

(*
var PGPOSendEventIfKeyFound: function(context: pPGPContext;
                                      sendEventIfKeyFound: PGPBOOLEAN): pPGPOptionsList cdecl;

var PGPOPassThroughIfUnrecognized: function(context: pPGPContext;
                                            passThroughIfUnrecognized: PGPBOOLEAN): pPGPOptionsList cdecl;
*)

function PGPOKeyGenParams(context: pPGPContext; pubKeyAlg: PGPPUBLICKEYALGORITHM; bits: PGPUINT32): pPGPOptionsList; cdecl;
function PGPOKeyGenName(context: pPGPContext; name: Pointer; nameLength: PGPSIZE): pPGPOptionsList; cdecl;

function PGPOExpiration(context: pPGPContext; expirationDays: PGPUINT32): pPGPOptionsList; cdecl;

(*
var PGPOAdditionalRecipientRequestKeySet: function(context: pPGPContext;
                                                   arKeySetRef: pPGPKeySet;
                                                   arkClass: PGPBYTE): pPGPOptionsList cdecl;

*)
function PGPOKeyGenMasterKey(context: pPGPContext; masterKeyRef: pPGPKey): pPGPOptionsList cdecl;

function PGPOPreferredAlgorithms(context: pPGPContext; var prefAlg: PGPCIPHERALGORITHM;
                                 numAlgs: PGPUINT32): pPGPOptionsList; cdecl;

function PGPOKeyGenFast(context: pPGPContext; fastGen: PGPBOOLEAN): pPGPOptionsList; cdecl;


function PGPOCommentString(context: pPGPContext; comment: PChar): pPGPOptionsList; cdecl;

function PGPOVersionString(context: pPGPContext; version: PChar): pPGPOptionsList;

(*
var PGPOSigRegularExpression: function(context: pPGPContext;
                                       regularExpression: PChar): pPGPOptionsList cdecl;

var PGPOExportFormat: function(context: pPGPContext; exportable: PGPBOOLEAN): pPGPOptionsList cdecl;
*)

function PGPOExportKeySet(Context: pPGPContext; KeySet: pPGPKeySet): pPGPOptionsList;

function PGPOExportPrivateKeys(context: pPGPContext; exportPrivate: PGPBOOLEAN): pPGPOptionsList cdecl;
(*
var PGPOExportable: function(context: pPGPContext;
                             exportable: PGPBOOLEAN): pPGPOptionsList cdecl;

var PGPOSigTrust: function(context: pPGPContext;
                           trustLevel: PGPUINT32;
                           trustValue: PGPUINT32): pPGPOptionsList cdecl;
*)

function PGPAddJobOptions(job: pPGPJob; Args: array of pPGPOptionsList): PGPError;

function PGPAppendOptionList(context: pPGPContext; optionList: pPGPOptionsList;
  Args: array of pPGPOptionsList): PGPError;

function PGPBuildOptionList(context: pPGPContext; var outList: pPGPOptionsList;
  Args: array of pPGPOptionsList): PGPError;

function PGPGenerateKey(Context:pPGPContext; var Key: pPGPKey; Options: pPGPOptionsList): PGPError;
//function PGPGenerateKey(Context:pPGPContext; var Key: pPGPKey; Args: array of pPGPOptionsList): PGPError;

function PGPGenerateSubKey(Context: pPGPContext; var Subkey: pPGPSubkey; Options: pPGPOptionsList): PGPError;
//function PGPGenerateSubKey(Context: pPGPContext; var subkey: pPGPSubkey; Args: array of pPGPOptionsList): PGPError;

function PGPGetKeyEntropyNeeded(context: pPGPContext; Args: array of pPGPOptionsList): PGPError;

function PGPExportKeySet(keys: pPGPKeySet; Args: array of pPGPOptionsList): PGPError;

function PGPImportKeySet(context: pPGPContext; var keys: pPGPKeyset; Args: array of pPGPOptionsList): PGPError;

function PGPSignUserID(userID: pPGPUserID; certifyingKey: pPGPKey; Args: array of pPGPOptionsList): PGPError;

function PGPAddUserID(key: pPGPKey; const userID: PChar; Args: array of pPGPOptionsList): PGPError;

function PGPRevokeSig(cert: pPGPSig; allkeys: pPGPKeySet; Args: array of pPGPOptionsList): PGPError;

function PGPRevokeKey(key: pPGPKey; Args: array of pPGPOptionsList): PGPError;

function PGPRevokeSubKey(subkey: pPGPSubKey; Args: array of pPGPOptionsList): PGPError;


{------------------------------------------------------------------------------
  6. Key Server Support routines
 ------------------------------------------------------------------------------}

(*
var PGPKeyServerInit: function: PGPERROR cdecl;

var PGPKeyServerCleanup: function: PGPERROR cdecl;

var PGPKeyServerOpen: function(keyServer: pPGPKeyServer): PGPERROR cdecl;

var PGPKeyServerClose: function(keyServer: pPGPKeyServer): PGPERROR cdecl;

var PGPNewKeyServerFromURL: function(thecontext: pPGPContext;
                                     Const url: PChar;
                                     accessType: PGPKeyServerAccessType;
                                     keySpace: PGPKeyServerKeySpace;
                                     keyServer: pPGPKeyServer): PGPERROR cdecl;

var PGPFreeKeyServer: function(keyServer: pPGPKeyServer): PGPERROR cdecl;

var PGPQueryKeyServer: function(keyServer: pPGPKeyServer;
                                filter: pPGPFilter;
                                callBack: TPGPEventHandlerProcPtr;
                                callBackArg: PGPUserValue;
                                var resultSet: pPGPKeySet;
                                var resultInfo:PGPFlags): PGPERROR cdecl;

var PGPUploadToKeyServer: function(keyServer: pPGPKeyServer;
                                   keysToUpload: pPGPKeySet;
                                   callBack: TPGPEventHandlerProcPtr;
                                   callBackArg: PGPUserValue;
                                   var keysThatFailed: pPGPKeySet): PGPERROR cdecl;

var PGPDeleteFromKeyServer: function(keyServer: pPGPKeyServer;
                                     keysToDelete: pPGPKeySet;
                                     callBack: TPGPEventHandlerProcPtr;
                                     callBackArg: PGPUserValue;
                                     var keysThatFailed: pPGPKeySet): PGPERROR cdecl;

var PGPDisableFromKeyServer: function(keyServer: pPGPKeyServer;
                                      keysToDisable: pPGPKeySet;
                                      callBack: TPGPEventHandlerProcPtr;
                                      callBackArg: PGPUserValue;
                                      var keysThatFailed: pPGPKeySet): PGPERROR cdecl;

var PGPGetKeyServerErrorString: function(keyServer: pPGPKeyServer;
                                         var errorString:PChar): PGPERROR cdecl;

var PGPLDAPNewServerMonitor: function(keyServer: pPGPKeyServer;
                                      callBack: TPGPEventHandlerProcPtr;
                                      callBackArg: PGPUserValue;
                                      var monitor: pPGPKeyServerMonitor): PGPERROR cdecl;

var PGPLDAPFreeServerMonitor: function(keyServer: pPGPKeyServer;
                                       monitor: pPGPKeyServerMonitor): PGPERROR cdecl;
*)
{------------------------------------------------------------------------------
  7. Support Routines
 ------------------------------------------------------------------------------}

procedure PGPCheckResult(Error:PGPError; const PrependMsg:string);
function PGPErrorMsg(Error:PGPError):string;
function PGPDescribeEvent(etype:integer):string;


implementation

uses
  SysUtils;

var
  IsSDK171  : Boolean = True; // Default to SDK 1.7.x (from PGP 6.5.x)
  DLLHandle : HModule = 0;
  PGPDLLPath: String = '';

function GetPGPDLLPath: String;
begin
  result := PGPDLLPath;
end;

function IsOldSDK: Boolean;
begin
  result := IsSDK171;
end;

function IsLoadedDLL: Boolean;
var
  sz: array [0..MAX_PATH-1] of Char;
begin
  if DLLHandle = 0 then
  begin
    DLLHandle := LoadLibrary('PGP_SDK.DLL'); // try to load SDK 1.7.x

    if DLLHandle = 0 then                    // there's no 1.7.x, try 2.x+
    begin
      IsSDK171  := False;
      DLLHandle := LoadLibrary('PGPSDK.DLL');
    end;

    if DLLHandle <> 0 then
    begin
      FillChar(sz, SizeOf(sz), 0);
      if GetModuleFilename(DLLHandle, sz, SizeOf(sz)) <> 0 then;
        PGPDLLPath := ExtractFilePath(sz);
    end;
  end;

  result := (DLLHandle <> 0);
end; // IsLoadedDLL


function PGPsdkInit: PGPERROR;
var
  fn: function: PGPERROR cdecl;
begin
  if not IsLoadedDLL then
    result := kPGPError_FirstError
  else
    begin
      @fn := GetProcAddress(DLLHandle, 'PGPsdkInit');
      if Assigned(fn) then
        result := fn
      else
        result := kPGPError_FirstError;
    end;
end; // PGPsdkInit

function PGPsdkCleanup: PGPERROR;
var
  fn: function: PGPERROR cdecl;
begin
  @fn := GetProcAddress(DLLHandle, 'PGPsdkCleanup');
  if Assigned(fn) then
    result := fn
  else
    result := kPGPError_FirstError;

  if DLLHandle <> 0 then
  begin
    FreeLibrary(DLLHandle);
    DLLHandle := 0;
  end;
end; // PGPsdkCleanup



function PGPNewContext(clientAPIVersion: PGPUINT32; var newContext: pPGPContext): PGPERROR;
const
  fn: function(clientAPIVersion: PGPUINT32; var newContext: pPGPContext): PGPERROR cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPNewContext');

  if Assigned(fn) then
    result := fn(clientAPIVersion, newContext)
  else
    result := kPGPError_FirstError;
end; // PGPNewContext


function PGPFreeContext(context: pPGPContext): PGPERROR;
const
  fn: function(context: pPGPContext): PGPERROR cdecl = nil;
begin
  if context = nil then
  begin
    result := kPGPError_NoError;
    Exit;
  end;

  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPFreeContext');

  if Assigned(fn) then
    result := fn(context)
  else
    result := kPGPError_FirstError;
end; // PGPFreeContext


function PGPFreeData(allocation: Pointer): PGPERROR;
const
  fn: function(allocation: Pointer): PGPERROR cdecl = nil;
begin
  if allocation = nil then
  begin
    result := kPGPError_NoError;
    Exit;
  end;

  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPFreeData');

  if Assigned(fn) then
    result := fn(allocation)
  else
    result := kPGPError_FirstError;
end; // PGPFreeData



function PGPFreeKeySet(keys: pPGPKeySet): PGPERROR;
const
  fn: function(keys: pPGPKeySet): PGPERROR cdecl = nil;
begin
  if keys = nil then
  begin
    result := kPGPError_NoError;
    Exit;
  end;

  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPFreeKeySet');

  if Assigned(fn) then
    result := fn(keys)
  else
    result := kPGPError_FirstError;
end; // PGPFreeKeySet


function PGPFreeKeyList(keys: pPGPKeyList): PGPERROR;
const
  fn: function(keys: pPGPKeyList): PGPERROR cdecl = nil;
begin
  if keys = nil then
  begin
    result := kPGPError_NoError;
    Exit;
  end;

  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPFreeKeyList');

  if Assigned(fn) then
    result := fn(keys)
  else
    result := kPGPError_FirstError;
end; // PGPFreeKeyList


// PGP 7.X only
type
  { PGPKeyDBFileOptions }
  PGPOpenKeyDBFileOptions = PGPEnumType;
const
  kPGPOpenKeyDBFileOptions_None				= 0;
  kPGPOpenKeyDBFileOptions_Mutable			= (1 shl 0);
  kPGPOpenKeyDBFileOptions_Create			= (1 shl 1);

function PGPOpenKeyDBFile(Context: pPGPContext;
		Options: PGPOpenKeyDBFileOptions;
		PubKeysFileSpec, PrivKeysFileSpec: pPGPFileSpec;
		var KeyDB: pPGPKeyDB): PGPError;
const
  fn: function(Context: pPGPContext;
		Options: PGPOpenKeyDBFileOptions;
		PubKeysFileSpec, PrivKeysFileSpec: pPGPFileSpec;
		var KeyDB: pPGPKeyDB): PGPError; cdecl = nil;
begin
  @fn := GetProcAddress(DLLHandle, 'PGPOpenKeyDBFile');
  if Assigned(fn) then
    result := fn(context, Options, PubKeysFileSpec, PrivKeysFileSpec, keyDB)
  else
    result := kPGPError_FirstError;
end; // PGPOpenKeyDBFile


function PGPOpenDefaultKeyRings(context: pPGPContext;
  openFlags: PGPKEYRINGOPENFLAGS; var keySet: pPGPKeySet): PGPERROR;
const
  fn: function(context: pPGPContext; openFlags: PGPKEYRINGOPENFLAGS;
               var keySet: pPGPKeySet): PGPERROR cdecl = nil;
begin
  if IsSDK171 then
    begin
      @fn := GetProcAddress(DLLHandle, 'PGPOpenDefaultKeyRings');
      if Assigned(fn) then
        result := fn(context, openFlags, keySet)
      else
        result := kPGPError_FirstError;
    end
  else
    begin
      result := PGPOpenKeyDBFile(context
      , kPGPOpenKeyDBFileOptions_Mutable
      , nil
      , nil
      , keySet // it's KeyDB here
      );
    end;
end; // PGPOpenDefaultKeyRings


function PGPCompareKeys(a: pPGPKey; b: pPGPKey; order: PGPKEYORDERING): PGPINT32;
const
  fn: function(a: pPGPKey; b: pPGPKey; order: PGPKEYORDERING): PGPINT32 cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPCompareKeys');

  if Assigned(fn) then
    result := fn(a, b, order)
  else
    result := kPGPError_FirstError;
end; // PGPCompareKeys


function PGPOrderKeySet(src: pPGPKeySet; order: PGPKEYORDERING; var outRef: pPGPKeyList): PGPERROR;
const
  fn: function(src: pPGPKeySet; order: PGPKEYORDERING;
               var outRef: pPGPKeyList): PGPERROR cdecl = nil;
  fn7: function(src: pPGPKeySet; order: PGPKEYORDERING; reverseOrder: PGPBoolean;
                var outRef: pPGPKeyList): PGPERROR cdecl = nil;
begin
  result := kPGPError_FirstError;

  if IsSDK171 then
    begin
      if not Assigned(fn) then
        @fn := GetProcAddress(DLLHandle, 'PGPOrderKeySet');
      if Assigned(fn) then
        result := fn(src, order, outRef);
    end
  else
    begin
      if not Assigned(fn7) then
        @fn7 := GetProcAddress(DLLHandle, 'PGPOrderKeySet');
      if Assigned(fn7) then
        result := fn7(src, order, 0, outRef);
    end;
end; // PGPOrderKeySet


function PGPNewKeyIter(keys: pPGPKeyList; var outRef: pPGPKeyIter): PGPERROR;
const
  fn: function(keys: pPGPKeyList; var outRef: pPGPKeyIter): PGPERROR cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPNewKeyIter');
    
  if Assigned(fn) then
    result := fn(keys, outRef)
  else
    result := kPGPError_FirstError;
end; // PGPNewKeyIter

function PGPFreeKeyIter(iter: pPGPKeyIter): PGPERROR;
const
  fn: function(iter: pPGPKeyIter): PGPERROR cdecl = nil;
begin
  if iter = nil then
  begin
    result := kPGPError_NoError;
    Exit;
  end;

  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPFreeKeyIter');

  if Assigned(fn) then
    result := fn(iter)
  else
    result := kPGPError_FirstError;
end; // PGPFreeKeyIter


function PGPKeyIterNext(iter: pPGPKeyIter; var outRef: pPGPKey): PGPERROR;
const
  fn: function(iter: pPGPKeyIter; var outRef: pPGPKey): PGPERROR cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPKeyIterNext');
    
  if Assigned(fn) then
    result := fn(iter, outRef)
  else
    result := kPGPError_FirstError;
end; // PGPKeyIterNext


function PGPKeyIterSubKey(iter: pPGPKeyIter; var outRef: pPGPSubKey): PGPERROR;
const
  fn: function(iter: pPGPKeyIter; var outRef: pPGPSubKey): PGPERROR cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPKeyIterSubKey');
    
  if Assigned(fn) then
    result := fn(iter, outRef)
  else
    result := kPGPError_FirstError;
end; // PGPKeyIterSubKey

function PGPKeyIterUserID(iter: pPGPKeyIter; var outRef: pPGPUserID): PGPERROR;
const
  fn: function(iter: pPGPKeyIter; var outRef: pPGPUserID): PGPERROR cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPKeyIterUserID');
    
  if Assigned(fn) then
    result := fn(iter, outRef)
  else
    result := kPGPError_FirstError;
end; // PGPKeyIterUserID


function PGPKeyIterNextSubKey(iter: pPGPKeyIter; var outRef: pPGPSubKey): PGPERROR;
const
  fn: function(iter: pPGPKeyIter; var outRef: pPGPSubKey): PGPERROR cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPKeyIterNextSubKey');
    
  if Assigned(fn) then
    result := fn(iter, outRef)
  else
    result := kPGPError_FirstError;
end; // PGPKeyIterNextSubKey


function PGPKeyIterNextUserID(iter: pPGPKeyIter; var outRef: pPGPUserID): PGPERROR;
const
  fn: function(iter: pPGPKeyIter; var outRef: pPGPUserID): PGPERROR cdecl = nil;
begin
  result := kPGPError_FirstError;

  if IsOldSDK then
    begin
      if not Assigned(fn) then
        @fn := GetProcAddress(DLLHandle, 'PGPKeyIterNextUserID');

      if Assigned(fn) then
        result := fn(iter, outRef);
    end
  else
    result := PGPKeyIterNextKeyDBObj(Iter, kPGPKeyDBObjType_UserID, outRef);
end; // PGPKeyIterNextUserID


function PGPGetKeyIDFromString(astring: String; var id: TPGPKeyID): PGPERROR;
const
  fn: function(astring: PChar; var id: TPGPKeyID): PGPERROR cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPGetKeyIDFromString');
    
  if Assigned(fn) then
    result := fn(PChar(astring), id)
  else
    result := kPGPError_FirstError;
end; // PGPGetKeyIDFromString


function PGPGetKeyIDFromKey(key: pPGPKey; var id: TPGPKeyID): PGPERROR;
const
  fn: function(key: pPGPKey; var id: TPGPKeyID): PGPERROR cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPGetKeyIDFromKey');
  if Assigned(fn) then
    result := fn(key, id)
  else
    result := kPGPError_FirstError;
end; // PGPGetKeyIDFromKey


function PGPGetKeyIDFromSubKey(key: pPGPSubKey; var id: TPGPKeyID): PGPERROR;
const
  fn: function(key: pPGPSubKey; var id: TPGPKeyID): PGPERROR cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPGetKeyIDFromSubKey');
  if Assigned(fn) then
    result := fn(key, id)
  else
    result := kPGPError_FirstError;
end; // PGPGetKeyIDFromSubKey


function PGPGetKeyIDString(var ref: TPGPKeyID;
                           atype: PGPKeyIDStringType; outstring: PGPChar): PGPERROR;
const
  fn: function(var ref: TPGPKeyID;
               atype: PGPKEYIDSTRINGTYPE; outstring: PGPChar): PGPERROR cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPGetKeyIDString');
  if Assigned(fn) then
    result := fn(ref, atype, outstring)
  else
    result := kPGPError_FirstError;
end; // PGPGetKeyIDString


function PGPCompareKeyIDs(var key, key2: TPGPKeyID): PGPINT32;
const
  fn: function (var key, key2: TPGPKeyID): PGPINT32; cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPCompareKeyIDs');
  if Assigned(fn) then
    result := fn(key, key2)
  else
    result := kPGPError_FirstError;
end; // PGPCompareKeyIDs


function PGPGetPrimaryUserID(key: pPGPKey; var outRef: pPGPUserID): PGPERROR;
const
  fn: function(key: pPGPKey; var outRef: pPGPUserID): PGPERROR cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPGetPrimaryUserID');
  if Assigned(fn) then
    result := fn(key, outRef)
  else
    result := kPGPError_FirstError;
end; // PGPGetPrimaryUserID


function PGPGetPrimaryUserIDNameBuffer(key: pPGPKey; bufferSize: PGPSIZE;
  astring: PChar; var fullSize: PGPSIZE): PGPERROR;
const
  fn: function(key: pPGPKey; bufferSize: PGPSIZE;
        astring: PChar; var fullSize: PGPSIZE): PGPERROR cdecl = nil;
  fn7: function(key: pPGPKey; astring: PChar;
        bufferSize: PGPSIZE; var fullSize: PGPSIZE): PGPERROR cdecl = nil;
begin
  if IsOldSdk then
    begin
      if not Assigned(fn) then
        @fn := GetProcAddress(DLLHandle, 'PGPGetPrimaryUserIDNameBuffer');
      if Assigned(fn) then
        result := fn(key, bufferSize, astring, fullSize)
      else
        result := kPGPError_FirstError;
    end
  else
    begin
      if not Assigned(fn7) then
        @fn7 := GetProcAddress(DLLHandle, 'PGPGetPrimaryUserIDName');
      if Assigned(fn7) then
        result := fn7(key, astring, bufferSize, fullSize)
      else
        result := kPGPError_FirstError;
    end;
end; // PGPGetPrimaryUserIDNameBuffer


//----------------------------
// Key DB for 7.x and later
//----------------------------

function PGPNewKeyDB(Context: pPGPContext; var KeyDB: pPGPKeyDB): PGPError;
const
  fn: function(Context: pPGPContext; var KeyDB: pPGPKeyDB): PGPError; cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPNewKeyDB');
  if Assigned(fn) then
    result := fn(Context, KeyDB)
  else
    result := kPGPError_FirstError;
end; //

function PGPFreeKeyDB(KeyDB: pPGPKeyDB): PGPError;
const
  fn: function(KeyDB: pPGPKeyDB): PGPError; cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPFreeKeyDB');
  if Assigned(fn) then
    result := fn(KeyDB)
  else
    result := kPGPError_FirstError;
end; // PGPFreeKeyDB

function PGPPeekKeySetKeyDB(KeySet: pPGPKeySet): pPGPKeyDB;
const
  fn: function(KeySet: pPGPKeySet): pPGPKeyDB; cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPPeekKeySetKeyDB');
  if Assigned(fn) then
    result := fn(KeySet)
  else
    result := nil;
end; // PGPPeekKeySetKeyDB

function PGPPeekKeyDBRootKeySet(KeyDB: pPGPKeyDB): pPGPKeySet;
const
  fn: function(KeyDB: pPGPKeyDB): pPGPKeySet; cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPPeekKeyDBRootKeySet');
  if Assigned(fn) then
    result := fn(KeyDB)
  else
    result := nil;
end; // PGPPeekKeyDBRootKeySet

function PGPPeekKeyDBObjKeyDB(KeyDBObj: pPGPKeyDBObj): pPGPKeyDB; cdecl;
const
  fn: function(KeyDBObj: pPGPKeyDBObj): pPGPKeyDB; cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPPeekKeyDBObjKeyDB');
  if Assigned(fn) then
    result := fn(KeyDBObj)
  else
    result := nil;
end; // PGPPeekKeyDBObjKeyDB

function PGPPeekKeyDBObjKey(KeyDBObj: pPGPKeyDBObj): pPGPKeyDBObj; cdecl;
const
  fn: function(KeyDBObj: pPGPKeyDBObj): pPGPKeyDBObj; cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPPeekKeyDBObjKey');
  if Assigned(fn) then
    result := fn(KeyDBObj)
  else
    result := nil;
end; // PGPPeekKeyDBObjKey

function PGPPeekKeyDBObjUserID(KeyDBObj: pPGPKeyDBObj): pPGPKeyDBObj; cdecl;
const
  fn: function(KeyDBObj: pPGPKeyDBObj): pPGPKeyDBObj; cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPPeekKeyDBObjUserID');
  if Assigned(fn) then
    result := fn(KeyDBObj)
  else
    result := nil;
end; // PGPPeekKeyDBObjUserID


function PGPExport(Context: pPGPContext; Options: pPGPOptionsList): PGPError;
const
  fn: function(Context: pPGPContext; Options, LastOption: pPGPOptionsList): PGPError; cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPExport');
  if Assigned(fn) then
    result := fn(Context, Options, PGPOLastOption(Context))
  else
    result := kPGPError_FirstError;
end; // PGPExport

function PGPImport(Context: pPGPContext; var ImportedKeys: pPGPKeyDB; Options: pPGPOptionsList): PGPError;
const
  fn: function(Context: pPGPContext; var ImportedKeys: pPGPKeyDB; Options, LastOption: pPGPOptionsList): PGPError; cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPImport');
  if Assigned(fn) then
    result := fn(Context, ImportedKeys, Options, PGPOLastOption(Context))
  else
    result := kPGPError_FirstError;
end; // PGPImport

function PGPCopyKeys(KeySet: pPGPKeySet; DestKeyDB: pPGPKeyDB; var DestKeySet: pPGPKeySet): PGPError;
const
  fn: function(KeySet: pPGPKeySet; DestKeyDB: pPGPKeyDB; var DestKeySet: pPGPKeySet): PGPError; cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPCopyKeys');
  if Assigned(fn) then
    result := fn(KeySet, DestKeyDB, DestKeySet)
  else
    result := kPGPError_FirstError;
end; // PGPCopyKeys


function PGPNewKeyIterFromKeyDB(KeyDB: pPGPKeyDB; var OutRef: pPGPKeyIter): PGPError;
const
  fn: function(KeyDB: pPGPKeyDB; var OutRef: pPGPKeyIter): PGPError; cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPNewKeyIterFromKeyDB');
  if Assigned(fn) then
    result := fn(KeyDB, OutRef)
  else
    result := kPGPError_FirstError;
end; // PGPNewKeyIterFromKeyDB

function PGPKeyIterNextKeyDBObj(Iter: pPGPKeyIter; ObjectType: PGPKeyDBObjType; var OutRef: pPGPKeyDBObj): PGPError;
const
  fn: function(Iter: pPGPKeyIter; ObjectType: PGPKeyDBObjType; var OutRef: pPGPKeyDBObj): PGPError; cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPKeyIterNextKeyDBObj');
  if Assigned(fn) then
    result := fn(Iter, ObjectType, OutRef)
  else
    result := kPGPError_FirstError;
end; // PGPKeyIterNextKeyDBObj

// Creates a new memory-based KeyDB and returns it's root set
function PGPNewKeySet(context: pPGPContext; var keySet: pPGPKeySet): PGPERROR;
const
  fn: function(context: pPGPContext; var keySet: pPGPKeySet): PGPERROR cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPNewKeySet');
  if Assigned(fn) then
    result := fn(context, keySet)
  else
    result := kPGPError_FirstError;
end; // PGPNewKeySet


function PGPNewSingletonKeySet(key: pPGPKey; var keySet: pPGPKeySet): PGPERROR;
const
  fn: function(key: pPGPKey; var keySet: pPGPKeySet): PGPERROR cdecl = nil;
  arFnNames: array [Boolean] of PChar = (
    'PGPNewOneKeySet'           // for 7.x+
  , 'PGPNewSingletonKeySet'     // for 6.5.x
  );
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, arFnNames[IsSDK171]);
  if Assigned(fn) then
    result := fn(key, keySet)
  else
    result := kPGPError_FirstError;
end; // PGPNewSingletonKeySet


function PGPNewUserIDStringFilter(context: pPGPContext; userIDString: PChar;
  match: PGPMATCHCRITERION; var outFilter: pPGPFilter): PGPERROR;
const
  fn: function(context: pPGPContext; userIDString: PChar;
        match: PGPMATCHCRITERION; var outFilter: pPGPFilter): PGPERROR cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPNewUserIDStringFilter');
  if Assigned(fn) then
    result := fn(context, userIDString, match, outFilter)
  else
    result := kPGPError_FirstError;
end; // PGPNewUserIDStringFilter

function PGPFreeFilter(filter: pPGPFilter): PGPERROR;
const
  fn: function(filter: pPGPFilter): PGPERROR cdecl = nil;
begin
  if filter = nil then
  begin
    result := kPGPError_NoError;
    Exit;
  end;

  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPFreeFilter');
  if Assigned(fn) then
    result := fn(filter)
  else
    result := kPGPError_FirstError;
end; // PGPFreeFilter


function PGPFilterKeySet(origSet: pPGPKeySet; filter: pPGPFilter;
  var resultSet: pPGPKeySet): PGPERROR;
const
  fn: function(origSet: pPGPKeySet; filter: pPGPFilter; var resultSet: pPGPKeySet): PGPERROR cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPFilterKeySet');
  if Assigned(fn) then
    result := fn(origSet, filter, resultSet)
  else
    result := kPGPError_FirstError;
end; // PGPFilterKeySet


function PGPGetKeyByKeyID(keys: pPGPKeySet; const keyID: TPGPKeyID;
                          //pubKeyAlgorithm: PGPPUBLICKEYALGORITHM;
                          var outRef: pPGPKey): PGPERROR;
const
  fn: function(keys: pPGPKeySet; const keyID: TPGPKeyID;
               pubKeyAlgorithm: PGPPUBLICKEYALGORITHM;
               var outRef: pPGPKey): PGPERROR cdecl = nil;

  fn7: function(keys: pPGPKeySet; const keyID: TPGPKeyID;
               var outRef: pPGPKey): PGPERROR cdecl = nil;
begin
  result := kPGPError_FirstError;

  if IsSDK171 then
    begin
      if not Assigned(fn) then
        @fn := GetProcAddress(DLLHandle, 'PGPGetKeyByKeyID');
      if Assigned(fn) then
        result := fn(keys, KeyID, kPGPPublicKeyAlgorithm_Invalid{pubKeyAlgorithm}, outRef)
    end
  else
    begin
      if not Assigned(fn7) then
        @fn7 := GetProcAddress(DLLHandle, 'PGPFindKeyByKeyID');
      if Assigned(fn) then
        result := fn7(keys, KeyID, outRef)
    end
end; // PGPGetKeyByKeyID


function PGPCountKeys(keys: pPGPKeySet; var numKeys: PGPUINT32): PGPERROR;
const
  fn: function(keys: pPGPKeySet; var numKeys: PGPUINT32): PGPERROR cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPCountKeys');
  if Assigned(fn) then
    result := fn(keys, numKeys)
  else
    result := kPGPError_FirstError;
end; // PGPCountKeys


function PGPCountAdditionalRecipientRequests(basekey: pPGPKey;
  var numARKeys: PGPUINT32): PGPERROR;
const
  fn: function(basekey: pPGPKey; var numARKeys: PGPUINT32): PGPERROR cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPCountAdditionalRecipientRequests');
  if Assigned(fn) then
    result := fn(basekey, numARKeys)
  else
    result := kPGPError_FirstError;
end; // PGPCountAdditionalRecipientRequests


function PGPGetIndexedAdditionalRecipientRequest(basekey: pPGPKey; allkeys: pPGPKeySet;
  nth: PGPUINT32; var arkey: pPGPKey; var arclass: PGPBYTE): PGPERROR;
const
  fn: function(basekey: pPGPKey; allkeys: pPGPKeySet; nth: PGPUINT32;
               var arkey: pPGPKey; var arclass: PGPBYTE): PGPERROR cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPGetIndexedAdditionalRecipientRequest');
  if Assigned(fn) then
    result := fn(basekey, allkeys, nth, arkey, arclass)
  else
    result := kPGPError_FirstError;
end; // PGPGetIndexedAdditionalRecipientRequest


function PGPGetIndexedAdditionalRecipientRequestKeyID(basekey: pPGPKey; nth: PGPUINT32;
  var arkeyid: TPGPKeyID; var arclass: PGPBYTE): PGPERROR;
const
  fn: function(basekey: pPGPKey; nth: PGPUINT32;
               var arkeyid: TPGPKeyID; var arclass: PGPBYTE): PGPERROR cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPGetIndexedAdditionalRecipientRequestKeyID');
  if Assigned(fn) then
    result := fn(basekey, nth, arkeyid, arclass)
  else
    result := kPGPError_FirstError;
end; // PGPGetIndexedAdditionalRecipientRequestKeyID


{function PGPPassphraseIsValid(key: pPGPKey; passphrase: PChar): PGPBOOLEAN;
var
  fn: function(key: pPGPKey; passphrase: PChar): PGPBOOLEAN cdecl;
begin
  @fn := GetProcAddress(DLLHandle, 'PGPPassphraseIsValid');
  if Assigned(fn) then
    result := fn(key, passphrase)
  else
    result := 0;
end;}

var fn_PGPPassphraseIsValid: Pointer = nil;

function PGPPassphraseIsValid(key: pPGPKey; Args: array of pPGPOptionsList): PGPError;
var
  i: integer;
  x: pointer;
begin
  if fn_PGPPassphraseIsValid = nil then
    fn_PGPPassphraseIsValid := GetProcAddress(DLLHandle, 'PGPPassphraseIsValid');

  if not Assigned(fn_PGPPassphraseIsValid) then
  begin
    result := kPGPError_FirstError;
    Exit;
  end;

  x := pointer(-1);
  asm
    push x;
  end;

  for i := high(args) downto low(args) do
  begin
    x := Args[i];
    asm
      push x;
    end;
  end;

  asm
    push key;
    call fn_PGPPassphraseIsValid;
    mov result,eax;
    pop key;
  end;

  for i := high(args) + 1 downto low(args) do
  asm
    pop x;
  end;

  {asm
    pop x; // -1
  end;}
end; // PGPPassphraseIsValid


function PGPSetKeyTrust(key: pPGPKey; trust: PGPUINT32): PGPERROR;
const
  fn: function(key: pPGPKey; trust: PGPUINT32): PGPERROR cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPSetKeyTrust');
  if Assigned(fn) then
    result := fn(key, trust)
  else
    result := kPGPError_FirstError;
end; // PGPSetKeyTrust

var fn_PGPSetKeyAxiomatic: Pointer = nil;

function PGPSetKeyAxiomatic(key: pPGPKey; Args: array of pPGPOptionsList): PGPERROR;
var
  i: integer;
  x: pointer;
begin
  if not Assigned(fn_PGPSetKeyAxiomatic) then
    fn_PGPSetKeyAxiomatic := GetProcAddress(DLLHandle, 'PGPSetKeyAxiomatic');

  if not Assigned(fn_PGPSetKeyAxiomatic) then
  begin
    result := kPGPError_FirstError;
    Exit;
  end;

  x := pointer(-1);
  asm
    push x;
  end;

  for i := high(args) downto low(args) do
  begin
    x := Args[i];
    asm
      push x;
    end;
  end;

  asm
    push key;
    call fn_PGPSetKeyAxiomatic;
    mov result,eax;
    pop key;
  end;

  for i := high(args) + 1 downto low(args) do
  asm
    pop x;
  end;

  {asm
    pop x; // -1
  end;}
end; // PGPSetKeyAxiomatic

function PGPUnsetKeyAxiomatic(key: pPGPKey): PGPERROR;
const
  fn: function(key: pPGPKey): PGPERROR cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPUnsetKeyAxiomatic');
  if Assigned(fn) then
    result := fn(key)
  else
    result := kPGPError_FirstError;
end; // PGPUnsetKeyAxiomatic


function PGPGetKeyBoolean(key: pPGPKey; propname: PGPKEYPROPNAME;
  var prop: PGPBOOLEAN): PGPERROR;
const
  fn: function(key: pPGPKey; propname: PGPKEYPROPNAME; var prop: PGPBoolean): PGPERROR cdecl = nil;
  arFnNames: array [Boolean] of PChar = (
    'PGPGetKeyDBObjBooleanProperty' // for 7.x+
  , 'PGPGetKeyBoolean'    // for 6.5.x
  );
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, arFnNames[IsSDK171]);
  if Assigned(fn) then
    result := fn(key, propname, prop)
  else
    result := kPGPError_FirstError;
end; // PGPGetKeyBoolean

function PGPGetKeyNumber(key: pPGPKey; propname: PGPKEYPROPNAME; var prop: PGPINT32): PGPERROR;
const
  fn: function(key: pPGPKey; propname: PGPKEYPROPNAME; var prop: PGPINT32): PGPERROR cdecl = nil;
  arFnNames: array [Boolean] of PChar = (
    'PGPGetKeyDBObjNumericProperty' // for 7.x+
  , 'PGPGetKeyNumber'    // for 6.5.x
  );
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, arFnNames[IsSDK171]);
  if Assigned(fn) then
    result := fn(key, propname, prop)
  else
    result := kPGPError_FirstError;
end; // PGPGetKeyNumber

function PGPGetKeyPropertyBuffer(key: pPGPKey; propname: PGPKEYPROPNAME;
                                 bufferSize: PGPSIZE; data: Pointer;
                                 var fullSize: PGPSIZE): PGPERROR;
const
  fn: function(key: pPGPKey; propname: PGPKEYPROPNAME; bufferSize: PGPSIZE; data: Pointer; var fullSize: PGPSIZE): PGPERROR cdecl = nil;
  fn7: function(key: pPGPKey; propname: PGPKEYPROPNAME; data: Pointer; bufferSize: PGPSIZE; var fullSize: PGPSIZE): PGPERROR cdecl = nil;
begin
  result := kPGPError_FirstError;

  if IsOldSdk then
    begin
      if not Assigned(fn) then
        @fn := GetProcAddress(DLLHandle, 'PGPGetKeyPropertyBuffer');
      if Assigned(fn) then
        result := fn(key, propname, bufferSize, data, fullSize)
    end
  else
    begin
      if not Assigned(fn7) then
        @fn7 := GetProcAddress(DLLHandle, 'PGPGetKeyDBObjDataProperty');
      if Assigned(fn7) then
        result := fn7(key, propname, data, bufferSize, fullSize)
    end
end; // PGPGetKeyPropertyBuffer


function PGPGetKeyTime(key: pPGPKey; propname: PGPKEYPROPNAME; var prop: PGPTIME): PGPERROR;
const
  fn: function(key: pPGPKey; propname: PGPKEYPROPNAME; var prop: PGPTIME): PGPERROR cdecl = nil;
  arFnNames: array [Boolean] of PChar = (
    'PGPGetKeyDBObjTimeProperty' // for 7.x+
  , 'PGPGetKeyTime'              // for 6.5.x
  );
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, arFnNames[IsSDK171]);
  if Assigned(fn) then
    result := fn(key, propname, prop)
  else
    result := kPGPError_FirstError;
end; // PGPGetKeyTime


function PGPGetSubKeyBoolean(subkey: pPGPSubKey; propname: PGPKEYPROPNAME; var prop: PGPBOOLEAN): PGPERROR;
const
  fn: function(subkey: pPGPSubKey; propname: PGPKEYPROPNAME; var prop: PGPBOOLEAN): PGPERROR cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPGetSubKeyBoolean');
  if Assigned(fn) then
    result := fn(subkey, propname, prop)
  else
    result := kPGPError_FirstError;
end; // PGPGetSubKeyBoolean


function PGPGetSubKeyNumber(subkey: pPGPSubKey; propname: PGPKEYPROPNAME; var prop: PGPINT32): PGPERROR;
const
  fn: function(subkey: pPGPSubKey; propname: PGPKEYPROPNAME; var prop: PGPINT32): PGPERROR cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPGetSubKeyNumber');
  if Assigned(fn) then
    result := fn(subkey, propname, prop)
  else
    result := kPGPError_FirstError;
end; // PGPGetSubKeyNumber


function PGPGetSubKeyPropertyBuffer(subkey: pPGPSubKey;
                                    propname: PGPKEYPROPNAME;
                                    bufferSize: PGPSIZE;
                                    prop: Pointer;
                                    var fullSize: PGPSIZE): PGPERROR;
const
  fn: function(subkey: pPGPSubKey; propname: PGPKEYPROPNAME; bufferSize: PGPSIZE; prop: Pointer; var fullSize: PGPSIZE): PGPERROR cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPGetSubKeyPropertyBuffer');
  if Assigned(fn) then
    result := fn(subkey, propname, bufferSize, prop, fullSize)
  else
    result := kPGPError_FirstError;
end; // PGPGetSubKeyPropertyBuffer


function PGPGetSubKeyTime(subkey: pPGPSubKey; propname: PGPKEYPROPNAME; var prop: PGPTIME): PGPERROR;
const
  fn: function(subkey: pPGPSubKey; propname: PGPKEYPROPNAME; var prop: PGPTIME): PGPERROR cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPGetSubKeyTime');
  if Assigned(fn) then
    result := fn(subkey, propname, prop)
  else
    result := kPGPError_FirstError;
end; // PGPGetSubKeyTime


function PGPGetUserIDStringBuffer(userID: pPGPUserID; propname: PGPUSERIDPROPNAME;
                                  bufferSize: PGPSIZE; astring: PChar;
                                  var fullSize: PGPSIZE): PGPERROR;
const
  fn: function(userID: pPGPUserID; propname: PGPUSERIDPROPNAME; bufferSize: PGPSIZE; astring: PChar; var fullSize: PGPSIZE): PGPERROR cdecl = nil;
begin
  result := kPGPError_FirstError;

  if IsOldSDK then
    begin
      if not Assigned(fn) then
        @fn := GetProcAddress(DLLHandle, 'PGPGetUserIDStringBuffer');
      if Assigned(fn) then
        result := fn(userID, propname, bufferSize, astring, fullSize);
    end
  else
    result := PGPGetKeyPropertyBuffer(UserID, kPGPUserIDProperty_Name, bufferSize, astring, fullSize);
end; // PGPGetUserIDStringBuffer


function PGPNewFileSpecFromFullPath(context: pPGPContext;
  path: PChar; var ref: pPGPFileSpec): PGPERROR;
const
  fn: function(context: pPGPContext; path: PChar; var ref: pPGPFileSpec): PGPERROR cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPNewFileSpecFromFullPath');
  if Assigned(fn) then
    result := fn(context, path, ref)
  else
    result := kPGPError_FirstError;
end; // PGPNewFileSpecFromFullPath

function PGPFreeFileSpec(fileRef: pPGPFileSpec): PGPERROR;
const
  fn: function(fileRef: pPGPFileSpec): PGPERROR cdecl = nil;
begin
  if fileRef = nil then
  begin
    result := kPGPError_NoError;
    Exit;
  end;

  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPFreeFileSpec');
  if Assigned(fn) then
    result := fn(fileRef)
  else
    result := kPGPError_FirstError;
end; // PGPFreeFileSpec


function PGPGetSDKVersion(var version: PGPUINT32): PGPERROR;
const
  arFnNames: array [Boolean] of PChar = (
    'PGPGetPGPsdkVersion' // for 7.x+
  , 'PGPGetSDKVersion'    // for 6.5.x
  );
var
  fn: function(var version: PGPUINT32): PGPERROR; cdecl;
begin
  @fn := GetProcAddress(DLLHandle, arFnNames[IsSDK171]);
  if Assigned(fn) then
    result := fn(version)
  else
    result := kPGPError_FirstError;
end;

function PGPMajorVersion(v: PGPUINT32): Integer;
begin
  result := (v and $FF000000) shr 24
end;

function PGPMinorVersion(v: PGPUINT32): Integer;
begin
  result := (v and $00FFF000) shr 16
end;

function PGPRevVersion(v: PGPUINT32): Integer;
begin
  result := (v and $00000FF0) shr 4
end;


function PGPGetIndexedPublicKeyAlgorithmInfo(index: PGPUInt32; var info: TPGPPublicKeyAlgorithmInfo): PGPError;
const
  fn: function(index: PGPUInt32; var info: TPGPPublicKeyAlgorithmInfo): PGPError; cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPGetIndexedPublicKeyAlgorithmInfo');

  FillChar(info, SizeOf(info), 0);

  if Assigned(fn) then
    result := fn(index, info)
  else
    result := kPGPError_FirstError;
end;


procedure PGPGetYMDFromPGPTime(theTime: PGPTIME; var year: PGPUINT16; var month: PGPUINT16; var day: PGPUINT16);
const
  fn: procedure(theTime: PGPTIME; var year: PGPUINT16; var month: PGPUINT16; var day: PGPUINT16); cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPGetYMDFromPGPTime');
  if Assigned(fn) then
    fn(theTime, year, month, day)
  else
    begin
      year  := 0;
      month := 0;
      day   := 0;
    end;
end; // PGPGetYMDFromPGPTime


function PGPGlobalRandomPoolAddKeystroke(event: PGPINT32): PGPUINT32;
const
  fn: function (event: PGPINT32): PGPUINT32; cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPGlobalRandomPoolAddKeystroke');
  if Assigned(fn) then
    result := fn(event)
  else
    result := 0;
end;

function PGPGlobalRandomPoolAddMouse(x: PGPUINT32; y: PGPUINT32): PGPUINT32;
const
  fn: function (x: PGPUINT32; y: PGPUINT32): PGPUINT32; cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPGlobalRandomPoolAddMouse');
  if Assigned(fn) then
    result := fn(x, y)
  else
    result := 0;
end;

function PGPGlobalRandomPoolMouseMoved: PGPUInt32;
const
  fn: function: PGPUInt32; cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPGlobalRandomPoolMouseMoved');
  if Assigned(fn) then
    result := fn
  else
    result := 0;
end;


function PGPGlobalRandomPoolGetEntropy: PGPUINT32;
const
  fn: function: PGPUINT32; cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPGlobalRandomPoolGetEntropy');
  if Assigned(fn) then
    result := fn
  else
    result := 0;
end;

function PGPGlobalRandomPoolGetSize: PGPUINT32;
const
  fn: function: PGPUINT32; cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPGlobalRandomPoolGetSize');
  if Assigned(fn) then
    result := fn
  else
    result := 0;
end;

function PGPGlobalRandomPoolGetMinimumEntropy: PGPUINT32;
const
  fn: function: PGPUINT32; cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPGlobalRandomPoolGetMinimumEntropy');
  if Assigned(fn) then
    result := fn
  else
    result := 0;
end;

function PGPGlobalRandomPoolHasMinimumEntropy: PGPBOOLEAN;
const
  fn: function: PGPBOOLEAN; cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPGlobalRandomPoolHasMinimumEntropy');
  if Assigned(fn) then
    result := fn
  else
    result := 0;
end;



function PGPGetErrorString(theError: PGPERROR; bufferSize: PGPSIZE; theString: PChar): PGPERROR;
const
  fn: function(theError: PGPERROR; bufferSize: PGPSIZE; theString: PChar): PGPERROR; cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPGetErrorString');
  if Assigned(fn) then
    result := fn(theError, bufferSize, theString)
  else
    result := kPGPError_FirstError;
end; // PGPGetErrorString


// ver 7.x+
function PGPAddKey(keyToAdd: pPGPKeyDBObj; aset: pPGPKeySet): PGPERROR;
const
  fn: function (keyToAdd: pPGPKeyDBObj; aset: pPGPKeySet): PGPERROR; cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPAddKey');
  if Assigned(fn) then
    result := fn(keyToAdd, aset)
  else
    result := kPGPError_FirstError;
end; // PGPAddKeys


function PGPAddKeys(keysToAdd: pPGPKeySet; aset: pPGPKeySet): PGPERROR cdecl;
const
  fn: function (keysToAdd: pPGPKeySet; aset: pPGPKeySet): PGPERROR; cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPAddKeys');
  if Assigned(fn) then
    result := fn(keysToAdd, aset)
  else
    result := kPGPError_FirstError;
end; // PGPAddKeys


function PGPRemoveKeys(keysToRemove: pPGPKeySet; aset: pPGPKeySet): PGPERROR;
const
  fn: function(keysToRemove: pPGPKeySet; aset: pPGPKeySet): PGPERROR cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPRemoveKeys');
  if Assigned(fn) then
    result := fn(keysToRemove, aset)
  else
    result := kPGPError_FirstError;
end; // PGPRemoveKeys


{function PGPChangePassphrase(key: pPGPKey; const oldphrase, newphrase: String): PGPERROR;
var
  fn: function(key: pPGPKey; oldphrase, newphrase: PChar): PGPERROR cdecl;
begin
  @fn := GetProcAddress(DLLHandle, 'PGPChangePassphrase');
  if Assigned(fn) then
    result := fn(key, PChar(oldphrase), PChar(newphrase))
  else
    result := kPGPError_FirstError;
end; // PGPChangePassphrase}

var fn_PGPChangePassphrase: Pointer = nil;

function PGPChangePassphrase(key: pPGPKey; Args: array of pPGPOptionsList): PGPERROR;
var
  i: integer;
  x: pointer;
begin
  if not Assigned(fn_PGPChangePassphrase) then
    fn_PGPChangePassphrase := GetProcAddress(DLLHandle, 'PGPChangePassphrase');

  if not Assigned(fn_PGPChangePassphrase) then
  begin
    result := kPGPError_FirstError;
    Exit;
  end;

  x := pointer(-1);
  asm
    push x;
  end;

  for i := high(args) downto low(args) do
  begin
    x := Args[i];
    asm
      push x;
    end;
  end;

  asm
    push key;
    call fn_PGPChangePassphrase;
    mov result,eax;
    pop key;
  end;

  for i := high(args) + 1 downto low(args) do
  asm
    pop x;
  end;

  //asm pop x; { -1 } end;
end; // PGPChangePassphrase


var fn_PGPChangeSubKeyPassphrase: Pointer = nil;

function PGPChangeSubKeyPassphrase(subkey: pPGPSubKey; Args: array of pPGPOptionsList): PGPERROR;
var
  i: integer;
  x: pointer;
begin
  if not Assigned(fn_PGPChangeSubKeyPassphrase) then
    fn_PGPChangeSubKeyPassphrase := GetProcAddress(DLLHandle, 'PGPChangeSubKeyPassphrase');

  if not Assigned(fn_PGPChangeSubKeyPassphrase) then
  begin
    result := kPGPError_FirstError;
    Exit;
  end;

  x := pointer(-1);
  asm
    push x;
  end;

  for i := high(args) downto low(args) do
  begin
    x := Args[i];
    asm
      push x;
    end;
  end;

  asm
    push subkey;
    call fn_PGPChangeSubKeyPassphrase;
    mov result,eax;
    pop subkey;
  end;

  for i := high(args) + 1 downto low(args) do
  asm
    pop x;
  end;
end; // PGPChangeSubKeyPassphrase



function PGPRemoveUserID(userID: pPGPUserID): PGPERROR;
const
  fn: function(userID: pPGPUserID): PGPERROR cdecl = nil;
  arFnNames: array [Boolean] of PChar = (
    'PGPDeleteKeyDBObj' // for 7.x+
  , 'PGPRemoveUserID'   // for 6.5.x
  );
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, arFnNames[IsSDK171]);
  if Assigned(fn) then
    result := fn(userID)
  else
    result := kPGPError_FirstError;
end; // PGPRemoveUserID


var
  fn_PGPSetPrimaryUserID7: Pointer = nil;

function PGPSetPrimaryUserID(userID: pPGPUserID; Args: array of pPGPOptionsList): PGPERROR;
const
  fn: function(userID: pPGPUserID): PGPERROR cdecl = nil;
var
  i: integer;
  x: pointer;
begin
  result := kPGPError_FirstError;

  if IsOldSDK then
    begin
      if not Assigned(fn) then
        @fn := GetProcAddress(DLLHandle, 'PGPSetPrimaryUserID');
      if Assigned(fn) then
        result := fn(userID)
    end
  else
    begin
      if not Assigned(fn_PGPSetPrimaryUserID7) then
        fn_PGPSetPrimaryUserID7 := GetProcAddress(DLLHandle, 'PGPSetPrimaryUserID');

      if not Assigned(fn_PGPSetPrimaryUserID7) then
      begin
        result := kPGPError_FirstError;
        Exit;
      end;

      x := pointer(-1);
      asm
        push x;
      end;

      for i := high(args) downto low(args) do
      begin
        x := Args[i];
        asm
          push x;
        end;
      end;

      asm
        push userID;
        call fn_PGPSetPrimaryUserID7;
        mov result,eax;
        pop userID;
      end;

      for i := high(args) + 1 downto low(args) do
      asm
        pop x;
      end;

      //asm pop x; { -1 } end;
    end;
end; // PGPSetPrimaryUserID


function PGPCommitKeyRingChanges(keys: pPGPKeySet): PGPERROR cdecl;
const
  fn: function (keys: pPGPKeySet): PGPERROR cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPCommitKeyRingChanges');
  if Assigned(fn) then
    result := fn(keys)
  else
    result := kPGPError_FirstError;
end; // PGPCommitKeyRingChanges

function PGPPropagateTrust(keys: pPGPKeySet): PGPERROR;
const
  fn: function (keys: pPGPKeySet): PGPERROR cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPPropagateTrust');
  if Assigned(fn) then
    result := fn(keys)
  else
    result := kPGPError_FirstError;
end; // PGPPropagateTrust


function PGPCheckKeyRingSigs(keysToCheck: pPGPKeySet; keysSigning: pPGPKeySet;
                             checkAll: PGPBOOLEAN; progress: TPGPEVENTHANDLERPROCPTR;
                             userValue: PGPUSERVALUE): PGPERROR;
const
  fn: function(keysToCheck: pPGPKeySet; keysSigning: pPGPKeySet; checkAll: PGPBOOLEAN;
               progress: TPGPEVENTHANDLERPROCPTR; userValue: PGPUSERVALUE): PGPERROR cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPCheckKeyRingSigs');
  if Assigned(fn) then
    result := fn(keysToCheck, keysSigning, checkAll, nil, nil)
  else
    result := kPGPError_FirstError;
end; // PGPCheckKeyRingSigs


// Data input (required):
function PGPOInputFile(context: pPGPContext; fileRef: pPGPFileSpec): pPGPOptionsList;
const
  fn: function(context: pPGPContext; fileRef: pPGPFileSpec): pPGPOptionsList cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPOInputFile');
  if Assigned(fn) then
    result := fn(context, fileRef)
  else
    result := nil;
end; // PGPOInputFile

function PGPOInputBuffer(context: pPGPContext; buffer: Pointer;
  bufferSize: PGPSIZE): pPGPOptionsList;
const
  fn: function(context: pPGPContext; buffer: Pointer; bufferSize: PGPSIZE): pPGPOptionsList cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPOInputBuffer');
  if Assigned(fn) then
    result := fn(context, buffer, bufferSize)
  else
    result := nil;
end; // PGPOInputBuffer


// Data output (optional, generates event if missing):
function PGPOOutputFile(context: pPGPContext; fileRef: pPGPFileSpec): pPGPOptionsList;
const
  fn: function(context: pPGPContext; fileRef: pPGPFileSpec): pPGPOptionsList cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPOOutputFile');
  if Assigned(fn) then
    result := fn(context, fileRef)
  else
    result := nil;
end; // PGPOOutputFile


function PGPOOutputBuffer(context: pPGPContext; buffer: Pointer;
  bufferSize: PGPSIZE; var outputDataLength: PGPSIZE): pPGPOptionsList;
const
  fn: function(context: pPGPContext; buffer: Pointer; bufferSize: PGPSIZE; var outputDataLength: PGPSIZE): pPGPOptionsList cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPOOutputBuffer');
  if Assigned(fn) then
    result := fn(context, buffer, bufferSize, outputDataLength)
  else
    result := nil;
end; // PGPOOutputBuffer


// '*buffer' must be disposed of via PGPFreeData()
// maximum memory usage will be no more than maximumBufferSize
function PGPOAllocatedOutputBuffer(context: pPGPContext; var buffer: Pointer;
      maximumBufferSize: PGPSIZE; var actualBufferSize: PGPSIZE): pPGPOptionsList;
const
  fn: function(context: pPGPContext; var buffer: Pointer; maximumBufferSize: PGPSIZE; var actualBufferSize: PGPSIZE): pPGPOptionsList cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPOAllocatedOutputBuffer');
  if Assigned(fn) then
    result := fn(context, buffer, maximumBufferSize, actualBufferSize)
  else
    result := nil;
end; // PGPOAllocatedOutputBuffer


function PGPNewOptionList(context: pPGPContext; var outList: pPGPOptionsList): PGPERROR;
const
  fn: function(context: pPGPContext; var outList: pPGPOptionsList): PGPERROR cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPNewOptionList');
  if Assigned(fn) then
    result := fn(context, outList)
  else
    result := kPGPError_FirstError;
end;

function PGPFreeOptionList(optionList: pPGPOptionsList): PGPERROR;
const
  fn: function(optionList: pPGPOptionsList): PGPERROR cdecl = nil;
begin
  if optionList = nil then
  begin
    result := kPGPError_NoError;
    Exit;
  end;

  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPFreeOptionList');
  if Assigned(fn) then
    result := fn(optionList)
  else
    result := kPGPError_FirstError;
end;

// Delphi: You don't generally need to use this
function PGPOLastOption(context: pPGPContext): pPGPOptionsList; cdecl;
const
  fn: function(context: pPGPContext): pPGPOptionsList cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPOLastOption');
  if Assigned(fn) then
    result := fn(context)
  else
    result := nil;
end; // PGPOLastOption


// ** Special pPGPOptionsList which is always ignored: }
function PGPONullOption(context: pPGPContext): pPGPOptionsList; cdecl;
const
  fn: function(context: pPGPContext): pPGPOptionsList cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPONullOption');
  if Assigned(fn) then
    result := fn(context)
  else
    result := nil;
end;


function PGPOEncryptToKeySet(context: pPGPContext; keySetRef: pPGPKeySet): pPGPOptionsList;
const
  fn: function(context: pPGPContext; keySetRef: pPGPKeySet): pPGPOptionsList cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPOEncryptToKeySet');
  if Assigned(fn) then
    result := fn(context, keySetRef)
  else
    result := nil;
end; // PGPOEncryptToKeySet


function PGPOEventHandler(context: pPGPContext; handler: TPGPEVENTHANDLERPROCPTR;
                          userValue: PGPUSERVALUE): pPGPOptionsList;
const
  fn: function(context: pPGPContext; handler: TPGPEVENTHANDLERPROCPTR; userValue: PGPUSERVALUE): pPGPOptionsList cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPOEventHandler');
  if Assigned(fn) then
    result := fn(context, handler, userValue)
  else
    result := nil;
end; // PGPOEventHandler


function PGPOSendNullEvents(context: pPGPContext;
                            approxInterval: PGPTIMEINTERVAL): pPGPOptionsList cdecl;
const
  fn: function(context: pPGPContext; approxInterval: PGPTIMEINTERVAL): pPGPOptionsList cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPOSendNullEvents');
  if Assigned(fn) then
    result := fn(context, approxInterval)
  else
    result := nil;
end; // PGPOSendNullEvents


function PGPOArmorOutput(context: pPGPContext; armorOutput: PGPBOOLEAN): pPGPOptionsList;
const
  fn: function(context: pPGPContext; armorOutput: PGPBOOLEAN): pPGPOptionsList cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPOArmorOutput');
  if Assigned(fn) then
    result := fn(context, armorOutput)
  else
    result := nil;
end; // PGPOArmorOutput


function PGPOClearSign(context: pPGPContext; clearSign: PGPBOOLEAN): pPGPOptionsList;
const
  fn: function(context: pPGPContext; clearSign: PGPBOOLEAN): pPGPOptionsList cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPOClearSign');
  if Assigned(fn) then
    result := fn(context, clearSign)
  else
    result := nil;
end; // PGPOClearSign



function PGPOKeyDBRef(Context: pPGPContext; KeyDBRef: pPGPKeyDBRef): pPGPOptionsList;
// ver 7.x+
const
  fn: function(context: pPGPContext; KeyDBRef: pPGPKeyDBRef): pPGPOptionsList cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPOKeyDBRef');
  if Assigned(fn) then
    result := fn(Context, KeyDBRef)
  else
    result := nil;
end; // PGPOKeyDBRef


function PGPOKeySetRef(context: pPGPContext; keysetRef: pPGPKeySet): pPGPOptionsList;
const
  fn: function(context: pPGPContext; keysetRef: pPGPKeySet): pPGPOptionsList cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPOKeySetRef');
  if Assigned(fn) then
    result := fn(context, keysetRef)
  else
    result := nil;
end; // PGPOKeySetRef


function PGPOImportKeysTo(context: pPGPContext; keysetRef: pPGPKeySet): pPGPOptionsList;
const
  fn: function(context: pPGPContext; keysetRef: pPGPKeySet): pPGPOptionsList cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPOImportKeysTo');
  if Assigned(fn) then
    result := fn(context, keysetRef)
  else
    result := nil;
end; // PGPOImportKeysTo


function PGPOKeyGenParams(context: pPGPContext;
  pubKeyAlg: PGPPUBLICKEYALGORITHM; bits: PGPUINT32): pPGPOptionsList;
const
  fn: function(context: pPGPContext; pubKeyAlg: PGPPUBLICKEYALGORITHM; bits: PGPUINT32): pPGPOptionsList cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPOKeyGenParams');
  if Assigned(fn) then
    result := fn(context, pubKeyAlg, bits)
  else
    result := nil;
end; // PGPOKeyGenParams

function PGPOKeyGenName(context: pPGPContext; name: Pointer; nameLength: PGPSIZE): pPGPOptionsList;
const
  fn: function(context: pPGPContext; name: Pointer; nameLength: PGPSIZE): pPGPOptionsList cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPOKeyGenName');
  if Assigned(fn) then
    result := fn(context, name, nameLength)
  else
    result := nil;
end; // PGPOKeyGenName


function PGPOExpiration(context: pPGPContext; expirationDays: PGPUINT32): pPGPOptionsList;
const
  fn: function(context: pPGPContext; expirationDays: PGPUINT32): pPGPOptionsList cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPOExpiration');
  if Assigned(fn) then
    result := fn(context, expirationDays)
  else
    result := nil;
end; // PGPOExpiration


function PGPOKeyGenMasterKey(context: pPGPContext; masterKeyRef: pPGPKey): pPGPOptionsList cdecl;
const
  fn: function(context: pPGPContext; masterKeyRef: pPGPKey): pPGPOptionsList cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPOKeyGenMasterKey');
  if Assigned(fn) then
    result := fn(context, masterKeyRef)
  else
    result := nil;
end; // PGPOKeyGenMasterKey


function PGPOPreferredAlgorithms(context: pPGPContext; var prefAlg: PGPCIPHERALGORITHM;
                                 numAlgs: PGPUINT32): pPGPOptionsList;
const
  fn: function(context: pPGPContext; var prefAlg: PGPCIPHERALGORITHM; numAlgs: PGPUINT32): pPGPOptionsList cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPOPreferredAlgorithms');
  if Assigned(fn) then
    result := fn(context, prefAlg, numAlgs)
  else
    result := nil;
end; // PGPOPreferredAlgorithms


function PGPOKeyGenFast(context: pPGPContext; fastGen: PGPBOOLEAN): pPGPOptionsList;
const
  fn: function(context: pPGPContext; fastGen: PGPBOOLEAN): pPGPOptionsList cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPOKeyGenFast');
  if Assigned(fn) then
    result := fn(context, fastGen)
  else
    result := nil;
end; // PGPOKeyGenFast


function PGPOPassphraseBuffer(context: pPGPContext; passphrase: Pointer; passphraseLength: PGPSIZE): pPGPOptionsList;
const
  fn: function(context: pPGPContext; passphrase: Pointer; passphraseLength: PGPSIZE): pPGPOptionsList cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPOPassphraseBuffer');
  if Assigned(fn) then
    result := fn(context, passphrase, passphraseLength)
  else
    result := nil;
end;

function PGPOPassphrase(context: pPGPContext; passphrase: PChar): pPGPOptionsList;
const
  fn: function(context: pPGPContext; passphrase: PChar): pPGPOptionsList cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPOPassphrase');
  if Assigned(fn) then
    result := fn(context, passphrase)
  else
    result := nil;
end; // PGPOPassphrase

function PGPOAskUserForEntropy(context: pPGPContext; askUserForEntropy: PGPBOOLEAN): pPGPOptionsList cdecl;
const
  fn: function(context: pPGPContext; askUserForEntropy: PGPBOOLEAN): pPGPOptionsList cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPOAskUserForEntropy');
  if Assigned(fn) then
    result := fn(context, askUserForEntropy)
  else
    result := nil;
end; // PGPOPassphrase

function PGPOCompression(context: pPGPContext; compression: PGPBOOLEAN): pPGPOptionsList;
const
  fn: function(context: pPGPContext; compression: PGPBOOLEAN): pPGPOptionsList cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPOCompression');
  if Assigned(fn) then
    result := fn(context, compression)
  else
    result := nil;
end; // PGPOCompression

{------------------------------------------------------------------------------
  7. Support Routines
 ------------------------------------------------------------------------------}

function PGPErrorMsg(Error:PGPError):string;
var
  p: PChar;
  i: Integer;
begin
  if (error >= kPGPError_FirstError) and (error <= kPGPError_LastError) then
    begin
      GetMem(p, 1028);
      try
        i := PGPGetErrorString(Error, 1027, p);
        if i <> kPGPError_NoError then
          raise exception.create('Error '+inttostr(i)+' getting PGP error message '+inttostr(Error));
        result := p;
      finally
        FreeMem(p);
      end;
    end
  else
    result := 'Undescribed PGP Error '+inttostr(error);
end;

procedure PGPCheckResult(Error:PGPError; const PrependMsg:string);
begin
  if Error <> kPGPError_NoError then
    raise exception.create(PrependMsg+': '+PGPErrorMsg(Error));
end;


function PGPDescribeEvent(etype:integer):string;
begin
  case etype of
    kPGPEvent_NullEvent              :result := 'Nothing happened';
    kPGPEvent_InitialEvent           :result := 'Initial event';
    kPGPEvent_FinalEvent             :result := 'Final event';
    kPGPEvent_ErrorEvent             :result := 'An error occurred';
    kPGPEvent_WarningEvent           :result := 'Warning event';
    kPGPEvent_EntropyEvent           :result := 'More entropy is needed';
    kPGPEvent_PassphraseEvent        :result := 'A passphrase is needed';
    kPGPEvent_InsertKeyEvent         :result := 'Smart card must be inserted';
    kPGPEvent_AnalyzeEvent           :result := 'Initial analysis event, before any output';
    kPGPEvent_RecipientsEvent        :result := 'Recipient list report, before any output';
    kPGPEvent_KeyFoundEvent          :result := 'Key packet found';
    kPGPEvent_OutputEvent            :result := 'Output specification needed';
    kPGPEvent_SignatureEvent         :result := 'Signature status report';
    kPGPEvent_BeginLexEvent          :result := 'Initial event per lexical unit';
    kPGPEvent_EndLexEvent            :result := 'Final event per lexical unit';
    kPGPEvent_RecursionEvent         :result := 'Notification of recursive job creation';
    kPGPEvent_DetachedSignatureEvent :result := 'Need input for verification of detached signature';
    kPGPEvent_KeyGenEvent            :result := 'Key generation progress';
    kPGPEvent_KeyServerEvent         :result := 'Key Server progress';
    kPGPEvent_KeyServerSignEvent     :result := 'Key Server passphrase';
  else
    result := 'Unknown Event '+inttostr(etype);
  end;
end;



function PGPNewMemoryMgr(reserved: PGPFlags; var mgr: pPGPMemoryMgr): PGPError;
const
  fn: function(reserved: PGPFlags; var mgr: pPGPMemoryMgr): PGPError; cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPNewMemoryMgr');
  if Assigned(fn) then
    result := fn(reserved, mgr)
  else
    result := kPGPError_FirstError;
end;

function PGPFreeMemoryMgr(mgr: pPGPMemoryMgr): PGPError;
const
  fn: function(mgr: pPGPMemoryMgr): PGPError; cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPFreeMemoryMgr');
  if Assigned(fn) then
    result := fn(mgr)
  else
    result := kPGPError_FirstError;
end;



function PGPNewSymmetricCipherContext(pMemoryMgr: pPGPMemoryMgr;
                                      algorithm: PGPCIPHERALGORITHM; keySizeInBytes: PGPSIZE;
                                      var outRef: pPGPSymmetricCipherContext): PGPERROR;
const
  fn: function(context: pPGPContext; algorithm: PGPCIPHERALGORITHM; keySizeInBytes: PGPSIZE; var outRef: pPGPSymmetricCipherContext): PGPERROR cdecl = nil;
  fn8: function(context: pPGPContext; algorithm: PGPCIPHERALGORITHM; var outRef: pPGPSymmetricCipherContext): PGPERROR cdecl = nil;
begin
  result := kPGPError_FirstError;

  if IsSdk171 then
    begin
      if not Assigned(fn) then
        @fn := GetProcAddress(DLLHandle, 'PGPNewSymmetricCipherContext');
      if Assigned(fn) then
        result := fn(pMemoryMgr, algorithm, keySizeInBytes, outRef)
    end
  else
    begin
      if not Assigned(fn) then
        @fn8 := GetProcAddress(DLLHandle, 'PGPNewSymmetricCipherContext');
      if Assigned(fn8) then
        result := fn8(pMemoryMgr, algorithm, outRef)
    end
end; // PGPNewSymmetricCipherContext

function PGPFreeSymmetricCipherContext(ref: pPGPSymmetricCipherContext): PGPERROR;
const
  fn: function(ref: pPGPSymmetricCipherContext): PGPERROR cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPFreeSymmetricCipherContext');
  if Assigned(fn) then
    result := fn(ref)
  else
    result := kPGPError_FirstError;
end; // PGPFreeSymmetricCipherContext


function PGPInitSymmetricCipher(ref: pPGPSymmetricCipherContext; key: Pointer): PGPERROR;
const
  fn: function(ref: pPGPSymmetricCipherContext; key: Pointer): PGPERROR cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPInitSymmetricCipher');
  if Assigned(fn) then
    result := fn(ref, key)
  else
    result := kPGPError_FirstError;
end;

function PGPSymmetricCipherEncrypt(ref: pPGPSymmetricCipherContext;
                                   datain, dataout: Pointer): PGPERROR;
const
  fn: function(ref: pPGPSymmetricCipherContext; datain, dataout: Pointer): PGPERROR cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPSymmetricCipherEncrypt');
  if Assigned(fn) then
    result := fn(ref, datain, dataout)
  else
    result := kPGPError_FirstError;
end;

function PGPSymmetricCipherDecrypt(ref: pPGPSymmetricCipherContext;
                                   datain, dataout: Pointer): PGPERROR;
const
  fn: function(ref: pPGPSymmetricCipherContext; datain, dataout: Pointer): PGPERROR; cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPSymmetricCipherDecrypt');
  if Assigned(fn) then
    result := fn(ref, datain, dataout)
  else
    result := kPGPError_FirstError;
end;


function PGPGetSymmetricCipherSizes(ref: pPGPSymmetricCipherContext;
                                         var keySize, blockSize: PGPSIZE): PGPERROR;
const
  fn: function(ref: pPGPSymmetricCipherContext; var keySize, blockSize: PGPSIZE): PGPERROR cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPGetSymmetricCipherSizes');
  if Assigned(fn) then
    result := fn(ref, keySize, blockSize)
  else
    result := kPGPError_FirstError;
end; // PGPGetSymmetricCipherSizes


function PGPNewCFBContext(ref: pPGPSymmetricCipherContext; interleaveFactor: PGPUInt16; var outRef: pPGPCFBContext): PGPERROR;
const
  fn: function(ref: pPGPSymmetricCipherContext; interleaveFactor: PGPUInt16; var outRef: pPGPCBCContext): PGPERROR cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPNewCFBContext');
  if Assigned(fn) then
    result := fn(ref, interleaveFactor, outRef)
  else
    result := kPGPError_FirstError;
end; // PGPNewCFBContext

function PGPFreeCFBContext(ref: pPGPCFBContext): PGPERROR;
const
  fn: function(ref: pPGPCBCContext): PGPERROR cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPFreeCFBContext');
  if Assigned(fn) then
    result := fn(ref)
  else
    result := kPGPError_FirstError;
end; // PGPFreeCFBContext

function PGPInitCFB(ref: pPGPCFBContext; key, initializationVector: Pointer): PGPERROR;
const
  fn: function(ref: pPGPCFBContext; key, initializationVector: Pointer): PGPERROR cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPInitCFB');
  if Assigned(fn) then
    result := fn(ref, key, initializationVector)
  else
    result := kPGPError_FirstError;
end; // PGPInitCFB

function PGPCFBEncrypt(ref: pPGPCFBContext; datain: Pointer; bytesIn: PGPSIZE; dataout: Pointer): PGPERROR;
const
  fn: function(ref: pPGPCBCContext; datain: Pointer; bytesIn: PGPSIZE; dataout: Pointer): PGPERROR cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPCFBEncrypt');
  if Assigned(fn) then
    result := fn(ref, datain, bytesIn, dataout)
  else
    result := kPGPError_FirstError;
end; // PGPCFBEncrypt

function PGPCFBDecrypt(ref: pPGPCFBContext; datain: Pointer; bytesIn: PGPSIZE; dataout: Pointer): PGPERROR;
const
  fn: function(ref: pPGPCBCContext; datain: Pointer; bytesIn: PGPSIZE; dataout: Pointer): PGPERROR cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPCFBDecrypt');
  if Assigned(fn) then
    result := fn(ref, datain, bytesIn, dataout)
  else
    result := kPGPError_FirstError;
end; // PGPCFBDecrypt



function PGPNewCBCContext(ref: pPGPSymmetricCipherContext; var outRef: pPGPCBCContext): PGPERROR;
const
  fn: function(ref: pPGPSymmetricCipherContext; var outRef: pPGPCBCContext): PGPERROR cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPNewCBCContext');
  if Assigned(fn) then
    result := fn(ref, outRef)
  else
    result := kPGPError_FirstError;
end; // PGPNewCBCContext

function PGPFreeCBCContext(ref: pPGPCBCContext): PGPERROR;
const
  fn: function(ref: pPGPCBCContext): PGPERROR cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPFreeCBCContext');
  if Assigned(fn) then
    result := fn(ref)
  else
    result := kPGPError_FirstError;
end; // PGPFreeCBCContext

function PGPCopyCBCContext(ref: pPGPCBCContext; var outRef: pPGPCBCContext): PGPERROR;
const
  fn: function(ref: pPGPCBCContext; var outRef: pPGPCBCContext): PGPERROR cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPCopyCBCContext');
  if Assigned(fn) then
    result := fn(ref, outRef)
  else
    result := kPGPError_FirstError;
end; // PGPCopyCBCContext

function PGPInitCBC(ref: pPGPCBCContext; key, initializationVector: Pointer): PGPERROR;
const
  fn: function(ref: pPGPCBCContext; key, initializationVector: Pointer): PGPERROR cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPInitCBC');
  if Assigned(fn) then
    result := fn(ref, key, initializationVector)
  else
    result := kPGPError_FirstError;
end; // PGPInitCBC

function PGPCBCEncrypt(ref: pPGPCBCContext; datain: Pointer; bytesIn: PGPSIZE; dataout: Pointer): PGPERROR;
const
  fn: function(ref: pPGPCBCContext; datain: Pointer; bytesIn: PGPSIZE; dataout: Pointer): PGPERROR cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPCBCEncrypt');
  if Assigned(fn) then
    result := fn(ref, datain, bytesIn, dataout)
  else
    result := kPGPError_FirstError;
end; // PGPCBCEncrypt

function PGPCBCDecrypt(ref: pPGPCBCContext; datain: Pointer; bytesIn: PGPSIZE; dataout: Pointer): PGPERROR;
const
  fn: function(ref: pPGPCBCContext; datain: Pointer; bytesIn: PGPSIZE; dataout: Pointer): PGPERROR cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPCBCDecrypt');
  if Assigned(fn) then
    result := fn(ref, datain, bytesIn, dataout)
  else
    result := kPGPError_FirstError;
end; // PGPCBCDecrypt

function PGPCBCGetSymmetricCipher(ref: pPGPCBCContext; var outRef: pPGPSymmetricCipherContext): PGPERROR;
const
  fn: function(ref: pPGPCBCContext; var outRef: pPGPSymmetricCipherContext): PGPERROR cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPCBCGetSymmetricCipher');
  if Assigned(fn) then
    result := fn(ref, outRef)
  else
    result := kPGPError_FirstError;
end; // PGPCBCGetSymmetricCipher

//------------- HASH FUNCTIONs ---------------

function PGPNewHashContext(pMemoryMgr: pPGPMemoryMgr; algID: PGPHashAlgorithm; var outRef: pPGPHashContext): PGPERROR;
const
  fn: function(pMemoryMgr: pPGPMemoryMgr; algID: PGPHashAlgorithm; var outRef: pPGPHashContext): PGPERROR cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPNewHashContext');
  if Assigned(fn) then
    result := fn(pMemoryMgr, algID, outRef)
  else
    result := kPGPError_FirstError;
end; //

function PGPFreeHashContext(ref: pPGPHashContext): PGPERROR;
const
  fn: function(ref: pPGPHashContext): PGPERROR cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPFreeHashContext');
  if Assigned(fn) then
    result := fn(ref)
  else
    result := kPGPError_FirstError;
end; //

function PGPResetHash(ref: pPGPHashContext): PGPERROR;
const
  fn: function(ref: pPGPHashContext): PGPERROR cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPResetHash');
  if Assigned(fn) then
    result := fn(ref)
  else
    result := kPGPError_FirstError;
end; //

function PGPContinueHash(ref: pPGPHashContext; datain: Pointer; numBytes: PGPSIZE): PGPERROR;
const
  fn: function(ref: pPGPHashContext; datain: Pointer; numBytes: PGPSIZE): PGPERROR cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPContinueHash');
  if Assigned(fn) then
    result := fn(ref, datain, numBytes)
  else
    result := kPGPError_FirstError;
end; //

function PGPFinalizeHash(ref: pPGPHashContext; hashOut: Pointer): PGPERROR;
const
  fn: function(ref: pPGPHashContext; hashOut: Pointer): PGPERROR cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPFinalizeHash');
  if Assigned(fn) then
    result := fn(ref, hashOut)
  else
    result := kPGPError_FirstError;
end; //

function PGPGetHashSize(ref: pPGPHashContext; var hashSize: PGPSIZE): PGPERROR;
const
  fn: function(ref: pPGPHashContext; var hashSize: PGPSIZE): PGPERROR cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPGetHashSize');
  if Assigned(fn) then
    result := fn(ref, hashSize)
  else
    result := kPGPError_FirstError;
end; //


{$O-}

var fn_PGPEncode: Pointer = nil;

function PGPEncode(context: pPGPContext; Args: array of pPGPOptionsList): PGPError;
var
  i: integer;
  x: pointer;
begin
  if not Assigned(fn_PGPEncode) then
    fn_PGPEncode := GetProcAddress(DLLHandle, 'PGPEncode');

  if not Assigned(fn_PGPEncode) then
  begin
    result := kPGPError_FirstError;
    Exit;
  end;

  x := pointer(-1);
  asm
    push x;
  end;

  for i := high(args) downto low(args) do
  begin
    x := Args[i];
    asm
      push x;
    end;
  end;

  asm
    push context;
    call fn_PGPEncode;
    mov result,eax;
    pop context;
  end;

  for i := high(args) + 1 downto low(args) do
  asm
    pop x;
  end;

  //asm pop x; { -1} end;
end; // PGPEncode


var fn_PGPDecode: Pointer = nil;

function PGPDecode(context: pPGPContext; Args: array of pPGPOptionsList): PGPError;
var
  i: integer;
  x: pointer;
begin
  if not Assigned(fn_PGPDecode) then
    fn_PGPDecode := GetProcAddress(DLLHandle, 'PGPDecode');

  if not Assigned(fn_PGPDecode) then
  begin
    result := kPGPError_FirstError;
    Exit;
  end;

  x := pointer(-1);
  asm
    push x;
  end;

  for i := high(args) downto low(args) do
  begin
    x := Args[i];
    asm
      push x;
    end;
  end;

  asm
    push context;
    call fn_PGPDecode;
    mov result,eax;
    pop context;
  end;

  for i := high(args) + 1 downto low(args) do
  asm
    pop x;
  end;

  //asm pop x; { -1} end;
end; // PGPDecode

function PGPGenerateKey(Context: pPGPContext; var Key: pPGPKey; Options: pPGPOptionsList): PGPError;
const
  fn: function (context: pPGPContext; var key: pPGPKey;
                Options, LastOption: pPGPOptionsList): PGPError; cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPGenerateKey');

  if Assigned(fn) then
    result := fn(Context, Key, Options, PGPOLastOption(context))
  else
    result := kPGPError_FirstError;
end; // PGPGenerateKey

function PGPGenerateSubKey(Context: pPGPContext; var Subkey: pPGPKey; Options: pPGPOptionsList): PGPError;
const
  fn: function (context: pPGPContext; var key: pPGPKey;
                Options, LastOption: pPGPOptionsList): PGPError; cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPGenerateSubKey');

  if Assigned(fn) then
    result := fn(Context, Subkey, Options, PGPOLastOption(context))
  else
    result := kPGPError_FirstError;
end; // PGPGenerateSubKey

(*
var fn_PGPGenerateKey: Pointer = nil;

function PGPGenerateKey(context: pPGPContext; var key: pPGPKey;
                        Args: array of pPGPOptionsList): PGPError;
var
  i: integer;
  x: pointer;
begin
  if not Assigned(fn_PGPGenerateKey) then
    fn_PGPGenerateKey := GetProcAddress(DLLHandle, 'PGPGenerateKey');

  if not Assigned(fn_PGPGenerateKey) then
  begin
    result := kPGPError_FirstError;
    Exit;
  end;

  x := pointer(-1);
  asm
    push x;
  end;

  for i := high(args) downto low(args) do
  begin
    x := Args[i];
    asm
      push x;
    end;
  end;

  asm
    push key;
    push context;
    call fn_PGPGenerateKey;
    mov result,eax;
    pop context;
    pop key;
  end;

  for i := high(args) + 1 downto low(args) do
  asm
    pop x;
  end;

  {asm
    pop x; // -1
  end;}
end; // PGPGenerateKey


var fn_PGPGenerateSubKey: Pointer = nil;

function PGPGenerateSubKey(context: pPGPContext; var subkey: pPGPSubkey;
  Args: array of pPGPOptionsList): PGPError;
var
  i: integer;
  x: pointer;
begin
  fn_PGPGenerateSubKey := GetProcAddress(DLLHandle, 'PGPGenerateSubKey');
  if not Assigned(fn_PGPGenerateSubKey) then
  begin
    result := kPGPError_FirstError;
    Exit;
  end;

  x := pointer(-1);
  asm
    push x;
  end;

  for i := high(args) downto low(args) do
  begin
    x := Args[i];
    asm
      push x;
    end;
  end;

  asm
    push subkey;
    push context;
    call fn_PGPGenerateSubKey;
    mov result,eax;
    pop context;
    pop subkey;
  end;

  for i := high(args) + 1 downto low(args) do
  asm
    pop x;
  end;

  {asm
    pop x; // -1
  end;}
end; // PGPGenerateSubKey
*)

var fn_: Pointer = nil;

function PGPGetKeyEntropyNeeded(context: pPGPContext; Args: array of pPGPOptionsList): PGPError;
var
  i: integer;
  x: pointer;
  FuncPtr: Pointer;
begin
  FuncPtr := GetProcAddress(DLLHandle, 'PGPGetKeyEntropyNeeded');
  if not Assigned(FuncPtr) then
  begin
    result := kPGPError_FirstError;
    Exit;
  end;

  x := pointer(-1);
  asm
    push x;
  end;

  for i := high(args) downto low(args) do
  begin
    x := Args[i];
    asm
      push x;
    end;
  end;

  asm
    push context;
    call FuncPtr;
    mov result,eax;
    pop context;
  end;

  for i := high(args) + 1 downto low(args) do
  asm
    pop x;
  end;

  //asm pop x; { -1} end;
end; // PGPGetKeyEntropyNeeded


var fn_PGPExportKeySet: Pointer = nil;

function PGPExportKeySet(keys: pPGPKeySet; Args: array of pPGPOptionsList): PGPError;
var
  i: integer;
  x: pointer;
begin
  if not Assigned(fn_PGPExportKeySet) then
    fn_PGPExportKeySet := GetProcAddress(DLLHandle, 'PGPExportKeySet');

  if not Assigned(fn_PGPExportKeySet) then
  begin
    result := kPGPError_FirstError;
    Exit;
  end;

  x := pointer(-1);
  asm
    push x;
  end;

  for i := high(args) downto low(args) do
  begin
    x := Args[i];
    asm
      push x;
    end;
  end;

  asm
    push keys;
    call fn_PGPExportKeySet;
    mov result,eax;
    pop keys;
  end;

  for i := high(args) + 1 downto low(args) do
  asm
    pop x;
  end;

  //asm pop x; { -1 } end;
end; // PGPExportKeySet


var fn_PGPImportKeySet: Pointer = nil;

function PGPImportKeySet(context: pPGPContext; var keys: pPGPKeyset;
  Args: array of pPGPOptionsList): PGPError;
var
  i: integer;
  x: pointer;
begin
  if not Assigned(fn_PGPImportKeySet) then
    fn_PGPImportKeySet := GetProcAddress(DLLHandle, 'PGPImportKeySet');

  if not Assigned(fn_PGPImportKeySet) then
  begin
    result := kPGPError_FirstError;
    Exit;
  end;

  x := pointer(-1);
  asm
    push x;
  end;

  for i := high(args) downto low(args) do
  begin
    x := Args[i];
    asm
      push x;
    end;
  end;

  asm
    push keys;
    push context;
    call fn_PGPImportKeySet;
    mov result,eax;
    pop context;
    pop keys;
  end;

  for i := high(args)+1 downto low(args) do
  asm
    pop x;
  end;

  //asm pop x; { -1 } end;
end; // PGPImportKeySet


var fn_PGPSignUserID: Pointer = nil;

function PGPSignUserID(userID: pPGPUserID; certifyingKey: pPGPKey;
                       Args: array of pPGPOptionsList):PGPError;
var
  i: integer;
  x: pointer;
begin
  if not Assigned(fn_PGPSignUserID) then
    fn_PGPSignUserID := GetProcAddress(DLLHandle, 'PGPSignUserID');
    
  if not Assigned(fn_PGPSignUserID) then
  begin
    result := kPGPError_FirstError;
    Exit;
  end;

  x := pointer(-1);
  asm
    push x;
  end;

  for i := high(args) downto low(args) do
  begin
    x := Args[i];
    asm
      push x;
    end;
  end;

  asm
    push CertifyingKey;
    push userID;
    call fn_PGPSignUserID;
    mov result,eax;
    pop userID;
    pop CertifyingKey;
  end;

  for i := high(args) + 1 downto low(args) do
  begin
    asm
      pop x;
    end;
  end;

  //asm pop x; { -1 } end;
end;


var fn_PGPAddUserID: Pointer = nil;

function PGPAddUserID(key: pPGPKey; const userID: PChar;
                      Args: array of pPGPOptionsList): PGPError;
var
  i: integer;
  x: pointer;
begin
  if not Assigned(fn_PGPAddUserID) then
    fn_PGPAddUserID := GetProcAddress(DLLHandle, 'PGPAddUserID');
    
  if not Assigned(fn_PGPAddUserID) then
  begin
    result := kPGPError_FirstError;
    Exit;
  end;

  x := pointer(-1);
  asm
    push x;
  end;

  for i := high(args) downto low(args) do
  begin
    x := Args[i];
    asm
      push x;
    end;
  end;

  asm
    push userID;
    push key;
    call fn_PGPAddUserID;
    mov result,eax;
    pop key;
    pop userId;
  end;

  for i := high(args) + 1 downto low(args) do
  asm
    pop x;
  end;

  //asm pop x; { -1 } end;
end;

var fn_PGPRevokeSig: Pointer = nil;

function PGPRevokeSig(cert: pPGPSig;
                      allkeys: pPGPKeySet;
                      Args: array of pPGPOptionsList): PGPError;
var
  i: integer;
  x: pointer;
begin
  if not Assigned(fn_PGPRevokeSig) then
    fn_PGPRevokeSig := GetProcAddress(DLLHandle, 'PGPRevokeSig');

  if not Assigned(fn_PGPRevokeSig) then
  begin
    result := kPGPError_FirstError;
    Exit;
  end;

  x := pointer(-1);
  asm
    push x;
  end;

  for i := high(args) downto low(args) do
  begin
    x := Args[i];
    asm
      push x;
    end;
  end;

  asm
    push allkeys;
    push cert;
    call fn_PGPRevokeSig;
    mov result,eax;
    pop cert;
    pop allkeys;
  end;

  for i := high(args) + 1 downto low(args) do
  asm
    pop x;
  end;

  //asm pop x; { -1} end;
end;


var fn_PGPRevokeKey: Pointer = nil;

function PGPRevokeKey(key: pPGPKey; Args: array of pPGPOptionsList): PGPError;
var
  i: integer;
  x: pointer;
begin
  if not Assigned(fn_PGPRevokeKey) then
    fn_PGPRevokeKey := GetProcAddress(DLLHandle, 'PGPRevokeKey');

  if not Assigned(fn_PGPRevokeKey) then
  begin
    result := kPGPError_FirstError;
    Exit;
  end;

  x := pointer(-1);
  asm
    push x;
  end;

  for i := high(args) downto low(args) do
  begin
    x := Args[i];
    asm
      push x;
    end;
  end;

  asm
    push key;
    call fn_PGPRevokeKey;
    mov result,eax;
    pop key;
  end;

  for i := high(args) + 1 downto low(args) do
  asm
    pop x;
  end;

  //asm pop x; { -1} end;
end;

var fn_PGPRevokeSubKey: Pointer = nil;

function PGPRevokeSubKey(subkey: pPGPSubKey; Args: array of pPGPOptionsList): PGPError;
var
  i: integer;
  x: pointer;
begin
  if not Assigned(fn_PGPRevokeSubKey) then
    fn_PGPRevokeSubKey := GetProcAddress(DLLHandle, 'PGPRevokeSubKey');

  if not Assigned(fn_PGPRevokeSubKey) then
  begin
    result := kPGPError_FirstError;
    Exit;
  end;

  x := pointer(-1);
  asm
    push x;
  end;

  for i := high(args) downto low(args) do
  begin
    x := Args[i];
    asm
      push x;
    end;
  end;

  asm
    push subkey;
    call fn_PGPRevokeSubKey;
    mov result,eax;
    pop subkey;
  end;

  for i := high(args) + 1 downto low(args) do
  asm
    pop x;
  end;

  //asm pop x; { -1} end;
end; // PGPRevokeSubKey

var fn_PGPAppendOptionList: Pointer = nil;

function PGPAppendOptionList(context: pPGPContext;
                             optionList: pPGPOptionsList;
                             Args: array of pPGPOptionsList): PGPError;
var
  i: integer;
  x: pointer;
begin
  if not Assigned(fn_PGPAppendOptionList) then
    fn_PGPAppendOptionList := GetProcAddress(DLLHandle, 'PGPAppendOptionList');

  if not Assigned(fn_PGPAppendOptionList) then
  begin
    result := kPGPError_FirstError;
    Exit;
  end;

  x := pointer(-1);
  asm
    push x;
  end;

  for i := high(Args) downto low(Args) do
  begin
    x := Args[i];
    asm
      push x;
    end;
  end;

  asm
    push optionList;
    call fn_PGPAppendOptionList;
    mov result, eax;
    pop optionList;
  end;

  for i := high(Args) + 1 downto low(Args) do
  asm
    pop x;
  end;

  //asm pop x; { -1 } end;
end; // PGPAppendOptionList


var fn_PGPBuildOptionList: Pointer = nil;

function PGPBuildOptionList(context: pPGPContext; var outList: pPGPOptionsList;
                            Args: array of pPGPOptionsList): PGPError;
var
  i: integer;
  x: pointer;
begin
  outList := nil;
  
  if not Assigned(fn_PGPBuildOptionList) then
    fn_PGPBuildOptionList := GetProcAddress(DLLHandle, 'PGPBuildOptionList');

  if not Assigned(fn_PGPBuildOptionList) then
  begin
    result := kPGPError_FirstError;
    Exit;
  end;

  x := pointer(-1);
  asm push x; end;

  for i := high(args) downto low(args) do
  begin
    x := Args[i];
    asm push x; end;
  end;

  asm
    push outlist;
    push context;
    call fn_PGPBuildOptionList;
    mov result, eax;
    pop context;
    pop outlist;
  end;

  for i := high(args) + 1 downto low(args) do
    asm pop x; end;

  //asm pop x; { -1 } end;
end; // PGPBuildOptionList


var fn_PGPOSignWithKey: Pointer = nil;

function PGPOSignWithKey(context: pPGPContext; keyRef: pPGPKey;
  Args: array of pPGPOptionsList): pPGPOptionsList; cdecl;
var
  i: integer;
  x: pointer;
begin
  if not Assigned(fn_PGPOSignWithKey) then
    fn_PGPOSignWithKey := GetProcAddress(DLLHandle, 'PGPOSignWithKey');
    
  if not Assigned(fn_PGPOSignWithKey) then
  begin
    result := nil;
    Exit;
  end;

  x := pointer(-1);
  asm
    push x;
  end;

  for i := high(Args) downto low(Args) do
  begin
    x := Args[i];
    asm
      push x;
    end;
  end;

  asm
    push keyRef;
    push context;
    call fn_PGPOSignWithKey;
    mov result,eax;
    pop context;
    pop keyRef;
  end;

  for i := high(Args) + 1 downto low(Args) do
  asm
    pop x;
  end;

  //asm pop x; { -1} end;
end; // PGPOSignWithKey

var fn_PGPOConventionalEncrypt: Pointer = nil;

function PGPOConventionalEncrypt(context: pPGPContext;
                        Args: array of pPGPOptionsList): pPGPOptionsList; cdecl;
var
  i: integer;
  x: pointer;
begin
  if not Assigned(fn_PGPOConventionalEncrypt) then
    fn_PGPOConventionalEncrypt := GetProcAddress(DLLHandle, 'PGPOConventionalEncrypt');

  if not Assigned(fn_PGPOConventionalEncrypt) then
  begin
    result := nil;
    Exit;
  end;

  x := pointer(-1);
  asm
    push x;
  end;

  for i := high(Args) downto low(Args) do
  begin
    x := Args[i];
    asm
      push x;
    end;
  end;

  asm
    push context;
    call fn_PGPOConventionalEncrypt;
    mov result,eax;
    pop context;
  end;

  for i := high(args) + 1 downto low(args) do
  asm
    pop x;
  end;

  //asm pop x; { -1} end;
end; // PGPOConventionalEncrypt

var fn_PGPODetachedSig: Pointer = nil;

function PGPODetachedSig(context: pPGPContext;
                         Args: array of pPGPOptionsList): pPGPOptionsList;
var
  i: integer;
  x: pointer;
begin
  if not Assigned(fn_PGPODetachedSig) then
    fn_PGPODetachedSig := GetProcAddress(DLLHandle, 'PGPODetachedSig');

  if not Assigned(fn_PGPODetachedSig) then
  begin
    result := nil;
    Exit;
  end;

  x := pointer(-1);
  asm
    push x;
  end;

  for i := high(args) downto low(args) do
  begin
    x := Args[i];
    asm
      push x;
    end;
  end;

  asm
    push context;
    call fn_PGPODetachedSig;
    mov result,eax;
    pop context;
  end;

  for i := high(args) + 1 downto low(args) do
  asm
    pop x;
  end;

  //asm pop x; { -1} end;
end; // PGPODetachedSig


function PGPOCommentString(context: pPGPContext; comment: PChar): pPGPOptionsList; cdecl;
const
  fn: function(context: pPGPContext; comment: PChar): pPGPOptionsList cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPOCommentString');
  if Assigned(fn) then
    result := fn(context, comment)
  else
    result := nil;
end; // PGPOCommentString

function PGPOVersionString(context: pPGPContext; version: PChar): pPGPOptionsList;
const
  fn: function(context: pPGPContext; version: PChar): pPGPOptionsList cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPOVersionString');
  if Assigned(fn) then
    result := fn(context, version)
  else
    result := nil;
end; // PGPOVersionString


function PGPOExportKeySet(Context: pPGPContext; KeySet: pPGPKeySet): pPGPOptionsList;
const
  fn: function(Context: pPGPContext; KeySet: pPGPKeySet): pPGPOptionsList; cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPOExportKeySet');
  if Assigned(fn) then
    result := fn(Context, KeySet)
  else
    result := nil;
end; // PGPOExportKeySet


function PGPOExportPrivateKeys(context: pPGPContext; exportPrivate: PGPBOOLEAN): pPGPOptionsList cdecl;
const
  fn: function(context: pPGPContext; exportPrivate: PGPBOOLEAN): pPGPOptionsList cdecl = nil;
begin
  if not Assigned(fn) then
    @fn := GetProcAddress(DLLHandle, 'PGPOExportPrivateKeys');
  if Assigned(fn) then
    result := fn(context, exportPrivate)
  else
    result := nil;
end; // PGPOExportPrivateKeys


var fn_PGPAddJobOptions: Pointer = nil;

function PGPAddJobOptions(job: pPGPJob; Args: array of pPGPOptionsList): PGPError;
var
  i: integer;
  x: pointer;
begin
  if not Assigned(fn_PGPAddJobOptions) then
    fn_PGPAddJobOptions := GetProcAddress(DLLHandle, 'PGPAddJobOptions');

  if not Assigned(fn_PGPAddJobOptions) then
  begin
    result := kPGPError_FirstError;
    Exit;
  end;

  x := pointer(-1);
  asm
    push x;
  end;

  for i := high(args) downto low(args) do
  begin
    x := Args[i];
    asm
      push x;
    end;
  end;

  asm
    push job;
    call fn_PGPAddJobOptions;
    mov result,eax;
    pop job;
  end;

  for i := high(args) + 1 downto low(args) do
  asm
    pop x;
  end;

  //asm pop x; { -1} end;
end; // PGPAddJobOptions


{initialization
finalization}
end.


